@@ -1,29 +1,20 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2010 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2017 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation and included
-   in the file LICENSE.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   This notice must be preserved when any source code is
+   conveyed and/or propagated.
 
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  * bacula.h -- main header file to include in all Bacula source
@@ -47,8 +38,9 @@
 
 #if defined(HAVE_WIN32)
 #if defined(HAVE_MINGW)
-#include "mingwconfig.h"
+#include "winhdrs.h"
 #else
+#error "Only MINGW is supported"
 #include "winconfig.h"
 #endif
 #else
@@ -63,16 +55,18 @@
 
 
 /* System includes */
-#if HAVE_STDINT_H
+#if defined(HAVE_STDINT_H)
 #ifndef __sgi
 #include <stdint.h>
 #endif
+#elif defined(HAVE_INTTYPES_H)
+#include  <inttypes.h>
 #endif
-#if HAVE_STDARG_H
+#if defined(HAVE_STDARG_H)
 #include <stdarg.h>
 #endif
 #include <stdio.h>
-#if HAVE_STDLIB_H
+#if defined(HAVE_STDLIB_H)
 #include <stdlib.h>
 #endif
 #if HAVE_UNISTD_H
@@ -163,8 +157,8 @@ extern "C" {
  *  includes before these.
  */
 #if defined(HAVE_WIN32)
-#include <windows.h>
-#include "win32/compat/compat.h"
+//#include <windows.h>
+#include "compat.h"
 #endif
 
 #include "version.h"
@@ -174,6 +168,15 @@ extern "C" {
 #include "baconfig.h"
 #include "lib/lib.h"
 
+/* manually enable feature that you want to test in DEVELOPER mode*/
+#ifdef DEVELOPER
+#endif
+
+#ifdef DDE_EXTRA_CHECKS
+const bool have_dde_extra_check = true;
+#else
+const bool have_dde_extra_check = false;
+#endif
 /*
  * For wx-console compiles, we undo some Bacula defines.
  *  This prevents conflicts between wx-Widgets and Bacula.
@@ -188,12 +191,7 @@ extern "C" {
 #undef malloc
 #endif
 
-#if defined(HAVE_WIN32)
-#include "win32/winapi.h"
-#include "winhost.h"
-#else
 #include "host.h"
-#endif
 
 #ifndef HAVE_ZLIB_H
 #undef HAVE_LIBZ                      /* no good without headers */
