@@ -1,17 +1,20 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2003-2014 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2017 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from many
-   others, a complete list can be found in the file AUTHORS.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
    You may use this file and others of this release according to the
    license defined in the LICENSE file, which includes the Affero General
    Public License, v3.0 ("AGPLv3") and some additional permissions and
    terms pursuant to its AGPLv3 Section 7.
 
-   Bacula® is a registered trademark of Kern Sibbald.
+   This notice must be preserved when any source code is
+   conveyed and/or propagated.
+
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  * Bacula job queue routines.
@@ -150,7 +153,7 @@ struct wait_pkt {
  *  this routine is only used for jobs started from the console
  *  for which the user explicitly specified a start time. Otherwise
  *  most jobs are put into the job queue only when their
- *  scheduled time arives.
+ *  scheduled time arrives.
  */
 extern "C"
 void *sched_wait(void *arg)
@@ -188,6 +191,32 @@ void *sched_wait(void *arg)
    return NULL;
 }
 
+/* Procedure to update the client->NumConcurrentJobs */
+static void update_client_numconcurrentjobs(JCR *jcr, int val)
+{
+   int num;
+   if (!jcr->client) {
+      return;
+   }
+
+   switch (jcr->getJobType())
+   {
+   case JT_MIGRATE:
+   case JT_COPY:
+   case JT_ADMIN:
+      break;
+   case JT_BACKUP:
+   /* Fall through wanted */
+   default:
+      if (jcr->no_client_used() || jcr->wasVirtualFull) {
+         break;
+      }
+      num = jcr->client->getNumConcurrentJobs();
+      jcr->client->setNumConcurrentJobs(num + val);
+      break;
+   }
+}
+
 /*
  *  Add a job to the queue
  *    jq is a queue that was created with jobq_init
@@ -442,9 +471,6 @@ void *jobq_server(void *arg)
          remove_jcr_from_tsd(je->jcr);
          je->jcr->set_killable(false);
 
-         /* Clear the threadid, probably not necessary */
-         memset(&jcr->my_thread_id, 0, sizeof(jcr->my_thread_id));
-
          Dmsg2(2300, "Back from user engine jobid=%d use=%d.\n", jcr->JobId,
             jcr->use_count());
 
@@ -458,10 +484,12 @@ void *jobq_server(void *arg)
           *  put into the ready queue.
           */
          if (jcr->acquired_resource_locks) {
+            int num;
             dec_read_store(jcr);
             dec_write_store(jcr);
-            jcr->client->NumConcurrentJobs--;
-            jcr->job->NumConcurrentJobs--;
+            update_client_numconcurrentjobs(jcr, -1);
+            num = jcr->job->getNumConcurrentJobs() - 1;
+            jcr->job->setNumConcurrentJobs(num);
             jcr->acquired_resource_locks = false;
          }
 
@@ -611,12 +639,18 @@ static bool reschedule_job(JCR *jcr, jobq_t *jq, jobq_item_t *je)
    /* Basic condition is that more reschedule times remain */
    if (jcr->job->RescheduleTimes == 0 ||
        jcr->reschedule_count < jcr->job->RescheduleTimes) {
-      resched =
+
+      /* Check for incomplete jobs */
+      if (jcr->is_incomplete()) {
+         resched = (jcr->RescheduleIncompleteJobs && jcr->is_JobType(JT_BACKUP) &&
+                    !(jcr->HasBase||jcr->is_JobLevel(L_BASE)));
+      } else {
          /* Check for failed jobs */
-         (jcr->job->RescheduleOnError &&
-          !jcr->is_JobStatus(JS_Terminated) &&
-          !jcr->is_JobStatus(JS_Canceled) &&
-          jcr->is_JobType(JT_BACKUP));
+         resched = (jcr->job->RescheduleOnError &&
+                    !jcr->is_JobStatus(JS_Terminated) &&
+                    !jcr->is_JobStatus(JS_Canceled) &&
+                    jcr->is_JobType(JT_BACKUP));
+      }
    }
    if (resched) {
        char dt[50], dt2[50];
@@ -625,6 +659,7 @@ static bool reschedule_job(JCR *jcr, jobq_t *jq, jobq_item_t *je)
         * Reschedule this job by cleaning it up, but
         *  reuse the same JobId if possible.
         */
+      jcr->rerunning = jcr->is_incomplete();   /* save incomplete status */
       time_t now = time(NULL);
       jcr->reschedule_count++;
       jcr->sched_time = now + jcr->job->RescheduleInterval;
@@ -642,8 +677,8 @@ static bool reschedule_job(JCR *jcr, jobq_t *jq, jobq_item_t *je)
       if (!allow_duplicate_job(jcr)) {
          return false;
       }
-      /* Only jobs with no output jobs can run on same JCR */
-      if (jcr->JobBytes == 0) {
+      /* Only jobs with no output or Incomplete jobs can run on same JCR */
+      if (jcr->JobBytes == 0 || jcr->rerunning) {
          Dmsg2(2300, "Requeue job=%d use=%d\n", jcr->JobId, jcr->use_count());
          V(jq->mutex);
          /*
@@ -653,9 +688,9 @@ static bool reschedule_job(JCR *jcr, jobq_t *jq, jobq_item_t *je)
          if (jcr->wasVirtualFull) {
             jcr->setJobLevel(L_VIRTUAL_FULL);
          }
-         /* 
+         /*
           * When we are using the same jcr then make sure to reset
-          *   RealEndTime back to zero.  
+          *   RealEndTime back to zero.
           */
          jcr->jr.RealEndTime = 0;
          jobq_add(jq, jcr);     /* queue the job to run again */
@@ -672,6 +707,21 @@ static bool reschedule_job(JCR *jcr, jobq_t *jq, jobq_item_t *je)
        */
       JCR *njcr = new_jcr(sizeof(JCR), dird_free_jcr);
       set_jcr_defaults(njcr, jcr->job);
+      /*
+       * Eliminate the new job_end_push, then copy the one from
+       *  the old job, and set the old one to be empty.
+       */
+      void *v;
+      lock_jobs();              /* protect ourself from reload_config() */
+      LockRes();
+      foreach_alist(v, (&jcr->job_end_push)) {
+         njcr->job_end_push.append(v);
+      }
+      jcr->job_end_push.destroy();
+      jcr->job_end_push.init(1, false);
+      UnlockRes();
+      unlock_jobs();
+
       njcr->reschedule_count = jcr->reschedule_count;
       njcr->sched_time = jcr->sched_time;
       njcr->initial_sched_time = jcr->initial_sched_time;
@@ -689,7 +739,9 @@ static bool reschedule_job(JCR *jcr, jobq_t *jq, jobq_item_t *je)
       njcr->next_pool = jcr->next_pool;
       njcr->run_next_pool_override = jcr->run_next_pool_override;
       njcr->full_pool = jcr->full_pool;
+      njcr->vfull_pool = jcr->vfull_pool;
       njcr->run_full_pool_override = jcr->run_full_pool_override;
+      njcr->run_vfull_pool_override = jcr->run_vfull_pool_override;
       njcr->inc_pool = jcr->inc_pool;
       njcr->run_inc_pool_override = jcr->run_inc_pool_override;
       njcr->diff_pool = jcr->diff_pool;
@@ -748,7 +800,7 @@ static bool acquire_resources(JCR *jcr)
    if (jcr->rstore) {
       Dmsg1(200, "Rstore=%s\n", jcr->rstore->name());
       if (!inc_read_store(jcr)) {
-         Dmsg1(200, "Fail rncj=%d\n", jcr->rstore->NumConcurrentJobs);
+         Dmsg1(200, "Fail rncj=%d\n", jcr->rstore->getNumConcurrentJobs());
          jcr->setJobStatus(JS_WaitStoreRes);
          return false;
       }
@@ -756,14 +808,15 @@ static bool acquire_resources(JCR *jcr)
 
    if (jcr->wstore) {
       Dmsg1(200, "Wstore=%s\n", jcr->wstore->name());
-      if (jcr->wstore->NumConcurrentJobs < jcr->wstore->MaxConcurrentJobs) {
-         jcr->wstore->NumConcurrentJobs++;
-         Dmsg1(200, "Inc wncj=%d\n", jcr->wstore->NumConcurrentJobs);
+      int num = jcr->wstore->getNumConcurrentJobs();
+      if (num < jcr->wstore->MaxConcurrentJobs) {
+         Dmsg1(200, "Inc wncj=%d\n", num + 1);
+         jcr->wstore->setNumConcurrentJobs(num + 1);
       } else if (jcr->rstore) {
          dec_read_store(jcr);
          skip_this_jcr = true;
       } else {
-         Dmsg1(200, "Fail wncj=%d\n", jcr->wstore->NumConcurrentJobs);
+         Dmsg1(200, "Fail wncj=%d\n", num);
          skip_this_jcr = true;
       }
    }
@@ -772,22 +825,26 @@ static bool acquire_resources(JCR *jcr)
       return false;
    }
 
-   if (jcr->client->NumConcurrentJobs < jcr->client->MaxConcurrentJobs) {
-      jcr->client->NumConcurrentJobs++;
-   } else {
-      /* Back out previous locks */
-      dec_write_store(jcr);
-      dec_read_store(jcr);
-      jcr->setJobStatus(JS_WaitClientRes);
-      return false;
+   if (jcr->client) {
+      if (jcr->client->getNumConcurrentJobs() < jcr->client->MaxConcurrentJobs) {
+         update_client_numconcurrentjobs(jcr, 1);
+      } else {
+         /* Back out previous locks */
+         dec_write_store(jcr);
+         dec_read_store(jcr);
+         jcr->setJobStatus(JS_WaitClientRes);
+         return false;
+      }
    }
-   if (jcr->job->NumConcurrentJobs < jcr->job->MaxConcurrentJobs) {
-      jcr->job->NumConcurrentJobs++;
+   if (jcr->job->getNumConcurrentJobs() < jcr->job->MaxConcurrentJobs) {
+      int num;
+      num = jcr->job->getNumConcurrentJobs() + 1;
+      jcr->job->setNumConcurrentJobs(num);
    } else {
       /* Back out previous locks */
       dec_write_store(jcr);
       dec_read_store(jcr);
-      jcr->client->NumConcurrentJobs--;
+      update_client_numconcurrentjobs(jcr, -1);
       jcr->setJobStatus(JS_WaitJobRes);
       return false;
    }
@@ -805,13 +862,17 @@ static pthread_mutex_t rstore_mutex = PTHREAD_MUTEX_INITIALIZER;
 bool inc_read_store(JCR *jcr)
 {
    P(rstore_mutex);
-   if (jcr->rstore->NumConcurrentJobs < jcr->rstore->MaxConcurrentJobs &&
+   int num = jcr->rstore->getNumConcurrentJobs();
+   int numread = jcr->rstore->getNumConcurrentReadJobs();
+   if (num < jcr->rstore->MaxConcurrentJobs &&
        (jcr->getJobType() == JT_RESTORE ||
-        jcr->rstore->MaxConcurrentReadJobs == 0 ||
-        jcr->rstore->NumConcurrentReadJobs < jcr->rstore->MaxConcurrentReadJobs)) {
-      jcr->rstore->NumConcurrentReadJobs++;
-      jcr->rstore->NumConcurrentJobs++;
-      Dmsg1(200, "Inc rncj=%d\n", jcr->rstore->NumConcurrentJobs);
+        numread == 0 ||
+        numread < jcr->rstore->MaxConcurrentReadJobs)) {
+      num++;
+      numread++;
+      jcr->rstore->setNumConcurrentReadJobs(numread);
+      jcr->rstore->setNumConcurrentJobs(num);
+      Dmsg1(200, "Inc rncj=%d\n", num);
       V(rstore_mutex);
       return true;
    }
@@ -823,20 +884,20 @@ void dec_read_store(JCR *jcr)
 {
    if (jcr->rstore) {
       P(rstore_mutex);
-      jcr->rstore->NumConcurrentReadJobs--;    /* back out rstore */
-      jcr->rstore->NumConcurrentJobs--;        /* back out rstore */
-      Dmsg1(200, "Dec rncj=%d\n", jcr->rstore->NumConcurrentJobs);
+      int numread = jcr->rstore->getNumConcurrentReadJobs() - 1;
+      int num = jcr->rstore->getNumConcurrentJobs() - 1;
+      jcr->rstore->setNumConcurrentReadJobs(numread);
+      jcr->rstore->setNumConcurrentJobs(num);
+      Dmsg1(200, "Dec rncj=%d\n", num);
       V(rstore_mutex);
-      ASSERT(jcr->rstore->NumConcurrentReadJobs >= 0);
-      ASSERT(jcr->rstore->NumConcurrentJobs >= 0);
    }
 }
 
 static void dec_write_store(JCR *jcr)
 {
    if (jcr->wstore) {
-      jcr->wstore->NumConcurrentJobs--;
-      Dmsg1(200, "Dec wncj=%d\n", jcr->wstore->NumConcurrentJobs);
-      ASSERT(jcr->wstore->NumConcurrentJobs >= 0);
+      int num = jcr->wstore->getNumConcurrentJobs() - 1;
+      Dmsg1(200, "Dec wncj=%d\n", num);
+      jcr->wstore->setNumConcurrentJobs(num);
    }
 }
