@@ -1,20 +1,22 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2004-2014 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2017 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from many
-   others, a complete list can be found in the file AUTHORS.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
    You may use this file and others of this release according to the
    license defined in the LICENSE file, which includes the Affero General
    Public License, v3.0 ("AGPLv3") and some additional permissions and
    terms pursuant to its AGPLv3 Section 7.
 
-   Bacula® is a registered trademark of Kern Sibbald.
+   This notice must be preserved when any source code is
+   conveyed and/or propagated.
+
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
- *
  *   Bacula Director -- mac.c -- responsible for doing
  *     migration and copy jobs.
  *
@@ -28,7 +30,6 @@
  *     Open connection with Storage daemon and pass him commands
  *       to do the backup.
  *     When the Storage daemon finishes the job, update the DB.
- *
  */
 
 #include "bacula.h"
@@ -179,6 +180,19 @@ bool do_mac_init(JCR *jcr)
     *   the previous backup job "prev_job".
     */
    set_jcr_defaults(wjcr, prev_job);
+   /* fix MA 987 cannot copy/migrate jobs with a Level=VF in the job resource
+    * If the prev_job level definition is VirtualFull,
+    * change it to Incremental, otherwise the writing SD would do a VF
+    */
+   if (wjcr->getJobLevel() == L_VIRTUAL_FULL) {
+      wjcr->setJobLevel(L_INCREMENTAL);
+   }
+
+   /* Don't check for duplicates on this jobs. We do it before setup_job(),
+    * because we check allow_duplicate_job() here.
+    */
+   wjcr->IgnoreDuplicateJobChecking = true;
+
    if (!setup_job(wjcr)) {
       Jmsg(jcr, M_FATAL, 0, _("setup job failed.\n"));
       return false;
@@ -197,8 +211,6 @@ bool do_mac_init(JCR *jcr)
 
    /* Don't let WatchDog checks Max*Time value on this Job */
    wjcr->no_maxtime = true;
-   /* Don't check for duplicates on this jobs */
-   wjcr->job->IgnoreDuplicateJobChecking = true;
    Dmsg4(dbglevel, "wjcr: Name=%s JobId=%d Type=%c Level=%c\n",
       wjcr->jr.Name, (int)wjcr->jr.JobId,
       wjcr->jr.JobType, wjcr->jr.JobLevel);
@@ -361,7 +373,7 @@ bool do_mac(JCR *jcr)
    /* TODO: See priority with bandwidth parameter */
    if (jcr->job->max_bandwidth > 0) {
       jcr->max_bandwidth = jcr->job->max_bandwidth;
-   } else if (jcr->client->max_bandwidth > 0) {
+   } else if (jcr->client && jcr->client->max_bandwidth > 0) {
       jcr->max_bandwidth = jcr->client->max_bandwidth;
    }
 
@@ -395,7 +407,9 @@ bool do_mac(JCR *jcr)
       goto bail_out;
    }
    sd = jcr->store_bsock;
-   jcr->sd_calls_client = jcr->client->sd_calls_client;
+   if (jcr->client) {
+      jcr->sd_calls_client = jcr->client->sd_calls_client;
+   }
 
    Dmsg2(dbglevel, "Read store=%s, write store=%s\n",
       ((STORE *)jcr->rstorage->first())->name(),
@@ -607,17 +621,21 @@ void mac_cleanup(JCR *jcr, int TermCode, int writeTermCode)
 {
    char sdt[MAX_TIME_LENGTH], edt[MAX_TIME_LENGTH];
    char ec1[30], ec2[30], ec3[30], ec4[30], ec5[30], elapsed[50];
-   char ec6[50], ec7[50], ec8[50];
-   char term_code[100], sd_term_msg[100];
-   const char *term_msg;
+   char ec6[50], ec7[50], ec8[50], ec9[30], ec10[30];
+   char sd_term_msg[100];
+   POOL_MEM term_code;
+   POOL_MEM term_msg;
    int msg_type = M_INFO;
    MEDIA_DBR mr;
    double kbps;
    utime_t RunTime;
+   bool goterrors=false;
    JCR *wjcr = jcr->wjcr;
    POOL_MEM query(PM_MESSAGE);
    POOL_MEM vol_info;
 
+   remove_dummy_jobmedia_records(jcr);
+
    Dmsg2(100, "Enter mac_cleanup %d %c\n", TermCode, TermCode);
    update_job_end(jcr, TermCode);
 
@@ -635,7 +653,7 @@ void mac_cleanup(JCR *jcr, int TermCode, int writeTermCode)
       wjcr->JobBytes = jcr->JobBytes = wjcr->SDJobBytes;
       wjcr->jr.RealEndTime = 0;
       wjcr->jr.PriorJobId = jcr->previous_jr.JobId;
-
+      wjcr->JobErrors += wjcr->SDErrors;
       update_job_end(wjcr, TermCode);
 
       /* Update final items to set them to the previous job's values */
@@ -646,13 +664,24 @@ void mac_cleanup(JCR *jcr, int TermCode, int writeTermCode)
          new_jobid);
       db_sql_query(wjcr->db, query.c_str(), NULL, NULL);
 
+      goterrors = jcr->SDErrors > 0 || jcr->JobErrors > 0 ||
+         jcr->SDJobStatus == JS_Canceled ||
+         jcr->SDJobStatus == JS_ErrorTerminated ||
+         jcr->SDJobStatus == JS_FatalError;
+
+      if (goterrors && jcr->getJobType() == JT_MIGRATE && jcr->JobStatus == JS_Terminated) {
+         Jmsg(jcr, M_WARNING, 0, _("Found errors during the migration process. "
+                                   "The original job %s will be kept in the catalog "
+                                   "and the Migration job will be marked in Error\n"), old_jobid);
+      }
+
       /*
        * If we terminated a migration normally:
        *   - mark the previous job as migrated
        *   - move any Log records to the new JobId
        *   - Purge the File records from the previous job
        */
-      if (jcr->getJobType() == JT_MIGRATE && jcr->JobStatus == JS_Terminated) {
+      if (!goterrors && jcr->getJobType() == JT_MIGRATE && jcr->JobStatus == JS_Terminated) {
          Mmsg(query, "UPDATE Job SET Type='%c' WHERE JobId=%s",
               (char)JT_MIGRATED_JOB, old_jobid);
          db_sql_query(wjcr->db, query.c_str(), NULL, NULL);
@@ -662,6 +691,11 @@ void mac_cleanup(JCR *jcr, int TermCode, int writeTermCode)
            new_jobid, old_jobid);
          db_sql_query(wjcr->db, query.c_str(), NULL, NULL);
 
+         /* Move RestoreObjects */
+         Mmsg(query, "UPDATE RestoreObject SET JobId=%s WHERE JobId=%s",
+           new_jobid, old_jobid);
+         db_sql_query(wjcr->db, query.c_str(), NULL, NULL);
+
          if (jcr->job->PurgeMigrateJob) {
             /* Purge old Job record */
             purge_jobs_from_catalog(ua, old_jobid);
@@ -678,15 +712,27 @@ void mac_cleanup(JCR *jcr, int TermCode, int writeTermCode)
        *   - copy any Log records to the new JobId
        *   - set type="Job Copy" for the new job
        */
-      if (jcr->getJobType() == JT_COPY && jcr->JobStatus == JS_Terminated) {
+      if (goterrors || (jcr->getJobType() == JT_COPY && jcr->JobStatus == JS_Terminated)) {
          /* Copy JobLog to new JobId */
          Mmsg(query, "INSERT INTO Log (JobId, Time, LogText ) "
                       "SELECT %s, Time, LogText FROM Log WHERE JobId=%s",
               new_jobid, old_jobid);
          db_sql_query(wjcr->db, query.c_str(), NULL, NULL);
+
+         /* We are in a real copy job */
          Mmsg(query, "UPDATE Job SET Type='%c' WHERE JobId=%s",
               (char)JT_JOB_COPY, new_jobid);
          db_sql_query(wjcr->db, query.c_str(), NULL, NULL);
+
+         /* Copy RestoreObjects */
+         Mmsg(query, "INSERT INTO RestoreObject (ObjectName,PluginName,RestoreObject,"
+              "ObjectLength,ObjectFullLength,ObjectIndex,ObjectType,"
+              "ObjectCompression,FileIndex,JobId) "
+        "SELECT ObjectName,PluginName,RestoreObject,"
+          "ObjectLength,ObjectFullLength,ObjectIndex,ObjectType,"
+          "ObjectCompression,FileIndex,%s FROM RestoreObject WHERE JobId=%s",
+           new_jobid, old_jobid);
+         db_sql_query(wjcr->db, query.c_str(), NULL, NULL);
       }
 
       if (!db_get_job_record(jcr, jcr->db, &jcr->jr)) {
@@ -725,76 +771,93 @@ void mac_cleanup(JCR *jcr, int TermCode, int writeTermCode)
          }
       }
 
+      /* We keep all information in the catalog because most of the work is
+       * done, and users might restore things from what we have
+       */
+      if (goterrors) {
+         jcr->setJobStatus(JS_ErrorTerminated);
+         Mmsg(query, "UPDATE Job SET JobStatus='%c' WHERE JobId=%s",
+              JS_ErrorTerminated, new_jobid);
+         db_sql_query(wjcr->db, query.c_str(), NULL, NULL);
+      }
+
       switch (jcr->JobStatus) {
       case JS_Terminated:
          if (jcr->JobErrors || jcr->SDErrors) {
-            term_msg = _("%s OK -- with warnings");
+            Mmsg(term_msg, _("%%s OK -- %s"), jcr->StatusErrMsg[0] ? jcr->StatusErrMsg : _("with warnings"));
          } else {
-            term_msg = _("%s OK");
+            Mmsg(term_msg, _("%%s OK"));
          }
          break;
       case JS_FatalError:
       case JS_ErrorTerminated:
-         term_msg = _("*** %s Error ***");
+         Mmsg(term_msg, _("*** %%s Error ***"));
          msg_type = M_ERROR;          /* Generate error message */
          if (jcr->store_bsock) {
             jcr->store_bsock->signal(BNET_TERMINATE);
-            if (jcr->SD_msg_chan) {
+            if (jcr->SD_msg_chan_started) {
                pthread_cancel(jcr->SD_msg_chan);
             }
          }
          if (wjcr->store_bsock) {
             wjcr->store_bsock->signal(BNET_TERMINATE);
-            if (wjcr->SD_msg_chan) {
+            if (wjcr->SD_msg_chan_started) {
                pthread_cancel(wjcr->SD_msg_chan);
             }
          }
          break;
       case JS_Canceled:
-         term_msg = _("%s Canceled");
+         Mmsg(term_msg, _("%%s Canceled"));
          if (jcr->store_bsock) {
             jcr->store_bsock->signal(BNET_TERMINATE);
-            if (jcr->SD_msg_chan) {
+            if (jcr->SD_msg_chan_started) {
                pthread_cancel(jcr->SD_msg_chan);
             }
          }
          if (wjcr->store_bsock) {
             wjcr->store_bsock->signal(BNET_TERMINATE);
-            if (wjcr->SD_msg_chan) {
+            if (wjcr->SD_msg_chan_started) {
                pthread_cancel(wjcr->SD_msg_chan);
             }
          }
          break;
       default:
-         term_msg = _("Inappropriate %s term code");
+         Mmsg(term_msg, _("Inappropriate %s term code"));
          break;
       }
    } else {
-      if (jcr->getJobType() == JT_MIGRATE && jcr->previous_jr.JobId != 0) {
+      if (!goterrors && jcr->getJobType() == JT_MIGRATE && jcr->previous_jr.JobId != 0) {
          /* Mark previous job as migrated */
          Mmsg(query, "UPDATE Job SET Type='%c' WHERE JobId=%s",
               (char)JT_MIGRATED_JOB, edit_uint64(jcr->previous_jr.JobId, ec1));
          db_sql_query(jcr->db, query.c_str(), NULL, NULL);
       }
-      term_msg = _("%s -- no files to %s");
+      Mmsg(term_msg, _("%%s -- no files to %%s"));
    }
 
-   bsnprintf(term_code, sizeof(term_code), term_msg, jcr->get_OperationName(), jcr->get_ActionName(0));
+   Mmsg(term_code, term_msg.c_str(), jcr->get_OperationName(), jcr->get_ActionName(0));
    bstrftimes(sdt, sizeof(sdt), jcr->jr.StartTime);
    bstrftimes(edt, sizeof(edt), jcr->jr.EndTime);
    RunTime = jcr->jr.EndTime - jcr->jr.StartTime;
    if (RunTime <= 0) {
-      kbps = 0;
-   } else {
-      kbps = (double)jcr->SDJobBytes / (1000 * RunTime);
+      RunTime = 1;
    }
+   kbps = (double)jcr->SDJobBytes / (1000.0 * (double)RunTime);
 
    jobstatus_to_ascii(jcr->SDJobStatus, sd_term_msg, sizeof(sd_term_msg));
 
    /* Edit string for last volume size */
-   Mmsg(vol_info, _("%s (%sB)"),
+   if (mr.VolABytes != 0) {
+      Mmsg(vol_info, _("meta: %s (%sB) aligned: %s (%sB)"),
+        edit_uint64_with_commas(mr.VolBytes, ec4),
+        edit_uint64_with_suffix(mr.VolBytes, ec5),
+        edit_uint64_with_commas(mr.VolABytes, ec9),
+        edit_uint64_with_suffix(mr.VolBytes, ec10));
+   } else {
+     Mmsg(vol_info, _("%s (%sB)"),
         edit_uint64_with_commas(mr.VolBytes, ec4),
         edit_uint64_with_suffix(mr.VolBytes, ec5));
+   }
 
    Jmsg(jcr, msg_type, 0, _("%s %s %s (%s):\n"
 "  Build OS:               %s %s %s\n"
@@ -856,13 +919,8 @@ void mac_cleanup(JCR *jcr, int TermCode, int writeTermCode)
         vol_info.c_str(),
         jcr->SDErrors,
         sd_term_msg,
-        term_code);
+        term_code.c_str());
 
-   Dmsg1(100, "migrate_cleanup() wjcr=0x%x\n", jcr->wjcr);
-   if (jcr->wjcr) {
-      free_jcr(jcr->wjcr);
-      jcr->wjcr = NULL;
-   }
    Dmsg0(100, "Leave migrate_cleanup()\n");
 }
 
