@@ -1,20 +1,22 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2014 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2017 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from many
-   others, a complete list can be found in the file AUTHORS.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
    You may use this file and others of this release according to the
    license defined in the LICENSE file, which includes the Affero General
    Public License, v3.0 ("AGPLv3") and some additional permissions and
    terms pursuant to its AGPLv3 Section 7.
 
-   Bacula® is a registered trademark of Kern Sibbald.
+   This notice must be preserved when any source code is
+   conveyed and/or propagated.
+
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
- *
  *   Bacula Director -- msgchan.c -- handles the message channel
  *    to the Storage daemon and the File daemon.
  *
@@ -27,7 +29,6 @@
  *      to authenticate ourself and to pass the JobId.
  *    Create a thread to interact with the Storage daemon
  *      who returns a job status and requests Catalog services, etc.
- *
  */
 
 #include "bacula.h"
@@ -53,7 +54,7 @@ static char OK_device[]  = "3000 OK use device device=%s\n";
 /* Storage Daemon requests */
 static char Job_start[]  = "3010 Job %127s start\n";
 static char Job_end[]    =
-   "3099 Job %127s end JobStatus=%d JobFiles=%d JobBytes=%lld JobErrors=%u\n";
+   "3099 Job %127s end JobStatus=%d JobFiles=%d JobBytes=%lld JobErrors=%u ErrMsg=%256s\n";
 
 /* Forward referenced functions */
 extern "C" void *msg_thread(void *arg);
@@ -120,6 +121,10 @@ bool connect_to_storage_daemon(JCR *jcr, int retry_interval,
    sd->set_source_address(director->DIRsrc_addr);
    if (!sd->connect(jcr, retry_interval, max_retry_time, heart_beat, _("Storage daemon"),
          store->address, NULL, store->SDport, verbose)) {
+
+      if (!jcr->store_bsock) {  /* The bsock was locally created, so we free it here */
+         free_bsock(sd);
+      }
       sd = NULL;
    }
 
@@ -185,7 +190,11 @@ bool start_storage_daemon_job(JCR *jcr, alist *rstore, alist *wstore, bool send_
     */
    pm_strcpy(job_name, jcr->job->name());
    bash_spaces(job_name);
-   pm_strcpy(client_name, jcr->client->name());
+   if (jcr->client) {
+      pm_strcpy(client_name, jcr->client->name());
+   } else {
+      pm_strcpy(client_name, "**Dummy**");
+   }
    bash_spaces(client_name);
    pm_strcpy(fileset_name, jcr->fileset->name());
    bash_spaces(fileset_name);
@@ -368,14 +377,14 @@ bool start_storage_daemon_message_thread(JCR *jcr)
 
    jcr->inc_use_count();              /* mark in use by msg thread */
    jcr->sd_msg_thread_done = false;
-   jcr->SD_msg_chan = 0;
+   jcr->SD_msg_chan_started = false;
    Dmsg0(150, "Start SD msg_thread.\n");
    if ((status=pthread_create(&thid, NULL, msg_thread, (void *)jcr)) != 0) {
       berrno be;
       Jmsg1(jcr, M_ABORT, 0, _("Cannot create message thread: %s\n"), be.bstrerror(status));
    }
    /* Wait for thread to start */
-   while (jcr->SD_msg_chan == 0) {
+   while (jcr->SD_msg_chan_started == false) {
       bmicrosleep(0, 50);
       if (job_canceled(jcr) || jcr->sd_msg_thread_done) {
          return false;
@@ -388,15 +397,15 @@ bool start_storage_daemon_message_thread(JCR *jcr)
 extern "C" void msg_thread_cleanup(void *arg)
 {
    JCR *jcr = (JCR *)arg;
-   db_end_transaction(jcr, jcr->db);        /* terminate any open transaction */
+   db_end_transaction(jcr, jcr->db);      /* terminate any open transaction */
    jcr->lock();
    jcr->sd_msg_thread_done = true;
-   jcr->SD_msg_chan = 0;
+   jcr->SD_msg_chan_started = false;
    jcr->unlock();
    pthread_cond_broadcast(&jcr->term_wait); /* wakeup any waiting threads */
    Dmsg2(100, "=== End msg_thread. JobId=%d usecnt=%d\n", jcr->JobId, jcr->use_count());
-   db_thread_cleanup(jcr->db);              /* remove thread specific data */
-   free_jcr(jcr);                           /* release jcr */
+   db_thread_cleanup(jcr->db);           /* remove thread specific data */
+   free_jcr(jcr);                        /* release jcr */
 }
 
 /*
@@ -411,12 +420,15 @@ extern "C" void *msg_thread(void *arg)
    int JobStatus;
    int n;
    char Job[MAX_NAME_LENGTH];
+   char ErrMsg[256];
    uint32_t JobFiles, JobErrors;
    uint64_t JobBytes;
+   ErrMsg[0] = 0;
 
    pthread_detach(pthread_self());
    set_jcr_in_tsd(jcr);
    jcr->SD_msg_chan = pthread_self();
+   jcr->SD_msg_chan_started = true;
    pthread_cleanup_push(msg_thread_cleanup, arg);
    sd = jcr->store_bsock;
 
@@ -430,11 +442,13 @@ extern "C" void *msg_thread(void *arg)
          continue;
       }
       if (sscanf(sd->msg, Job_end, Job, &JobStatus, &JobFiles,
-                 &JobBytes, &JobErrors) == 5) {
+                 &JobBytes, &JobErrors, ErrMsg) == 6) {
          jcr->SDJobStatus = JobStatus; /* termination status */
          jcr->SDJobFiles = JobFiles;
          jcr->SDJobBytes = JobBytes;
          jcr->SDErrors = JobErrors;
+         unbash_spaces(ErrMsg); /* Error message if any */
+         pm_strcpy(jcr->StatusErrMsg, ErrMsg);
          break;
       }
       Dmsg1(400, "end loop use=%d\n", jcr->use_count());
@@ -472,7 +486,7 @@ void wait_for_storage_daemon_termination(JCR *jcr)
       pthread_cond_timedwait(&jcr->term_wait, &mutex, &timeout);
       V(mutex);
       if (jcr->is_canceled()) {
-         if (jcr->SD_msg_chan) {
+         if (jcr->SD_msg_chan_started) {
             jcr->store_bsock->set_timed_out();
             jcr->store_bsock->set_terminated();
             sd_msg_thread_send_signal(jcr, TIMEOUT_SIGNAL);
@@ -502,7 +516,7 @@ bool send_bootstrap_file(JCR *jcr, BSOCK *sd)
    if (!jcr->RestoreBootstrap) {
       return true;
    }
-   bs = fopen(jcr->RestoreBootstrap, "rb");
+   bs = bfopen(jcr->RestoreBootstrap, "rb");
    if (!bs) {
       berrno be;
       Jmsg(jcr, M_FATAL, 0, _("Could not open bootstrap file %s: ERR=%s\n"),
