@@ -1,29 +1,20 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2011 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2015 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation and included
-   in the file LICENSE.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   This notice must be preserved when any source code is 
+   conveyed and/or propagated.
 
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  *
@@ -44,7 +35,7 @@
 #define DBGLVL 0
 #else
 #undef SCHED_DEBUG
-#define DBGLVL 200
+#define DBGLVL DT_SCHEDULER|200
 #endif
 
 const int dbglvl = DBGLVL;
@@ -73,7 +64,7 @@ static void dump_job(job_item *ji, const char *msg);
 
 /* Imported variables */
 
-/**
+/*
  * called by reload_config to tell us that the schedules
  * we may have based our next jobs to run queues have been
  * invalidated.  In fact the schedules may not have changed
@@ -81,7 +72,7 @@ static void dump_job(job_item *ji, const char *msg);
  * on are new and no longer have a valid last_run time which
  * causes us to double run schedules that get put into the list
  * because run_nh = 1.
- */   
+ */
 static bool schedules_invalidated = false;
 void invalidate_schedules(void) {
     schedules_invalidated = true;
@@ -154,7 +145,7 @@ again:
       time_t twait;
       /** discard scheduled queue and rebuild with new schedule objects. **/
       lock_jobs();
-      if (schedules_invalidated) { 
+      if (schedules_invalidated) {
           dump_job(next_job, "Invalidated job");
           free(next_job);
           while (!jobs_to_run->empty()) {
@@ -186,11 +177,11 @@ again:
    jcr = new_jcr(sizeof(JCR), dird_free_jcr);
    run = next_job->run;               /* pick up needed values */
    job = next_job->job;
-   if (job->enabled) {
-      dump_job(next_job, _("Run job"));
+   if (job->is_enabled() && (!job->client || job->client->is_enabled())) {
+      dump_job(next_job, _("Run job"));  /* no client and job enabled */
    }
    free(next_job);
-   if (!job->enabled) {
+   if (!job->is_enabled() || (job->client && !job->client->is_enabled())) {
       free_jcr(jcr);
       goto again;                     /* ignore this job */
    }
@@ -205,10 +196,18 @@ again:
       jcr->pool = run->pool;          /* override pool */
       jcr->run_pool_override = true;
    }
+   if (run->next_pool) {
+      jcr->next_pool = run->next_pool; /* override next pool */
+      jcr->run_next_pool_override = true;
+   }
    if (run->full_pool) {
       jcr->full_pool = run->full_pool; /* override full pool */
       jcr->run_full_pool_override = true;
    }
+   if (run->vfull_pool) {
+      jcr->vfull_pool = run->vfull_pool; /* override virtual full pool */
+      jcr->run_vfull_pool_override = true;
+   }
    if (run->inc_pool) {
       jcr->inc_pool = run->inc_pool;  /* override inc pool */
       jcr->run_inc_pool_override = true;
@@ -266,9 +265,9 @@ static void find_runs()
    JOB *job;
    SCHED *sched;
    struct tm tm;
-   int hour, mday, wday, month, wom, woy;
+   int hour, mday, wday, month, wom, woy, ldom;
    /* Items corresponding to above at the next hour */
-   int nh_hour, nh_mday, nh_wday, nh_month, nh_wom, nh_woy;
+   int nh_hour, nh_mday, nh_wday, nh_month, nh_wom, nh_woy, nh_ldom;
 
    Dmsg0(dbglvl, "enter find_runs()\n");
 
@@ -281,6 +280,7 @@ static void find_runs()
    month = tm.tm_mon;
    wom = mday / 7;
    woy = tm_woy(now);                     /* get week of year */
+   ldom = tm_ldom(month, tm.tm_year + 1900);
 
    Dmsg7(dbglvl, "now = %x: h=%d m=%d md=%d wd=%d wom=%d woy=%d\n",
          now, hour, month, mday, wday, wom, woy);
@@ -298,6 +298,7 @@ static void find_runs()
    nh_month = tm.tm_mon;
    nh_wom = nh_mday / 7;
    nh_woy = tm_woy(next_hour);              /* get week of year */
+   nh_ldom = tm_ldom(nh_month, tm.tm_year + 1900);
 
    Dmsg7(dbglvl, "nh = %x: h=%d m=%d md=%d wd=%d wom=%d woy=%d\n",
          next_hour, nh_hour, nh_month, nh_mday, nh_wday, nh_wom, nh_woy);
@@ -306,7 +307,8 @@ static void find_runs()
    LockRes();
    foreach_res(job, R_JOB) {
       sched = job->schedule;
-      if (sched == NULL || !job->enabled) { /* scheduled? or enabled? */
+      if (!sched || !job->is_enabled() || (sched && !sched->is_enabled()) ||
+         (job->client && !job->client->is_enabled())) {
          continue;                    /* no, skip this job */
       }
       Dmsg1(dbglvl, "Got job: %s\n", job->hdr.name);
@@ -317,40 +319,47 @@ static void find_runs()
           */
 #ifdef xxxx
          Dmsg0(000, "\n");
-         Dmsg6(000, "run h=%d m=%d md=%d wd=%d wom=%d woy=%d\n",
-            hour, month, mday, wday, wom, woy);
-         Dmsg6(000, "bitset bsh=%d bsm=%d bsmd=%d bswd=%d bswom=%d bswoy=%d\n",
+         Dmsg7(000, "run h=%d m=%d md=%d wd=%d wom=%d woy=%d ldom=%d\n",
+            hour, month, mday, wday, wom, woy, ldom);
+         Dmsg7(000, "bitset bsh=%d bsm=%d bsmd=%d bswd=%d bswom=%d bswoy=%d bsldom=%d\n",
             bit_is_set(hour, run->hour),
             bit_is_set(month, run->month),
             bit_is_set(mday, run->mday),
             bit_is_set(wday, run->wday),
             bit_is_set(wom, run->wom),
-            bit_is_set(woy, run->woy));
+            bit_is_set(woy, run->woy),
+            bit_is_set(31, run->mday));
 
-         Dmsg6(000, "nh_run h=%d m=%d md=%d wd=%d wom=%d woy=%d\n",
-            nh_hour, nh_month, nh_mday, nh_wday, nh_wom, nh_woy);
-         Dmsg6(000, "nh_bitset bsh=%d bsm=%d bsmd=%d bswd=%d bswom=%d bswoy=%d\n",
+
+         Dmsg7(000, "nh_run h=%d m=%d md=%d wd=%d wom=%d woy=%d ldom=%d\n",
+            nh_hour, nh_month, nh_mday, nh_wday, nh_wom, nh_woy, nh_ldom);
+         Dmsg7(000, "nh_bitset bsh=%d bsm=%d bsmd=%d bswd=%d bswom=%d bswoy=%d bsldom=%d\n",
             bit_is_set(nh_hour, run->hour),
             bit_is_set(nh_month, run->month),
             bit_is_set(nh_mday, run->mday),
             bit_is_set(nh_wday, run->wday),
             bit_is_set(nh_wom, run->wom),
-            bit_is_set(nh_woy, run->woy));
+            bit_is_set(nh_woy, run->woy),
+            bit_is_set(31, run->mday));
 #endif
 
          run_now = bit_is_set(hour, run->hour) &&
-            bit_is_set(mday, run->mday) &&
-            bit_is_set(wday, run->wday) &&
-            bit_is_set(month, run->month) &&
-            bit_is_set(wom, run->wom) &&
-            bit_is_set(woy, run->woy);
+            ((bit_is_set(mday, run->mday) &&
+              bit_is_set(wday, run->wday) &&
+              bit_is_set(month, run->month) &&
+              bit_is_set(wom, run->wom) &&
+              bit_is_set(woy, run->woy)) ||
+             (bit_is_set(month, run->month) &&
+              bit_is_set(31, run->mday) && mday == ldom));
 
          run_nh = bit_is_set(nh_hour, run->hour) &&
-            bit_is_set(nh_mday, run->mday) &&
-            bit_is_set(nh_wday, run->wday) &&
-            bit_is_set(nh_month, run->month) &&
-            bit_is_set(nh_wom, run->wom) &&
-            bit_is_set(nh_woy, run->woy);
+            ((bit_is_set(nh_mday, run->mday) &&
+              bit_is_set(nh_wday, run->wday) &&
+              bit_is_set(nh_month, run->month) &&
+              bit_is_set(nh_wom, run->wom) &&
+              bit_is_set(nh_woy, run->woy)) ||
+             (bit_is_set(nh_month, run->month) &&
+              bit_is_set(31, run->mday) && nh_mday == nh_ldom));
 
          Dmsg3(dbglvl, "run@%p: run_now=%d run_nh=%d\n", run, run_now, run_nh);
 
@@ -384,14 +393,14 @@ static void add_job(JOB *job, RUN *run, time_t now, time_t runtime)
     */
    if (((runtime - run->last_run) < 61) || ((runtime+59) < now)) {
 #ifdef SCHED_DEBUG
-      Dmsg4(000, "Drop: Job=\"%s\" run=%lld. last_run=%lld. now=%lld\n", job->hdr.name, 
+      Dmsg4(000, "Drop: Job=\"%s\" run=%lld. last_run=%lld. now=%lld\n", job->hdr.name,
             (utime_t)runtime, (utime_t)run->last_run, (utime_t)now);
       fflush(stdout);
 #endif
       return;
    }
 #ifdef SCHED_DEBUG
-   Dmsg4(000, "Add: Job=\"%s\" run=%lld last_run=%lld now=%lld\n", job->hdr.name, 
+   Dmsg4(000, "Add: Job=\"%s\" run=%lld last_run=%lld now=%lld\n", job->hdr.name,
             (utime_t)runtime, (utime_t)run->last_run, (utime_t)now);
 #endif
    /* accept to run this job */
@@ -432,12 +441,13 @@ static void dump_job(job_item *ji, const char *msg)
 {
 #ifdef SCHED_DEBUG
    char dt[MAX_TIME_LENGTH];
-   int save_debug = debug_level;
-   if (debug_level < dbglvl) {
+   int64_t save_debug = debug_level;
+
+   if (!chk_dbglvl(dbglvl)) {
       return;
    }
    bstrftime_nc(dt, sizeof(dt), ji->runtime);
-   Dmsg4(dbglvl, "%s: Job=%s priority=%d run %s\n", msg, ji->job->hdr.name, 
+   Dmsg4(dbglvl, "%s: Job=%s priority=%d run %s\n", msg, ji->job->hdr.name,
       ji->Priority, dt);
    fflush(stdout);
    debug_level = save_debug;
