@@ -1,35 +1,25 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2001-2011 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2017 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation and included
-   in the file LICENSE.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   This notice must be preserved when any source code is
+   conveyed and/or propagated.
 
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  * Includes specific to the Director User Agent Server
  *
  *     Kern Sibbald, August MMI
- *
  */
 
 #ifndef __UA_H_
@@ -40,7 +30,9 @@ public:
    BSOCK *UA_sock;
    BSOCK *sd;
    JCR *jcr;
-   B_DB *db;
+   BDB *db;                          /* Pointing to shared or private db */
+   BDB *shared_db;                   /* Main Bacula DB access */
+   BDB *private_db;                  /* Private DB access */
    CAT *catalog;
    CONRES *cons;                      /* console resource */
    POOLMEM *cmd;                      /* return command/name buffer */
@@ -50,8 +42,10 @@ public:
    char *argv[MAX_CMD_ARGS];          /* argument values */
    int argc;                          /* number of arguments */
    char **prompt;                     /* list of prompts */
+   char **unique;                     /* extra unique field */
    int max_prompts;                   /* max size of list */
    int num_prompts;                   /* current number in list */
+   char api_opts[MAX_NAME_LENGTH];    /* Api options */
    int api;                           /* For programs want an API */
    int cmd_index;                     /* Index in command table */
    bool force_mult_db_connections;    /* overwrite cat.mult_db_connections */
@@ -67,6 +61,8 @@ public:
    int32_t  int32_val;                /* positive/negative */
    int64_t  int64_val;                /* big int */
 
+   void *bvfs;                        /* used in some bvfs queries */
+
    void signal(int sig) { UA_sock->signal(sig); };
 
    /* The below are in ua_output.c */
@@ -83,11 +79,17 @@ struct TREE_CTX {
    TREE_NODE *avail_node;             /* unused node last insert */
    int cnt;                           /* count for user feedback */
    bool all;                          /* if set mark all as default */
+   bool hardlinks_in_mem;             /* Set to optimize for speed */
+   bool no_auto_parent;               /* Set to not select parent directories */
    UAContext *ua;
    uint32_t FileEstimate;             /* estimate of number of files */
    uint32_t FileCount;                /* current count of files */
    uint32_t LastCount;                /* last count of files */
    uint32_t DeltaCount;               /* trigger for printing */
+   alist *uid_acl;                    /* UID allowed in the tree */
+   alist *gid_acl;                    /* GID allowed in the tree */
+   alist *dir_acl;                    /* Directories that can be displayed */
+   char  *last_dir_acl;               /* Last directory from the DirectoryACL list */
 };
 
 struct NAME_LIST {
@@ -106,6 +108,7 @@ struct RESTORE_CTX {
    JobId_t JobId;
    char ClientName[MAX_NAME_LENGTH];  /* backup client */
    char RestoreClientName[MAX_NAME_LENGTH];  /* restore client */
+   char RestoreMediaType[MAX_NAME_LENGTH];   /* restore Media type when storage override */
    char last_jobid[20];
    POOLMEM *JobIds;                   /* User entered string of JobIds */
    POOLMEM *BaseJobIds;               /* Base jobids */
@@ -118,7 +121,10 @@ struct RESTORE_CTX {
    char *where;
    char *RegexWhere;
    char *replace;
-   RBSR *bsr;
+   char *fileregex;
+
+   char *when;
+   rblist *bsr_list;
    POOLMEM *fname;                    /* filename only */
    POOLMEM *path;                     /* path only */
    POOLMEM *query;
@@ -126,7 +132,12 @@ struct RESTORE_CTX {
    int pnl;                           /* path length */
    bool found;
    bool all;                          /* mark all as default */
+   bool hardlinks_in_mem;             /* keep hard links in memory */
+   bool fdcalled;                     /* True if we should reuse the FD socket */
+   bool no_auto_parent;               /* Select or not parent directories */
    NAME_LIST name_list;
+   POOLMEM *component_fname;
+   FILE *component_fd;
 };
 
 #define MAX_ID_LIST_LEN 2000000
