@@ -1,24 +1,25 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2003-2014 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2017 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from many
-   others, a complete list can be found in the file AUTHORS.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
    You may use this file and others of this release according to the
    license defined in the LICENSE file, which includes the Affero General
    Public License, v3.0 ("AGPLv3") and some additional permissions and
    terms pursuant to its AGPLv3 Section 7.
 
-   Bacula® is a registered trademark of Kern Sibbald.
+   This notice must be preserved when any source code is
+   conveyed and/or propagated.
+
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
- *
  *   Bacula Director -- Tape labeling commands
  *
  *     Kern Sibbald, April MMIII
- *
  */
 
 #include "bacula.h"
@@ -174,7 +175,7 @@ void update_slots(UAContext *ua)
    if (!store.store) {
       return;
    }
-   pm_strcpy(store.store_source, _("command line"));
+   pm_strcpy(store.store_source, _("Command input"));
    set_wstorage(ua->jcr, &store);
    drive = get_storage_drive(ua, store.store);
 
@@ -302,8 +303,9 @@ void update_slots(UAContext *ua)
    db_unlock(ua->db);
 
 bail_out:
-
-   free_vol_list(vol_list);
+   if (vol_list) {
+      free_vol_list(vol_list);
+   }
    free(slot_list);
    close_sd_bsock(ua);
 
@@ -323,7 +325,7 @@ static int do_label(UAContext *ua, const char *cmd, int relabel)
    POOL_DBR pr;
    bool print_reminder = true;
    bool label_barcodes = false;
-   int ok = FALSE;
+   bool ok = false;
    int i, j;
    int drive;
    bool media_record_exists = false;
@@ -351,7 +353,7 @@ static int do_label(UAContext *ua, const char *cmd, int relabel)
    if (!store.store) {
       return 1;
    }
-   pm_strcpy(store.store_source, _("command line"));
+   pm_strcpy(store.store_source, _("Command input"));
    set_wstorage(ua->jcr, &store);
    drive = get_storage_drive(ua, store.store);
 
@@ -471,9 +473,9 @@ checkName:
          bstrncpy(dev_name, store.store->dev_name(), sizeof(dev_name));
          ua->info_msg(_("Requesting to mount %s ...\n"), dev_name);
          bash_spaces(dev_name);
-         sd->fsend("mount %s drive=%d", dev_name, drive);
+         sd->fsend("mount %s drive=%d slot=%d", dev_name, drive, mr.Slot);
          unbash_spaces(dev_name);
-         while (sd->recv() >= 0) {
+         while (bget_dirmsg(sd) >= 0) {
             ua->send_msg("%s", sd->msg);
             /* Here we can get
              *  3001 OK mount. Device=xxx      or
@@ -609,9 +611,11 @@ static void label_from_barcodes(UAContext *ua, int drive)
          }
          continue;                    /* done, go handle next volume */
       }
-      bstrncpy(mr.MediaType, store->media_type, sizeof(mr.MediaType));
 
+      /* Not a cleaning tape */
+      bstrncpy(mr.MediaType, store->media_type, sizeof(mr.MediaType));
       mr.Slot = vl->Slot;
+
       send_label_request(ua, &mr, &omr, &pr, 0, media_record_exists, drive);
    }
 
@@ -670,8 +674,9 @@ static bool send_label_request(UAContext *ua, MEDIA_DBR *mr, MEDIA_DBR *omr,
    BSOCK *sd;
    char dev_name[MAX_NAME_LENGTH];
    bool ok = false;
-   bool is_dvd = false;
    uint64_t VolBytes = 0;
+   uint64_t VolABytes = 0;
+   uint32_t VolType = 0;
 
    if (!(sd=open_sd_bsock(ua))) {
       return false;
@@ -700,54 +705,45 @@ static bool send_label_request(UAContext *ua, MEDIA_DBR *mr, MEDIA_DBR *omr,
          dev_name, mr->VolumeName, pr->Name, mr->MediaType, mr->Slot, drive);
    }
 
-   while (sd->recv() >= 0) {
-      int dvd;
+   while (bget_dirmsg(sd) >= 0) {
       ua->send_msg("%s", sd->msg);
-      if (sscanf(sd->msg, "3000 OK label. VolBytes=%llu DVD=%d ", &VolBytes,
-                 &dvd) == 2) {
-         is_dvd = dvd;
+      if (sscanf(sd->msg, "3000 OK label. VolBytes=%llu VolABytes=%lld VolType=%d ",
+                 &VolBytes, &VolABytes, &VolType) == 3) {
          ok = true;
-      }
-   }
-   unbash_spaces(mr->VolumeName);
-   unbash_spaces(mr->MediaType);
-   unbash_spaces(pr->Name);
-   mr->LabelDate = time(NULL);
-   mr->set_label_date = true;
-   if (is_dvd) {
-      /* We know that a freshly labelled DVD has 1 VolParts */
-      /* This does not apply to auto-labelled DVDs. */
-      mr->VolParts = 1;
-   }
-   if (ok) {
-      if (media_record_exists) {      /* we update it */
-         mr->VolBytes = VolBytes;
-         mr->InChanger = mr->Slot > 0;  /* if slot give assume in changer */
-         set_storageid_in_mr(ua->jcr->wstore, mr);
-         if (!db_update_media_record(ua->jcr, ua->db, mr)) {
-             ua->error_msg("%s", db_strerror(ua->db));
-             ok = false;
-         }
-      } else {                        /* create the media record */
-         set_pool_dbr_defaults_in_media_dbr(mr, pr);
-         mr->VolBytes = VolBytes;
-         mr->InChanger = mr->Slot > 0;  /* if slot give assume in changer */
-         mr->Enabled = 1;
-         set_storageid_in_mr(ua->jcr->wstore, mr);
-         if (db_create_media_record(ua->jcr, ua->db, mr)) {
-            ua->info_msg(_("Catalog record for Volume \"%s\", Slot %d  successfully created.\n"),
-            mr->VolumeName, mr->Slot);
-            /* Update number of volumes in pool */
-            pr->NumVols++;
-            if (!db_update_pool_record(ua->jcr, ua->db, pr)) {
+         if (media_record_exists) {      /* we update it */
+            mr->VolBytes = VolBytes;
+            mr->VolABytes = VolABytes;
+            mr->VolType = VolType;
+            mr->InChanger = mr->Slot > 0;  /* if slot give assume in changer */
+            set_storageid_in_mr(ua->jcr->wstore, mr);
+            if (!db_update_media_record(ua->jcr, ua->db, mr)) {
                ua->error_msg("%s", db_strerror(ua->db));
+               ok = false;
+            }
+         } else {                        /* create the media record */
+            set_pool_dbr_defaults_in_media_dbr(mr, pr);
+            mr->VolBytes = VolBytes;
+            mr->VolABytes = VolABytes;
+            mr->VolType = VolType;
+            mr->InChanger = mr->Slot > 0;  /* if slot give assume in changer */
+            mr->Enabled = 1;
+            set_storageid_in_mr(ua->jcr->wstore, mr);
+            if (db_create_media_record(ua->jcr, ua->db, mr)) {
+               ua->info_msg(_("Catalog record for Volume \"%s\", Slot %d  successfully created.\n"),
+                            mr->VolumeName, mr->Slot);
+               /* Update number of volumes in pool */
+               pr->NumVols++;
+               if (!db_update_pool_record(ua->jcr, ua->db, pr)) {
+                  ua->error_msg("%s", db_strerror(ua->db));
+               }
+            } else {
+               ua->error_msg("%s", db_strerror(ua->db));
+               ok = false;
             }
-         } else {
-            ua->error_msg("%s", db_strerror(ua->db));
-            ok = false;
          }
       }
-   } else {
+   }
+   if (!ok) {
       ua->error_msg(_("Label command failed for Volume %s.\n"), mr->VolumeName);
    }
    return ok;
@@ -853,7 +849,7 @@ static vol_list_t *get_vol_list_from_SD(UAContext *ua, bool scan)
          if (!is_volume_name_legal(ua, p)) {
             p--;
             *p = ':';
-            ua->error_msg(_("Invalid Volume name: %s\n"), sd->msg);
+            ua->error_msg(_("Invalid Volume name: %s. Volume skipped.\n"), sd->msg);
             continue;
          }
       }
@@ -990,6 +986,9 @@ int get_num_drives_from_SD(UAContext *ua)
  */
 static bool is_cleaning_tape(UAContext *ua, MEDIA_DBR *mr, POOL_DBR *pr)
 {
+   if (ua->jcr->pool->cleaning_prefix == NULL) {
+      return false;  /* if no cleaning prefix, this is not a cleaning tape */
+   }
    /* Find Pool resource */
    ua->jcr->pool = (POOL *)GetResWithName(R_POOL, pr->Name);
    if (!ua->jcr->pool) {
@@ -997,9 +996,6 @@ static bool is_cleaning_tape(UAContext *ua, MEDIA_DBR *mr, POOL_DBR *pr)
          pr->Name, mr->VolumeName);
       return false;
    }
-   if (ua->jcr->pool->cleaning_prefix == NULL) {
-      return false;
-   }
    Dmsg4(100, "CLNprefix=%s: Vol=%s: len=%d strncmp=%d\n",
       ua->jcr->pool->cleaning_prefix, mr->VolumeName,
       strlen(ua->jcr->pool->cleaning_prefix),
@@ -1009,7 +1005,10 @@ static bool is_cleaning_tape(UAContext *ua, MEDIA_DBR *mr, POOL_DBR *pr)
                   strlen(ua->jcr->pool->cleaning_prefix)) == 0;
 }
 
-static void content_send_info(UAContext *ua, char type, int Slot, char *vol_name)
+/*
+ * Send Volume info to caller in API format
+ */
+static void send_volume_info(UAContext *ua, char type, int Slot, char *vol_name)
 {
    char ed1[50], ed2[50], ed3[50];
    POOL_DBR pr;
@@ -1133,7 +1132,7 @@ void status_content(UAContext *ua, STORE *store)
          ua->send_msg("D|%d||\n", Drive);
 
       } else if (sscanf(sd->msg, "%c:%d:F:%127s", &type, &Slot, vol_name)== 3) {
-         content_send_info(ua, type, Slot, vol_name);
+         send_volume_info(ua, type, Slot, vol_name);
 
       } else if (sscanf(sd->msg, "%c:%d:E", &type, &Slot) == 2) {
          /* type can be S (slot) or I (Import/Export slot) */
@@ -1154,7 +1153,7 @@ void status_content(UAContext *ua, STORE *store)
             }
             prev = vl;
          }
-         content_send_info(ua, type, Slot, vol_name);
+         send_volume_info(ua, type, Slot, vol_name);
 
       } else {
          Dmsg1(10, "Discarding msg=%s\n", sd->msg);
@@ -1176,7 +1175,7 @@ void status_slots(UAContext *ua, STORE *store_r)
    int max_slots;
    int i=1;
    /* Slot | Volume | Status | MediaType | Pool */
-   const char *slot_hformat=" %4i%c| %16s | %9s | %20s | %18s |\n";
+   const char *slot_hformat="| %4i%c| %-20s | %-9s | %-15s | %-18s |\n";
 
    if (ua->api) {
       status_content(ua, store_r);
@@ -1188,7 +1187,7 @@ void status_slots(UAContext *ua, STORE *store_r)
    }
    store.store = store_r;
 
-   pm_strcpy(store.store_source, _("command line"));
+   pm_strcpy(store.store_source, _("Command input"));
    set_wstorage(ua->jcr, &store);
    get_storage_drive(ua, store.store);
 
@@ -1210,8 +1209,9 @@ void status_slots(UAContext *ua, STORE *store_r)
       ua->warning_msg(_("No Volumes found, or no barcodes.\n"));
       goto bail_out;
    }
-   ua->send_msg(_(" Slot |   Volume Name    |   Status  |     Media Type       |      Pool          |\n"));
-   ua->send_msg(_("------+------------------+-----------+----------------------+--------------------|\n"));
+   ua->send_msg(_("+------+----------------------+-----------+-----------------+--------------------+\n"));
+   ua->send_msg(_("| Slot | Volume Name          | Status    | Media Type      | Pool               |\n"));
+   ua->send_msg(_("+------+----------------------+-----------+-----------------+--------------------+\n"));
 
    /* Walk through the list getting the media records */
    for (vl=vol_list; vl; vl=vl->next) {
@@ -1244,6 +1244,7 @@ void status_slots(UAContext *ua, STORE *store_r)
             slot_list[i]=0;
          }
       }
+   ua->send_msg(_("+------+----------------------+-----------+-----------------+--------------------+\n"));
 
       memset(&mr, 0, sizeof(MEDIA_DBR));
       bstrncpy(mr.VolumeName, vl->VolName, sizeof(mr.VolumeName));
