@@ -1,29 +1,20 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2007 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2015 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation and included
-   in the file LICENSE.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   This notice must be preserved when any source code is 
+   conveyed and/or propagated.
 
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  *
@@ -31,7 +22,6 @@
  *
  *     Kern Sibbald, September MM
  *
- *    Version $Id$
  */
 
 #include "bacula.h"
@@ -99,11 +89,13 @@ JCR *new_control_jcr(const char *base_name, int job_type)
    LockRes();
    jcr->job = (JOB *)GetNextRes(R_JOB, NULL);
    set_jcr_defaults(jcr, jcr->job);
+   /* We use a resource, so we should count in the reload */
+   jcr->setJobType(job_type);
    UnlockRes();
+
    jcr->sd_auth_key = bstrdup("dummy"); /* dummy Storage daemon key */
    create_unique_job_name(jcr, base_name);
    jcr->sched_time = jcr->start_time;
-   jcr->setJobType(job_type);
    jcr->setJobLevel(L_NONE);
    jcr->setJobStatus(JS_Running);
    jcr->JobId = 0;
@@ -163,11 +155,14 @@ static void *handle_UA_client_request(void *arg)
             }
             if (!ua->api) user->signal(BNET_EOD);     /* send end of command */
          }
-      } else if (is_bnet_stop(user)) {
+      } else if (user->is_stop()) {
          ua->quit = true;
       } else { /* signal */
          user->signal(BNET_POLL);
       }
+
+      /* At the end of each command, revert to the main shared SQL link */
+      ua->db = ua->shared_db;
    }
 
 getout:
@@ -192,7 +187,7 @@ UAContext *new_ua_context(JCR *jcr)
    ua = (UAContext *)malloc(sizeof(UAContext));
    memset(ua, 0, sizeof(UAContext));
    ua->jcr = jcr;
-   ua->db = jcr->db;
+   ua->shared_db = ua->db = jcr->db;
    ua->cmd = get_pool_memory(PM_FNAME);
    ua->args = get_pool_memory(PM_FNAME);
    ua->errmsg = get_pool_memory(PM_FNAME);
@@ -215,10 +210,10 @@ void free_ua_context(UAContext *ua)
    if (ua->prompt) {
       free(ua->prompt);
    }
-   if (ua->UA_sock) {
-      bnet_close(ua->UA_sock);
-      ua->UA_sock = NULL;
+   if (ua->unique) {
+      free(ua->unique);
    }
+   free_bsock(ua->UA_sock);
    free(ua);
 }
 
