@@ -1,24 +1,25 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2001-2014 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2017 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from many
-   others, a complete list can be found in the file AUTHORS.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
    You may use this file and others of this release according to the
    license defined in the LICENSE file, which includes the Affero General
    Public License, v3.0 ("AGPLv3") and some additional permissions and
    terms pursuant to its AGPLv3 Section 7.
 
-   Bacula® is a registered trademark of Kern Sibbald.
+   This notice must be preserved when any source code is 
+   conveyed and/or propagated.
+
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
- *
  *   Bacula Director -- User Agent Status Command
  *
  *     Kern Sibbald, August MMI
- *
  */
 
 
@@ -65,7 +66,7 @@ bool dot_status_cmd(UAContext *ua, const char *cmd)
       if (strcasecmp(ua->argk[2], "current") == 0) {
          ua->send_msg(OKqstatus, ua->argk[2]);
          foreach_jcr(njcr) {
-            if (njcr->JobId != 0 && acl_access_ok(ua, Job_ACL, njcr->job->name())) {
+            if (!njcr->is_internal_job() && acl_access_ok(ua, Job_ACL, njcr->job->name())) {
                ua->send_msg(DotStatusJob, edit_int64(njcr->JobId, ed1),
                         njcr->JobStatus, njcr->JobErrors);
             }
@@ -93,7 +94,7 @@ bool dot_status_cmd(UAContext *ua, const char *cmd)
          return false;
       }
    } else if (strcasecmp(ua->argk[1], "client") == 0) {
-      client = get_client_resource(ua);
+      client = get_client_resource(ua, JT_BACKUP_RESTORE);
       if (client) {
          Dmsg2(200, "Client=%s arg=%s\n", client->name(), NPRT(ua->argk[2]));
          do_client_status(ua, client, ua->argk[2]);
@@ -113,6 +114,133 @@ bool dot_status_cmd(UAContext *ua, const char *cmd)
    return true;
 }
 
+/* Test the network between FD and SD */
+static int do_network_status(UAContext *ua)
+{
+   CLIENT *client = NULL;
+   USTORE  store;
+   JCR *jcr = ua->jcr;
+   char *store_address, ed1[50];
+   uint32_t store_port;
+   uint64_t nb = 50 * 1024 * 1024;
+   POOL_MEM buf;
+
+   int i = find_arg_with_value(ua, "bytes");
+   if (i > 0) {
+      if (!size_to_uint64(ua->argv[i], strlen(ua->argv[i]), &nb)) {
+         return 1;
+      }
+   }
+   
+   client = get_client_resource(ua, JT_BACKUP_RESTORE);
+   if (!client) {
+      return 1;
+   }
+
+   store.store = get_storage_resource(ua, false, true);
+   if (!store.store) {
+      return 1;
+   }
+
+   jcr->client = client;
+   set_wstorage(jcr, &store);
+
+   if (!ua->api) {
+      ua->send_msg(_("Connecting to Storage %s at %s:%d\n"),
+                   store.store->name(), store.store->address, store.store->SDport);
+   }
+
+   if (!connect_to_storage_daemon(jcr, 10, SDConnectTimeout, 1)) {
+      ua->error_msg(_("Failed to connect to Storage.\n"));
+      goto bail_out;
+   }
+
+   if (!start_storage_daemon_job(jcr, NULL, NULL)) {
+      goto bail_out;
+   }
+
+   /*
+    * Note startup sequence of SD/FD is different depending on
+    *  whether the SD listens (normal) or the SD calls the FD.
+    */
+   if (!client->sd_calls_client) {
+      if (!run_storage_and_start_message_thread(jcr, jcr->store_bsock)) {
+         goto bail_out;
+      }
+   } /* Else it's done in init_storage_job() */
+
+   if (!ua->api) {
+      ua->send_msg(_("Connecting to Client %s at %s:%d\n"),
+                   client->name(), client->address(buf.addr()), client->FDport);
+   }
+
+   if (!connect_to_file_daemon(jcr, 1, 15, 0)) {
+      ua->error_msg(_("Failed to connect to Client.\n"));
+      goto bail_out;
+   }
+
+   if (jcr->sd_calls_client) {
+      /*
+       * SD must call "client" i.e. FD
+       */
+      if (jcr->FDVersion < 10) {
+         Jmsg(jcr, M_FATAL, 0, _("The File daemon does not support SDCallsClient.\n"));
+         goto bail_out;
+      }
+      if (!send_client_addr_to_sd(jcr)) {
+         goto bail_out;
+      }
+      if (!run_storage_and_start_message_thread(jcr, jcr->store_bsock)) {
+         goto bail_out;
+      }
+
+      store_address = store.store->address;  /* dummy */
+      store_port = 0;                        /* flag that SD calls FD */
+
+   } else {
+      /*
+       * send Storage daemon address to the File daemon,
+       *   then wait for File daemon to make connection
+       *   with Storage daemon.
+       */
+      if (store.store->SDDport == 0) {
+         store.store->SDDport = store.store->SDport;
+      }
+
+      store_address = get_storage_address(jcr->client, store.store);
+      store_port = store.store->SDDport;
+   }
+
+   if (!send_store_addr_to_fd(jcr, store.store, store_address, store_port)) {
+      goto bail_out;
+   }
+
+   if (!ua->api) {
+      ua->info_msg(_("Running network test between Client=%s and Storage=%s with %sB ...\n"),
+                   client->name(), store.store->name(), edit_uint64_with_suffix(nb, ed1));
+   }
+
+   if (!jcr->file_bsock->fsend("testnetwork bytes=%lld\n", nb)) {
+      goto bail_out;
+   }
+
+   while (jcr->file_bsock->recv() > 0) {
+      ua->info_msg(jcr->file_bsock->msg);
+   }
+   
+bail_out:
+   jcr->file_bsock->signal(BNET_TERMINATE);
+   jcr->store_bsock->signal(BNET_TERMINATE);
+   wait_for_storage_daemon_termination(jcr);
+
+   free_bsock(jcr->file_bsock);
+   free_bsock(jcr->store_bsock);
+
+   jcr->client = NULL;
+   free_wstorage(jcr);
+   return 1;
+}
+
 /* This is the *old* command handler, so we must return
  *  1 or it closes the connection
  */
@@ -134,7 +262,10 @@ int status_cmd(UAContext *ua, const char *cmd)
    Dmsg1(20, "status:%s:\n", cmd);
 
    for (i=1; i<ua->argc; i++) {
-      if (strcasecmp(ua->argk[i], NT_("schedule")) == 0 ||
+      if (strcasecmp(ua->argk[i], NT_("network")) == 0) {
+         do_network_status(ua);
+         return 1;
+      } else if (strcasecmp(ua->argk[i], NT_("schedule")) == 0 ||
           strcasecmp(ua->argk[i], NT_("scheduled")) == 0) {
          llist_scheduled_jobs(ua);
          return 1;
@@ -146,7 +277,7 @@ int status_cmd(UAContext *ua, const char *cmd)
          do_director_status(ua);
          return 1;
       } else if (strcasecmp(ua->argk[i], NT_("client")) == 0) {
-         client = get_client_resource(ua);
+         client = get_client_resource(ua, JT_BACKUP_RESTORE);
          if (client) {
             do_client_status(ua, client, NULL);
          }
@@ -172,6 +303,7 @@ int status_cmd(UAContext *ua, const char *cmd)
       add_prompt(ua, NT_("Storage"));
       add_prompt(ua, NT_("Client"));
       add_prompt(ua, NT_("Scheduled"));
+      add_prompt(ua, NT_("Network"));
       add_prompt(ua, NT_("All"));
       Dmsg0(20, "do_prompt: select daemon\n");
       if ((item=do_prompt(ua, "",  _("Select daemon type for status"), prmt, sizeof(prmt))) < 0) {
@@ -189,7 +321,7 @@ int status_cmd(UAContext *ua, const char *cmd)
          }
          break;
       case 2:
-         client = select_client_resource(ua);
+         client = select_client_resource(ua, JT_BACKUP_RESTORE);
          if (client) {
             do_client_status(ua, client, NULL);
          }
@@ -198,6 +330,9 @@ int status_cmd(UAContext *ua, const char *cmd)
          llist_scheduled_jobs(ua);
          break;
       case 4:
+         do_network_status(ua);
+         break;
+      case 5:
          do_all_status(ua);
          break;
       default:
@@ -211,6 +346,7 @@ static void do_all_status(UAContext *ua)
 {
    STORE *store, **unique_store;
    CLIENT *client, **unique_client;
+   POOL_MEM buf1, buf2;
    int i, j;
    bool found;
 
@@ -261,11 +397,11 @@ static void do_all_status(UAContext *ua)
    i = 0;
    foreach_res(client, R_CLIENT) {
       found = false;
-      if (!acl_access_ok(ua, Client_ACL, client->name())) {
+      if (!acl_access_client_ok(ua, client->name(), JT_BACKUP_RESTORE)) {
          continue;
       }
       for (j=0; j<i; j++) {
-         if (strcmp(unique_client[j]->address, client->address) == 0 &&
+         if (strcmp(unique_client[j]->address(buf1.addr()), client->address(buf2.addr())) == 0 &&
              unique_client[j]->FDport == client->FDport) {
             found = true;
             break;
@@ -273,7 +409,7 @@ static void do_all_status(UAContext *ua)
       }
       if (!found) {
          unique_client[i++] = client;
-         Dmsg2(40, "Stuffing: %s:%d\n", client->address, client->FDport);
+         Dmsg2(40, "Stuffing: %s:%d\n", client->address(buf1.addr()), client->FDport);
       }
    }
    UnlockRes();
@@ -286,31 +422,71 @@ static void do_all_status(UAContext *ua)
 
 }
 
+static void api_list_dir_status_header(UAContext *ua)
+{
+   OutputWriter wt(ua->api_opts);
+   wt.start_group("header");
+   wt.get_output(
+      OT_STRING, "name",        my_name,
+      OT_STRING, "version",     VERSION " (" BDATE ")",
+      OT_STRING, "uname",       HOST_OS " " DISTNAME " " DISTVER,
+      OT_UTIME,  "started",     daemon_start_time,
+      OT_UTIME,  "reloaded",    last_reload_time,
+      OT_INT,    "jobs_run",    num_jobs_run,
+      OT_INT,    "jobs_running",job_count(),
+      OT_INT,    "nclients",    ((rblist *)res_head[R_CLIENT-r_first]->res_list)->size(),
+      OT_INT,    "nstores",     ((rblist *)res_head[R_STORAGE-r_first]->res_list)->size(),
+      OT_INT,    "npools",      ((rblist *)res_head[R_POOL-r_first]->res_list)->size(),
+      OT_INT,    "ncats",       ((rblist *)res_head[R_CATALOG-r_first]->res_list)->size(),
+      OT_INT,    "nfset",       ((rblist *)res_head[R_FILESET-r_first]->res_list)->size(),
+      OT_INT,    "nscheds",     ((rblist *)res_head[R_SCHEDULE-r_first]->res_list)->size(),
+      OT_PLUGINS,"plugins",     b_plugin_list,
+      OT_END);
+
+   ua->send_msg("%s", wt.end_group());
+}
+
 void list_dir_status_header(UAContext *ua)
 {
-   char dt[MAX_TIME_LENGTH];
+   char dt[MAX_TIME_LENGTH], dt1[MAX_TIME_LENGTH];
    char b1[35], b2[35], b3[35], b4[35], b5[35];
 
-   ua->send_msg(_("%s Version: %s (%s) %s %s %s\n"), my_name, VERSION, BDATE,
-            HOST_OS, DISTNAME, DISTVER);
+   if (ua->api > 1) {
+      api_list_dir_status_header(ua);
+      return;
+   }
+
+   ua->send_msg(_("%s %sVersion: %s (%s) %s %s %s\n"), my_name,
+            "", VERSION, BDATE, HOST_OS, DISTNAME, DISTVER);
    bstrftime_nc(dt, sizeof(dt), daemon_start_time);
-   ua->send_msg(_("Daemon started %s. Jobs: run=%d, running=%d "
-                  "mode=%d,%d\n"), dt,
-                num_jobs_run, job_count(), (int)DEVELOPER_MODE, (int)BEEF);
+   bstrftimes(dt1, sizeof(dt1), last_reload_time);
+   ua->send_msg(_("Daemon started %s, conf reloaded %s\n"), dt, dt1);
+   ua->send_msg(_(" Jobs: run=%d, running=%d mode=%d,%d\n"),
+      num_jobs_run, job_count(), (int)DEVELOPER_MODE, 0);
    ua->send_msg(_(" Heap: heap=%s smbytes=%s max_bytes=%s bufs=%s max_bufs=%s\n"),
-            edit_uint64_with_commas((char *)sbrk(0)-(char *)start_heap, b1),
-            edit_uint64_with_commas(sm_bytes, b2),
-            edit_uint64_with_commas(sm_max_bytes, b3),
-            edit_uint64_with_commas(sm_buffers, b4),
-            edit_uint64_with_commas(sm_max_buffers, b5));
+      edit_uint64_with_commas((char *)sbrk(0)-(char *)start_heap, b1),
+      edit_uint64_with_commas(sm_bytes, b2),
+      edit_uint64_with_commas(sm_max_bytes, b3),
+      edit_uint64_with_commas(sm_buffers, b4),
+      edit_uint64_with_commas(sm_max_buffers, b5));
+   ua->send_msg(_(" Res: njobs=%d nclients=%d nstores=%d npools=%d ncats=%d"
+                  " nfsets=%d nscheds=%d\n"),
+      ((rblist *)res_head[R_JOB-r_first]->res_list)->size(),
+      ((rblist *)res_head[R_CLIENT-r_first]->res_list)->size(),
+      ((rblist *)res_head[R_STORAGE-r_first]->res_list)->size(),
+      ((rblist *)res_head[R_POOL-r_first]->res_list)->size(),
+      ((rblist *)res_head[R_CATALOG-r_first]->res_list)->size(),
+      ((rblist *)res_head[R_FILESET-r_first]->res_list)->size(),
+      ((rblist *)res_head[R_SCHEDULE-r_first]->res_list)->size());
+
 
    /* TODO: use this function once for all daemons */
-   if (bplugin_list->size() > 0) {
+   if (b_plugin_list && b_plugin_list->size() > 0) {
       int len;
       Plugin *plugin;
       POOL_MEM msg(PM_FNAME);
       pm_strcpy(msg, " Plugin: ");
-      foreach_alist(plugin, bplugin_list) {
+      foreach_alist(plugin, b_plugin_list) {
          len = pm_strcat(msg, plugin->file);
          if (len > 80) {
             pm_strcat(msg, "\n   ");
@@ -355,7 +531,7 @@ static void do_storage_status(UAContext *ua, STORE *store, char *cmd)
    }
    /*
     * The Storage daemon is problematic because it shows information
-    *  related to multiple Jobs, so if there is a Client or Job
+    *  related to multiple Job, so if there is a Client or Job
     *  ACL restriction, we forbid all access to the Storage.
     */
    if (have_restricted_acl(ua, Client_ACL) ||
@@ -379,13 +555,29 @@ static void do_storage_status(UAContext *ua, STORE *store, char *cmd)
    sd = ua->jcr->store_bsock;
    if (cmd) {
       POOL_MEM devname;
-      int i = find_arg_with_value(ua, "device");
-      if (i>0) {
-         Mmsg(devname, "device=%s", ua->argv[i]);
+      /*
+       * For .status storage=xxx shstore list
+       *  send .status shstore list xxx-device
+       */
+      if (strcasecmp(cmd, "shstore") == 0) {
+         if (!ua->argk[3]) {
+            ua->send_msg(_("Must have three arguments\n"));
+            return;
+         }
+         pm_strcpy(devname, store->dev_name());
          bash_spaces(devname.c_str());
+         sd->fsend(".status %s %s %s api=%d api_opts=%s",
+                   cmd, ua->argk[3], devname.c_str(),
+                   ua->api, ua->api_opts);
+      } else {
+         int i = find_arg_with_value(ua, "device");
+         if (i>0) {
+            Mmsg(devname, "device=%s", ua->argv[i]);
+            bash_spaces(devname.c_str());
+         }
+         sd->fsend(".status %s api=%d api_opts=%s %s",
+                   cmd, ua->api, ua->api_opts, devname.c_str());
       }
-      sd->fsend(".status %s api=%d api_opts=%s %s",
-                cmd, ua->api, ua->api_opts, devname.c_str());
    } else {
       sd->fsend("status");
    }
@@ -400,8 +592,9 @@ static void do_storage_status(UAContext *ua, STORE *store, char *cmd)
 static void do_client_status(UAContext *ua, CLIENT *client, char *cmd)
 {
    BSOCK *fd;
+   POOL_MEM buf;
 
-   if (!acl_access_ok(ua, Client_ACL, client->name())) {
+   if (!acl_access_client_ok(ua, client->name(), JT_BACKUP_RESTORE)) {
       ua->error_msg(_("No authorization for Client \"%s\"\n"), client->name());
       return;
    }
@@ -416,7 +609,7 @@ static void do_client_status(UAContext *ua, CLIENT *client, char *cmd)
 
    /* Try to connect for 15 seconds */
    if (!ua->api) ua->send_msg(_("Connecting to Client %s at %s:%d\n"),
-      client->name(), client->address, client->FDport);
+                              client->name(), client->address(buf.addr()), client->FDport);
    if (!connect_to_file_daemon(ua->jcr, 1, 15, 0)) {
       ua->send_msg(_("Failed to connect to Client %s.\n====\n"),
          client->name());
@@ -469,7 +662,7 @@ struct sched_pkt {
    STORE *store;
 };
 
-static void prt_runtime(UAContext *ua, sched_pkt *sp)
+static void prt_runtime(UAContext *ua, sched_pkt *sp, OutputWriter *ow)
 {
    char dt[MAX_TIME_LENGTH];
    const char *level_ptr;
@@ -502,17 +695,36 @@ static void prt_runtime(UAContext *ua, sched_pkt *sp)
    bstrftime_nc(dt, sizeof(dt), sp->runtime);
    switch (sp->job->JobType) {
    case JT_ADMIN:
+      level_ptr = "Admin";
+      break;
    case JT_RESTORE:
-      level_ptr = " ";
+      level_ptr = "Restore";
       break;
    default:
       level_ptr = level_to_str(sp->level);
       break;
    }
-   if (ua->api) {
+   if (ua->api == 1) {
       ua->send_msg(_("%-14s\t%-8s\t%3d\t%-18s\t%-18s\t%s\n"),
          level_ptr, job_type_to_str(sp->job->JobType), sp->priority, dt,
          sp->job->name(), mr.VolumeName);
+
+   } else if (ua->api > 1) {
+      ua->send_msg("%s",
+                   ow->get_output(OT_CLEAR,
+                      OT_START_OBJ,
+                      OT_STRING,    "name",     sp->job->name(),
+                      OT_JOBLEVEL,  "level",   sp->level,
+                      OT_JOBTYPE,   "type",    sp->job->JobType,
+                      OT_INT,       "priority",sp->priority,
+                      OT_UTIME,     "schedtime", sp->runtime,
+                      OT_STRING,    "volume",  mr.VolumeName,
+                      OT_STRING,    "pool",    jcr->pool?jcr->pool->name():"",
+                      OT_STRING,    "storage", jcr->wstore?jcr->wstore->name():"",
+                      OT_END_OBJ,
+                      OT_END));
+
+
    } else {
       ua->send_msg(_("%-14s %-8s %3d  %-18s %-18s %s\n"),
          level_ptr, job_type_to_str(sp->job->JobType), sp->priority, dt,
@@ -549,8 +761,8 @@ static void llist_scheduled_jobs(UAContext *ua)
    i = find_arg_with_value(ua, NT_("days"));
    if (i >= 0) {
      days = atoi(ua->argv[i]);
-     if (((days < 0) || (days > 500)) && !ua->api) {
-       ua->send_msg(_("Ignoring invalid value for days. Max is 500.\n"));
+     if (((days < 0) || (days > 3000)) && !ua->api) {
+       ua->send_msg(_("Ignoring invalid value for days. Max is 3000.\n"));
        days = 10;
      }
    } else {
@@ -595,10 +807,11 @@ static void llist_scheduled_jobs(UAContext *ua)
    LockRes();
    foreach_res(job, R_JOB) {
       sched = job->schedule;
-      if (sched == NULL || !job->enabled) { /* scheduled? or enabled? */
+      if (!sched || !job->is_enabled() || (sched && !sched->is_enabled()) ||
+         (job->client && !job->client->is_enabled())) {
          continue;                    /* no, skip this job */
       }
-      if (job_name[0] && bstrcmp(job_name, job->name()) != 0) {
+      if (job_name[0] && strcmp(job_name, job->name()) != 0) {
          continue;
       }
       for (run=sched->run; run; run=run->next) {
@@ -642,16 +855,6 @@ static void llist_scheduled_jobs(UAContext *ua)
                next += 24 * 60 * 60;   /* Add one day */
                continue;
             }
-            for (int j=0; j < 24; j++) {
-               if (bit_is_set(j, run->hour)) {
-                  tm.tm_hour = j;
-                  tm.tm_min = run->minute;
-                  tm.tm_sec = 0;
-                  runtime = mktime(&tm);
-                  bstrftime_dn(dt, sizeof(dt), runtime);
-                  break;
-               }
-            }
 
             level = job->JobLevel;
             if (run->level) {
@@ -659,8 +862,10 @@ static void llist_scheduled_jobs(UAContext *ua)
             }
             switch (job->JobType) {
             case JT_ADMIN:
+               level_ptr = "Admin";
+               break;
             case JT_RESTORE:
-               level_ptr = " ";
+               level_ptr = "Restore";
                break;
             default:
                level_ptr = level_to_str(level);
@@ -674,14 +879,24 @@ static void llist_scheduled_jobs(UAContext *ua)
                prt_lrunhdr(ua);
                hdr_printed = true;
             }
-            if (ua->api) {
-               ua->send_msg(_("%-14s\t%-8s\t%3d\t%-18s\t%-18s\t%s\n"),
-                  level_ptr, job_type_to_str(job->JobType), priority, dt,
-                  job->name(), sched->name());
-            } else {
-               ua->send_msg(_("%-14s %-8s %3d  %-18s %-18s %s\n"),
-                  level_ptr, job_type_to_str(job->JobType), priority, dt,
-                  job->name(), sched->name());
+
+            for (int j=0; j < 24; j++) {
+               if (bit_is_set(j, run->hour)) {
+                  tm.tm_hour = j;
+                  tm.tm_min = run->minute;
+                  tm.tm_sec = 0;
+                  runtime = mktime(&tm);
+                  bstrftime_dn(dt, sizeof(dt), runtime);
+                  if (ua->api) {
+                     ua->send_msg(_("%-14s\t%-8s\t%3d\t%-18s\t%-18s\t%s\n"),
+                     level_ptr, job_type_to_str(job->JobType), priority, dt,
+                     job->name(), sched->name());
+                  } else {
+                     ua->send_msg(_("%-14s %-8s %3d  %-18s %-18s %s\n"),
+                     level_ptr, job_type_to_str(job->JobType), priority, dt,
+                     job->name(), sched->name());
+                  }
+               }
             }
             next += 24 * 60 * 60;   /* Add one day */
             num_jobs++;
@@ -727,6 +942,7 @@ static int my_compare(void *item1, void *item2)
  */
 static void list_scheduled_jobs(UAContext *ua)
 {
+   OutputWriter ow(ua->api_opts);
    utime_t runtime;
    RUN *run;
    JOB *job;
@@ -759,7 +975,7 @@ static void list_scheduled_jobs(UAContext *ua)
    /* Loop through all jobs */
    LockRes();
    foreach_res(job, R_JOB) {
-      if (!acl_access_ok(ua, Job_ACL, job->name()) || !job->enabled) {
+      if (!acl_access_ok(ua, Job_ACL, job->name()) || !job->is_enabled()) {
          continue;
       }
       if (sched_name[0] && job->schedule &&
@@ -795,7 +1011,7 @@ static void list_scheduled_jobs(UAContext *ua)
    } /* end for loop over resources */
    UnlockRes();
    foreach_dlist(sp, &sched) {
-      prt_runtime(ua, sp);
+      prt_runtime(ua, sp, &ow);
    }
    if (num_jobs == 0 && !ua->api) {
       ua->send_msg(_("No Scheduled Jobs.\n"));
@@ -808,48 +1024,63 @@ static void list_running_jobs(UAContext *ua)
 {
    JCR *jcr;
    int njobs = 0;
-   const char *msg;
+   int i;
+   int32_t status;
+   const char *msg, *msgdir;
    char *emsg;                        /* edited message */
    char dt[MAX_TIME_LENGTH];
    char level[10];
    bool pool_mem = false;
+   OutputWriter ow(ua->api_opts);
+   JobId_t jid = 0;
+
+   if ((i = find_arg_with_value(ua, "jobid")) >= 0) {
+      jid = str_to_int64(ua->argv[i]);
+   }
 
    Dmsg0(200, "enter list_run_jobs()\n");
-   if (!ua->api) ua->send_msg(_("\nRunning Jobs:\n"));
-   foreach_jcr(jcr) {
-      if (jcr->JobId == 0) {      /* this is us */
-         /* this is a console or other control job. We only show console
-          * jobs in the status output.
-          */
-         if (jcr->getJobType() == JT_CONSOLE && !ua->api) {
-            bstrftime_nc(dt, sizeof(dt), jcr->start_time);
-            ua->send_msg(_("Console connected at %s\n"), dt);
-         }
-         continue;
-      }
-      njobs++;
-   }
-   endeach_jcr(jcr);
 
-   if (njobs == 0) {
-      /* Note the following message is used in regress -- don't change */
-      if (!ua->api)  {
-         ua->send_msg(_("No Jobs running.\n====\n"));
-      }
-      Dmsg0(200, "leave list_run_jobs()\n");
-      return;
-   }
-   njobs = 0;
    if (!ua->api) {
-      ua->send_msg(_(" JobId  Type Level     Files     Bytes  Name              Status\n"));
-      ua->send_msg(_("======================================================================\n"));
+      ua->send_msg(_("\nRunning Jobs:\n"));
+      foreach_jcr(jcr) {
+         if (jcr->JobId == 0) {      /* this is us */
+            /* this is a console or other control job. We only show console
+             * jobs in the status output.
+             */
+            if (jcr->getJobType() == JT_CONSOLE) {
+               bstrftime_nc(dt, sizeof(dt), jcr->start_time);
+               ua->send_msg(_("Console connected %sat %s\n"),
+                            (ua->UA_sock && ua->UA_sock->tls)?_("using TLS "):"",
+                            dt);
+            }
+            continue;
+         }
+      }
+      endeach_jcr(jcr);
    }
+
+   njobs = 0; /* count the number of job really displayed */
    foreach_jcr(jcr) {
-      if (jcr->JobId == 0 || !acl_access_ok(ua, Job_ACL, jcr->job->name())) {
+      if (jcr->JobId == 0 || !jcr->job || !acl_access_ok(ua, Job_ACL, jcr->job->name())) {
          continue;
       }
-      njobs++;
-      switch (jcr->JobStatus) {
+      /* JobId keyword found in command line */
+      if (jid > 0 && jcr->JobId != jid) {
+         continue;
+      }
+
+      if (++njobs == 1) {
+         /* display the header for the first job */
+         if (!ua->api) {
+            ua->send_msg(_(" JobId  Type Level     Files     Bytes  Name              Status\n"));
+            ua->send_msg(_("======================================================================\n"));
+
+         } else if (ua->api > 1) {
+            ua->send_msg(ow.start_group("running", false));
+         }
+      }
+      status = jcr->JobStatus;
+      switch (status) {
       case JS_Created:
          msg = _("is waiting execution");
          break;
@@ -865,6 +1096,9 @@ static void list_running_jobs(UAContext *ua)
       case JS_Warnings:
          msg = _("has terminated with warnings");
          break;
+      case JS_Incomplete:
+         msg = _("has terminated in incomplete state");
+         break;
       case JS_ErrorTerminated:
          msg = _("has erred");
          break;
@@ -947,6 +1181,7 @@ static void list_running_jobs(UAContext *ua)
          msg = emsg;
          break;
       }
+      msgdir = msg;             /* Keep it to know if we update the status variable */
       /*
        * Now report Storage daemon status code
        */
@@ -966,17 +1201,21 @@ static void list_running_jobs(UAContext *ua)
          msg = _("is waiting for an appendable Volume");
          break;
       case JS_WaitFD:
-         if (!pool_mem) {
-            emsg = (char *)get_pool_memory(PM_FNAME);
-            pool_mem = true;
+         /* Special case when JobStatus=JS_WaitFD, we don't have a FD link yet 
+          * we need to stay in WaitFD status See bee mantis #1414 */
+         if (jcr->JobStatus != JS_WaitFD) {
+            if (!pool_mem) {
+               emsg = (char *)get_pool_memory(PM_FNAME);
+               pool_mem = true;
+            }
+            if (!jcr->client || !jcr->wstore) {
+               Mmsg(emsg, _("is waiting for Client to connect to Storage daemon"));
+            } else {
+               Mmsg(emsg, _("is waiting for Client %s to connect to Storage %s"),
+                    jcr->client->name(), jcr->wstore->name());
+            }
+            msg = emsg;
          }
-         if (!jcr->client || !jcr->wstore) {
-            Mmsg(emsg, _("is waiting for Client to connect to Storage daemon"));
-         } else {
-            Mmsg(emsg, _("is waiting for Client %s to connect to Storage %s"),
-                 jcr->client->name(), jcr->wstore->name());
-        }
-        msg = emsg;
         break;
       case JS_DataCommitting:
          msg = _("SD committing Data");
@@ -991,10 +1230,15 @@ static void list_running_jobs(UAContext *ua)
          msg = _("Dir inserting Attributes");
          break;
       }
+      if (msg != msgdir) {
+         status = jcr->SDJobStatus;
+      }
       switch (jcr->getJobType()) {
       case JT_ADMIN:
+         bstrncpy(level, "Admin", sizeof(level));
+         break;
       case JT_RESTORE:
-         bstrncpy(level, "      ", sizeof(level));
+         bstrncpy(level, "Restore", sizeof(level));
          break;
       default:
          bstrncpy(level, level_to_str(jcr->getJobLevel()), sizeof(level));
@@ -1007,6 +1251,31 @@ static void list_running_jobs(UAContext *ua)
          ua->send_msg(_("%6d\t%-6s\t%-20s\t%s\t%s\n"),
                       jcr->JobId, level, jcr->Job, msg, jcr->comment);
          unbash_spaces(jcr->comment);
+
+      } else if (ua->api > 1) {
+         ua->send_msg("%s", ow.get_output(OT_CLEAR,
+                         OT_START_OBJ,
+                         OT_INT32,   "jobid",     jcr->JobId,
+                         OT_JOBLEVEL,"level",     jcr->getJobLevel(),
+                         OT_JOBTYPE, "type",      jcr->getJobType(),
+                         OT_JOBSTATUS,"status",   status,
+                         OT_STRING,  "status_desc",msg,
+                         OT_STRING,  "comment",   jcr->comment,
+                         OT_SIZE,    "jobbytes",  jcr->JobBytes,
+                         OT_INT32,   "jobfiles",  jcr->JobFiles,
+                         OT_STRING,  "job",       jcr->Job,
+                         OT_STRING,  "name",      jcr->job->name(),
+                         OT_STRING,  "clientname",jcr->client?jcr->client->name():"",
+                         OT_STRING,  "fileset",   jcr->fileset?jcr->fileset->name():"",
+                         OT_STRING,  "storage",   jcr->wstore?jcr->wstore->name():"",
+                         OT_STRING,  "rstorage",  jcr->rstore?jcr->rstore->name():"",
+                         OT_UTIME,   "schedtime", jcr->sched_time,
+                         OT_UTIME,   "starttime", jcr->start_time,
+                         OT_INT32,   "priority",  jcr->JobPriority,
+                         OT_INT32,   "errors",    jcr->JobErrors,
+                         OT_END_OBJ,
+                         OT_END));
+
       } else {
          char b1[50], b2[50], b3[50];
          level[4] = 0;
@@ -1025,8 +1294,19 @@ static void list_running_jobs(UAContext *ua)
       }
    }
    endeach_jcr(jcr);
-   if (!ua->api) {
-      ua->send_msg("====\n");
+
+   if (njobs == 0) {
+      /* Note the following message is used in regress -- don't change */
+      ua->send_msg(_("No Jobs running.\n====\n"));
+      Dmsg0(200, "leave list_run_jobs()\n");
+      return;
+   } else {
+      /* display a closing header */
+      if (!ua->api) {
+         ua->send_msg("====\n");
+      } else if (ua->api > 1) {
+         ua->send_msg(ow.end_group(false));
+      }
    }
    Dmsg0(200, "leave list_run_jobs()\n");
 }
@@ -1035,6 +1315,7 @@ static void list_terminated_jobs(UAContext *ua)
 {
    char dt[MAX_TIME_LENGTH], b1[30], b2[30];
    char level[10];
+   OutputWriter ow(ua->api_opts);
 
    if (last_jobs->empty()) {
       if (!ua->api) ua->send_msg(_("No Terminated Jobs.\n"));
@@ -1044,8 +1325,10 @@ static void list_terminated_jobs(UAContext *ua)
    struct s_last_job *je;
    if (!ua->api) {
       ua->send_msg(_("\nTerminated Jobs:\n"));
-      ua->send_msg(_(" JobId  Level    Files      Bytes   Status   Finished        Name \n"));
+      ua->send_msg(_(" JobId  Level      Files    Bytes   Status   Finished        Name \n"));
       ua->send_msg(_("====================================================================\n"));
+   } else if (ua->api > 1) {
+      ua->send_msg(ow.start_group("terminated"));
    }
    foreach_dlist(je, last_jobs) {
       char JobName[MAX_NAME_LENGTH];
@@ -1067,8 +1350,10 @@ static void list_terminated_jobs(UAContext *ua)
       bstrftime_nc(dt, sizeof(dt), je->end_time);
       switch (je->JobType) {
       case JT_ADMIN:
+         bstrncpy(level, "Admin", sizeof(level));
+         break;
       case JT_RESTORE:
-         bstrncpy(level, "    ", sizeof(level));
+         bstrncpy(level, "Restore", sizeof(level));
          break;
       default:
          bstrncpy(level, level_to_str(je->JobLevel), sizeof(level));
@@ -1095,20 +1380,41 @@ static void list_terminated_jobs(UAContext *ua)
       case JS_Warnings:
          termstat = _("OK -- with warnings");
          break;
+      case JS_Incomplete:
+         termstat = _("Incomplete");
+         break;
       default:
          termstat = _("Other");
          break;
       }
       if (ua->api == 1) {
-         ua->send_msg(_("%6d\t%-6s\t%8s\t%10s\t%-7s\t%-8s\t%s\n"),
+         ua->send_msg(_("%7d\t%-6s\t%8s\t%10s\t%-7s\t%-8s\t%s\n"),
             je->JobId,
             level,
             edit_uint64_with_commas(je->JobFiles, b1),
             edit_uint64_with_suffix(je->JobBytes, b2),
             termstat,
             dt, JobName);
+      } else if (ua->api > 1) {
+         ua->send_msg("%s",
+                      ow.get_output(OT_CLEAR,
+                                    OT_START_OBJ,
+                                    OT_INT32,   "jobid",     je->JobId,
+                                    OT_JOBLEVEL,"level",     je->JobLevel,
+                                    OT_JOBTYPE, "type",      je->JobType,
+                                    OT_JOBSTATUS,"status",   je->JobStatus,
+                                    OT_STRING,  "status_desc",termstat,
+                                    OT_SIZE,    "jobbytes",  je->JobBytes,
+                                    OT_INT32,   "jobfiles",  je->JobFiles,
+                                    OT_STRING,  "job",       je->Job,
+                                    OT_UTIME,   "starttime", je->start_time,
+                                    OT_UTIME,   "endtime",   je->end_time,
+                                    OT_INT32,   "errors",    je->Errors,
+                                    OT_END_OBJ,
+                                    OT_END));
+
       } else {
-         ua->send_msg(_("%6d  %-6s %8s %10s  %-7s  %-8s %s\n"),
+         ua->send_msg(_("%6d  %-7s %8s %10s  %-7s  %-8s %s\n"),
             je->JobId,
             level,
             edit_uint64_with_commas(je->JobFiles, b1),
@@ -1119,6 +1425,8 @@ static void list_terminated_jobs(UAContext *ua)
    }
    if (!ua->api) {
       ua->send_msg(_("\n"));
+   } else if (ua->api > 1) {
+      ua->send_msg(ow.end_group(false));
    }
    unlock_last_jobs_list();
 }
