@@ -1,29 +1,20 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2012 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2017 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation and included
-   in the file LICENSE.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   This notice must be preserved when any source code is
+   conveyed and/or propagated.
 
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  *
@@ -66,6 +57,8 @@ int update_cmd(UAContext *ua, const char *cmd)
       NT_("slot"),   /* 4 */
       NT_("jobid"),  /* 5 */
       NT_("stats"),  /* 6 */
+      NT_("snap"),   /* 7 */
+      NT_("snapshot"),/* 8 */
       NULL};
 
    if (!open_client_db(ua)) {
@@ -90,6 +83,10 @@ int update_cmd(UAContext *ua, const char *cmd)
    case 6:
       update_stats(ua);
       return 1;
+   case 7:
+   case 8:
+      update_snapshot(ua);
+      return 1;
    default:
       break;
    }
@@ -99,6 +96,7 @@ int update_cmd(UAContext *ua, const char *cmd)
    add_prompt(ua, _("Pool from resource"));
    add_prompt(ua, _("Slots from autochanger"));
    add_prompt(ua, _("Long term statistics"));
+   add_prompt(ua, _("Snapshot parameters"));
    switch (do_prompt(ua, _("item"), _("Choose catalog item to update"), NULL, 0)) {
    case 0:
       update_volume(ua);
@@ -112,6 +110,9 @@ int update_cmd(UAContext *ua, const char *cmd)
    case 3:
       update_stats(ua);
       break;
+   case 4:
+      update_snapshot(ua);
+      break;
    default:
       break;
    }
@@ -174,6 +175,24 @@ static void update_volretention(UAContext *ua, char *val, MEDIA_DBR *mr)
    }
 }
 
+static void update_vol_cacheretention(UAContext *ua, char *val, MEDIA_DBR *mr)
+{
+   char ed1[150], ed2[50];
+   POOL_MEM query(PM_MESSAGE);
+   if (!duration_to_utime(val, &mr->CacheRetention)) {
+      ua->error_msg(_("Invalid cache retention period specified: %s\n"), val);
+      return;
+   }
+   Mmsg(query, "UPDATE Media SET CacheRetention=%s WHERE MediaId=%s",
+      edit_uint64(mr->CacheRetention, ed1), edit_int64(mr->MediaId,ed2));
+   if (!db_sql_query(ua->db, query.c_str(), NULL, NULL)) {
+      ua->error_msg("%s", db_strerror(ua->db));
+   } else {
+      ua->info_msg(_("New Cache Retention period is: %s\n"),
+         edit_utime(mr->CacheRetention, ed1, sizeof(ed1)));
+   }
+}
+
 static void update_voluseduration(UAContext *ua, char *val, MEDIA_DBR *mr)
 {
    char ed1[150], ed2[50];
@@ -285,7 +304,7 @@ static void update_volslot(UAContext *ua, char *val, MEDIA_DBR *mr)
 
    memset(&pr, 0, sizeof(POOL_DBR));
    pr.PoolId = mr->PoolId;
-   if (!db_get_pool_record(ua->jcr, ua->db, &pr)) {
+   if (!db_get_pool_numvols(ua->jcr, ua->db, &pr)) {
       ua->error_msg("%s", db_strerror(ua->db));
       return;
    }
@@ -346,7 +365,8 @@ void update_vol_recyclepool(UAContext *ua, char *val, MEDIA_DBR *mr)
 {
    POOL_DBR pr;
    POOL_MEM query(PM_MESSAGE);
-   char ed1[50], ed2[50], *poolname;
+   char ed1[50], ed2[50];
+   const char *poolname;
 
    if(val && *val) { /* update volume recyclepool="Scratch" */
      /* If a pool name is given, look up the PoolId */
@@ -385,7 +405,7 @@ static void update_vol_from_pool(UAContext *ua, MEDIA_DBR *mr)
 
    memset(&pr, 0, sizeof(pr));
    pr.PoolId = mr->PoolId;
-   if (!db_get_pool_record(ua->jcr, ua->db, &pr) ||
+   if (!db_get_pool_numvols(ua->jcr, ua->db, &pr) ||
        !acl_access_ok(ua, Pool_ACL, pr.Name)) {
       return;
    }
@@ -402,7 +422,7 @@ static void update_vol_from_pool(UAContext *ua, MEDIA_DBR *mr)
  * Refresh the Volume information from the Pool record
  *   for all Volumes
  */
-static void update_all_vols_from_pool(UAContext *ua, const char *pool_name) 
+static void update_all_vols_from_pool(UAContext *ua, const char *pool_name)
 {
    POOL_DBR pr;
    MEDIA_DBR mr;
@@ -431,7 +451,7 @@ static void update_all_vols(UAContext *ua)
    MEDIA_DBR mr;
 
    memset(&pr, 0, sizeof(pr));
-   
+
    if (!db_get_pool_ids(ua->jcr, ua->db, &num_pools, &ids)) {
       ua->error_msg(_("Error obtaining pool ids. ERR=%s\n"), db_strerror(ua->db));
       return;
@@ -439,7 +459,7 @@ static void update_all_vols(UAContext *ua)
 
    for (i=0; i<num_pools; i++) {
       pr.PoolId = ids[i];
-      if (!db_get_pool_record(ua->jcr, ua->db, &pr)) { /* ***FIXME*** use acl? */
+      if (!db_get_pool_numvols(ua->jcr, ua->db, &pr)) { /* ***FIXME*** use acl? */
          ua->warning_msg(_("Updating all pools, but skipped PoolId=%d. ERR=%s\n"), db_strerror(ua->db));
          continue;
       }
@@ -481,13 +501,13 @@ static void update_vol_actiononpurge(UAContext *ua, char *val, MEDIA_DBR *mr)
    } else {
       mr->ActionOnPurge = 0;
    }
-   
+
    set_storageid_in_mr(NULL, mr);
    if (!db_update_media_record(ua->jcr, ua->db, mr)) {
       ua->error_msg(_("Error updating media record ActionOnPurge: ERR=%s"),
                     db_strerror(ua->db));
    } else {
-      ua->info_msg(_("New ActionOnPurge is: %s\n"), 
+      ua->info_msg(_("New ActionOnPurge is: %s\n"),
                    action_on_purge_to_string(mr->ActionOnPurge, ret));
    }
 }
@@ -520,22 +540,32 @@ static int update_volume(UAContext *ua)
       NT_("InChanger"),                /* 7 */
       NT_("Slot"),                     /* 8 */
       NT_("Pool"),                     /* 9 */
-      NT_("FromPool"),                 /* 10 */
+      NT_("FromPool"),                 /* 10 !!! see below !!! */
       NT_("AllFromPool"),              /* 11 !!! see below !!! */
       NT_("Enabled"),                  /* 12 */
       NT_("RecyclePool"),              /* 13 */
       NT_("ActionOnPurge"),            /* 14 */
+      NT_("FromAllPools"),             /* 15 !!! see bellow !!! */
+      NT_("CacheRetention"),           /* 16 */
       NULL };
 
-#define AllFromPool 11               /* keep this updated with above */
+#define FromPool     10              /* keep this updated */
+#define AllFromPool  11              /* keep this updated with above */
+#define FromAllPools 15              /* keep this updated */
 
    for (i=0; kw[i]; i++) {
       int j;
       POOL_DBR pr;
+      /* No argv with these parameters */
+      if (i == FromPool || i == FromAllPools) {
+         j = find_arg(ua, kw[i]);
 
-      if ((j=find_arg_with_value(ua, kw[i])) > 0) {
-         /* If all from pool don't select a media record */
-         if (i != AllFromPool && !select_media_dbr(ua, &mr)) {
+      } else {
+         j = find_arg_with_value(ua, kw[i]);
+      }
+      if (j > 0) {
+         /* If all from pool/from all pools don't select a media record */
+         if (i != AllFromPool && i != FromAllPools && !select_media_dbr(ua, &mr)) {
             return 0;
          }
          switch (i) {
@@ -569,7 +599,7 @@ static int update_volume(UAContext *ua)
          case 9:
             memset(&pr, 0, sizeof(POOL_DBR));
             pr.PoolId = mr.PoolId;
-            if (!db_get_pool_record(ua->jcr, ua->db, &pr)) {
+            if (!db_get_pool_numvols(ua->jcr, ua->db, &pr)) {
                ua->error_msg("%s", db_strerror(ua->db));
                break;
             }
@@ -590,17 +620,17 @@ static int update_volume(UAContext *ua)
          case 14:
             update_vol_actiononpurge(ua, ua->argv[j], &mr);
             break;
+         case 15:
+            update_all_vols(ua);
+            break;
+         case 16:
+            update_vol_cacheretention(ua, ua->argv[j], &mr);
+            break;
          }
          done = true;
       }
    }
 
-   /* Allow user to simply update all volumes */
-   if (find_arg(ua, NT_("fromallpools")) > 0) {
-      update_all_vols(ua);
-      return 1;
-   }
-
    for ( ; !done; ) {
       start_prompt(ua, _("Parameters to modify:\n"));
       add_prompt(ua, _("Volume Status"));              /* 0 */
@@ -620,12 +650,13 @@ static int update_volume(UAContext *ua)
       add_prompt(ua, _("Enabled")),                    /* 14 */
       add_prompt(ua, _("RecyclePool")),                /* 15 */
       add_prompt(ua, _("Action On Purge")),            /* 16 */
-      add_prompt(ua, _("Done"));                       /* 17 */
-      i = do_prompt(ua, "", _("Select parameter to modify"), NULL, 0);  
+      add_prompt(ua, _("Cache Retention")),            /* 17 */
+      add_prompt(ua, _("Done"));                       /* 18 */
+      i = do_prompt(ua, "", _("Select parameter to modify"), NULL, 0);
 
       /* For All Volumes, All Volumes from Pool, and Done, we don't need
            * a Volume record */
-      if ( i != 12 && i != 13 && i != 17) {
+      if ( i != 12 && i != 13 && i != 18) {
          if (!select_media_dbr(ua, &mr)) {  /* Get Volume record */
             return 0;
          }
@@ -636,7 +667,7 @@ static int update_volume(UAContext *ua)
          /* Modify Volume Status */
          ua->info_msg(_("Current Volume status is: %s\n"), mr.VolStatus);
          start_prompt(ua, _("Possible Values are:\n"));
-         add_prompt(ua, NT_("Append")); 
+         add_prompt(ua, NT_("Append"));
          add_prompt(ua, NT_("Archive"));
          add_prompt(ua, NT_("Disabled"));
          add_prompt(ua, NT_("Full"));
@@ -710,7 +741,7 @@ static int update_volume(UAContext *ua)
          }
          update_volslot(ua, ua->cmd, &mr);
          break;
-         
+
       case 8:                         /* InChanger */
          ua->info_msg(_("Current InChanger flag is: %d\n"), mr.InChanger);
          bsnprintf(buf, sizeof(buf), _("Set InChanger flag for Volume \"%s\": yes/no: "),
@@ -761,7 +792,7 @@ static int update_volume(UAContext *ua)
       case 10:                        /* Volume's Pool */
          memset(&pr, 0, sizeof(POOL_DBR));
          pr.PoolId = mr.PoolId;
-         if (!db_get_pool_record(ua->jcr, ua->db, &pr)) {
+         if (!db_get_pool_numvols(ua->jcr, ua->db, &pr)) {
             ua->error_msg("%s", db_strerror(ua->db));
             return 0;
          }
@@ -791,22 +822,13 @@ static int update_volume(UAContext *ua)
          if (!get_cmd(ua, _("Enter new Enabled: "))) {
             return 0;
          }
-         if (strcasecmp(ua->cmd, "yes") == 0 || strcasecmp(ua->cmd, "true") == 0) {
-            mr.Enabled = 1;
-         } else if (strcasecmp(ua->cmd, "no") == 0 || strcasecmp(ua->cmd, "false") == 0) {
-            mr.Enabled = 0;
-         } else if (strcasecmp(ua->cmd, "archived") == 0) { 
-            mr.Enabled = 2;
-         } else {
-            mr.Enabled = atoi(ua->cmd);
-         }
          update_volenabled(ua, ua->cmd, &mr);
          break;
 
       case 15:
          memset(&pr, 0, sizeof(POOL_DBR));
          pr.PoolId = mr.RecyclePoolId;
-         if (db_get_pool_record(ua->jcr, ua->db, &pr)) {
+         if (db_get_pool_numvols(ua->jcr, ua->db, &pr)) {
             ua->info_msg(_("Current RecyclePool is: %s\n"), pr.Name);
          } else {
             ua->info_msg(_("No current RecyclePool\n"));
@@ -819,7 +841,7 @@ static int update_volume(UAContext *ua)
 
       case 16:
          pm_strcpy(ret, "");
-         ua->info_msg(_("Current ActionOnPurge is: %s\n"), 
+         ua->info_msg(_("Current ActionOnPurge is: %s\n"),
                       action_on_purge_to_string(mr.ActionOnPurge, ret));
          if (!get_cmd(ua, _("Enter new ActionOnPurge (one of: Truncate, None): "))) {
             return 0;
@@ -828,6 +850,16 @@ static int update_volume(UAContext *ua)
          update_vol_actiononpurge(ua, ua->cmd, &mr);
          break;
 
+      case 17:
+         pm_strcpy(ret, "");
+         ua->info_msg(_("Current Cache Retention period is: %s\n"),
+                      edit_utime(mr.CacheRetention, ed1, sizeof(ed1)));
+         if (!get_cmd(ua, _("Enter Cache Retention period: "))) {
+            return 0;
+         }
+         update_vol_cacheretention(ua, ua->cmd, &mr);
+         break;
+
       default:                        /* Done or error */
          ua->info_msg(_("Selection terminated.\n"));
          return 1;
@@ -899,17 +931,19 @@ static bool update_pool(UAContext *ua)
  */
 static bool update_job(UAContext *ua)
 {
-   int i;
-   char ed1[50], ed2[50];
+   int i, priority=0;
+   char ed1[50], ed2[50], ed3[50];
    POOL_MEM cmd(PM_MESSAGE);
    JOB_DBR jr;
    CLIENT_DBR cr;
+   JCR *jcr;
    utime_t StartTime;
    char *client_name = NULL;
    char *start_time = NULL;
    const char *kw[] = {
       NT_("starttime"),                   /* 0 */
       NT_("client"),                      /* 1 */
+      NT_("priority"),                    /* 2 */
       NULL };
 
    Dmsg1(200, "cmd=%s\n", ua->cmd);
@@ -921,11 +955,18 @@ static bool update_job(UAContext *ua)
    memset(&jr, 0, sizeof(jr));
    memset(&cr, 0, sizeof(cr));
    jr.JobId = str_to_int64(ua->argv[i]);
+   if (jr.JobId == 0) {
+      ua->error_msg("Bad jobid\n");
+      return false;
+   }
    if (!db_get_job_record(ua->jcr, ua->db, &jr)) {
       ua->error_msg("%s", db_strerror(ua->db));
       return false;
    }
-
+   if (!acl_access_ok(ua, Job_ACL, jr.Name)) {
+      ua->error_msg(_("Update failed. Job not authorized on this console\n"));
+      return false;
+   }
    for (i=0; kw[i]; i++) {
       int j;
       if ((j=find_arg_with_value(ua, kw[i])) >= 0) {
@@ -936,15 +977,32 @@ static bool update_job(UAContext *ua)
          case 1:                         /* Client name */
             client_name = ua->argv[j];
             break;
+         case 2:
+            priority = str_to_int64(ua->argv[j]);
+            break;
          }
       }
    }
-   if (!client_name && !start_time) {
-      ua->error_msg(_("Neither Client nor StartTime specified.\n"));
+   if (!client_name && !start_time && !priority) {
+      ua->error_msg(_("Neither Client, StartTime or Priority specified.\n"));
       return 0;
    }
+   if (priority > 0) {
+      foreach_jcr(jcr) {
+         if (jcr->JobId == jr.JobId) {
+            int old = jcr->JobPriority;
+            jcr->JobPriority = priority;
+            free_jcr(jcr);
+            ua->send_msg(_("Priority updated for running job \"%s\" from %d to %d\n"), jr.Job, old, priority);
+            return true;
+         }
+      }
+      endeach_jcr(jcr);
+      ua->error_msg(_("Job not found.\n"));
+      return true;
+   }
    if (client_name) {
-      if (!get_client_dbr(ua, &cr)) {
+      if (!get_client_dbr(ua, &cr, JT_BACKUP_RESTORE)) {
          return false;
       }
       jr.ClientId = cr.ClientId;
@@ -958,7 +1016,7 @@ static bool update_job(UAContext *ua)
          return false;
       }
       delta_start = StartTime - jr.StartTime;
-      Dmsg3(200, "ST=%lld jr.ST=%lld delta=%lld\n", StartTime, 
+      Dmsg3(200, "ST=%lld jr.ST=%lld delta=%lld\n", StartTime,
             (utime_t)jr.StartTime, delta_start);
       jr.StartTime = (time_t)StartTime;
       jr.SchedTime += (time_t)delta_start;
@@ -970,13 +1028,13 @@ static bool update_job(UAContext *ua)
       bstrutime(jr.cEndTime, sizeof(jr.cEndTime), jr.EndTime);
    }
    Mmsg(cmd, "UPDATE Job SET ClientId=%s,StartTime='%s',SchedTime='%s',"
-             "EndTime='%s',JobTDate=%s WHERE JobId=%s", 
-             edit_int64(jr.ClientId, ed1), 
+             "EndTime='%s',JobTDate=%s WHERE JobId=%s",
+             edit_int64(jr.ClientId, ed1),
              jr.cStartTime,
              jr.cSchedTime,
              jr.cEndTime,
-             edit_uint64(jr.JobTDate, ed1), 
-             edit_int64(jr.JobId, ed2));
+             edit_uint64(jr.JobTDate, ed2),
+             edit_int64(jr.JobId, ed3));
    if (!db_sql_query(ua->db, cmd.c_str(), NULL, NULL)) {
       ua->error_msg("%s", db_strerror(ua->db));
       return false;
