@@ -1,32 +1,22 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2011 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2017 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation and included
-   in the file LICENSE.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   This notice must be preserved when any source code is 
+   conveyed and/or propagated.
 
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
- *
  *   Bacula Director -- verify.c -- responsible for running file verification
  *
  *     Kern Sibbald, October MM
@@ -37,7 +27,6 @@
  *       to do the verify.
  *     When the File daemon sends the attributes, compare them to
  *       what is in the DB.
- *
  */
 
 
@@ -47,25 +36,22 @@
 
 /* Commands sent to File daemon */
 static char verifycmd[] = "verify level=%s\n";
-static char storaddr[]  = "storage address=%s port=%d ssl=0 Authorization=%s\n";
 
 /* Responses received from File daemon */
 static char OKverify[]  = "2000 OK verify\n";
-static char OKstore[]   = "2000 OK storage\n";
 
-/* Responses received from the Storage daemon */
+/* Commands received from Storage daemon */
 static char OKbootstrap[] = "3000 OK bootstrap\n";
 
 /* Forward referenced functions */
 static void prt_fname(JCR *jcr);
 static int missing_handler(void *ctx, int num_fields, char **row);
 
-
-/* 
+/*
  * Called here before the job is run to do the job
  *   specific setup.
  */
-bool do_verify_init(JCR *jcr) 
+bool do_verify_init(JCR *jcr)
 {
    if (!allow_duplicate_job(jcr)) {
       return false;
@@ -77,11 +63,10 @@ bool do_verify_init(JCR *jcr)
       free_rstorage(jcr);
       free_wstorage(jcr);
       break;
+   case L_VERIFY_DATA:
    case L_VERIFY_VOLUME_TO_CATALOG:
       free_wstorage(jcr);
       break;
-   case L_VERIFY_DATA:
-      break;
    default:
       Jmsg2(jcr, M_FATAL, 0, _("Unimplemented Verify level %d(%c)\n"), jcr->getJobLevel(),
          jcr->getJobLevel());
@@ -100,11 +85,13 @@ bool do_verify_init(JCR *jcr)
 bool do_verify(JCR *jcr)
 {
    const char *level;
-   BSOCK   *fd;
+   BSOCK *fd, *sd;
    int stat;
    char ed1[100];
    JOB_DBR jr;
    JobId_t verify_jobid = 0;
+   char *store_address;
+   uint32_t store_port;
    const char *Name;
 
    free_wstorage(jcr);                   /* we don't write */
@@ -125,11 +112,13 @@ bool do_verify(JCR *jcr)
     */
    if (jcr->getJobLevel() == L_VERIFY_CATALOG ||
        jcr->getJobLevel() == L_VERIFY_VOLUME_TO_CATALOG ||
-       jcr->getJobLevel() == L_VERIFY_DISK_TO_CATALOG) {
+       jcr->getJobLevel() == L_VERIFY_DISK_TO_CATALOG ||
+       jcr->getJobLevel() == L_VERIFY_DATA) {
       memcpy(&jr, &jcr->jr, sizeof(jr));
       if (jcr->verify_job &&
           (jcr->getJobLevel() == L_VERIFY_VOLUME_TO_CATALOG ||
-           jcr->getJobLevel() == L_VERIFY_DISK_TO_CATALOG)) {
+           jcr->getJobLevel() == L_VERIFY_DISK_TO_CATALOG   ||
+           jcr->getJobLevel() == L_VERIFY_DATA)) {
          Name = jcr->verify_job->name();
       } else {
          Name = NULL;
@@ -164,7 +153,8 @@ bool do_verify(JCR *jcr)
     */
    if (jcr->getJobLevel() == L_VERIFY_CATALOG ||
        jcr->getJobLevel() == L_VERIFY_VOLUME_TO_CATALOG ||
-       jcr->getJobLevel() == L_VERIFY_DISK_TO_CATALOG) {
+       jcr->getJobLevel() == L_VERIFY_DISK_TO_CATALOG ||
+       jcr->getJobLevel() == L_VERIFY_DATA) {
       jcr->previous_jr.JobId = verify_jobid;
       if (!db_get_job_record(jcr, jcr->db, &jcr->previous_jr)) {
          Jmsg(jcr, M_FATAL, 0, _("Could not get job record for previous Job. ERR=%s"),
@@ -187,7 +177,7 @@ bool do_verify(JCR *jcr)
     *   create a dummy authorization key (passed to
     *   File daemon but not used).
     */
-   if (jcr->getJobLevel() == L_VERIFY_VOLUME_TO_CATALOG) {
+   if (jcr->getJobLevel() == L_VERIFY_VOLUME_TO_CATALOG || jcr->getJobLevel() == L_VERIFY_DATA) {
       int stat;
       /*
        * Note: negative status is an error, zero status, means
@@ -205,10 +195,43 @@ bool do_verify(JCR *jcr)
       jcr->sd_auth_key = bstrdup("dummy");    /* dummy Storage daemon key */
    }
 
+   /* Pass the original fileset to the client */
+   if (jcr->getJobLevel() == L_VERIFY_DATA) {
+      FILESET_DBR fdbr;
+      memset(&fdbr, 0, sizeof(fdbr));
+      fdbr.FileSetId = jcr->previous_jr.FileSetId;
+      if (!db_get_fileset_record(jcr, jcr->db, &fdbr)) {
+         Jmsg(jcr, M_FATAL, 0,
+              _("Could not get fileset record from previous Job. ERR=%s"),
+              db_strerror(jcr->db));
+         return false;
+      }
+
+      jcr->fileset = (FILESET *)GetResWithName(R_FILESET, fdbr.FileSet);
+      if (!jcr->fileset) {
+         if (jcr->verify_job) {
+            jcr->fileset = jcr->verify_job->fileset;
+            Jmsg(jcr, M_WARNING, 0,
+                 _("Could not find FileSet resource \"%s\" from previous Job\n"),
+                 fdbr.FileSet);
+            Jmsg(jcr, M_INFO, 0,
+                 _("Using FileSet \"%\"\n"), jcr->fileset->name());
+
+         } else {
+            Jmsg(jcr, M_FATAL, 0,
+                 _("Could not get FileSet resource for verify Job."));
+            return false;
+         }
+      }
+      Dmsg1(50, "FileSet = %s\n", jcr->fileset->name());
+   }
+
+   /* Pass the current fileset to the client */
    if (jcr->getJobLevel() == L_VERIFY_DISK_TO_CATALOG && jcr->verify_job) {
       jcr->fileset = jcr->verify_job->fileset;
    }
-   Dmsg2(100, "ClientId=%u JobLevel=%c\n", jcr->previous_jr.ClientId, jcr->getJobLevel());
+   Dmsg2(100, "ClientId=%u JobLevel=%c\n",
+         jcr->previous_jr.ClientId, jcr->getJobLevel());
 
    if (!db_update_job_start_record(jcr, jcr->db, &jcr->jr)) {
       Jmsg(jcr, M_FATAL, 0, "%s", db_strerror(jcr->db));
@@ -219,8 +242,9 @@ bool do_verify(JCR *jcr)
    Jmsg(jcr, M_INFO, 0, _("Start Verify JobId=%s Level=%s Job=%s\n"),
       edit_uint64(jcr->JobId, ed1), level_to_str(jcr->getJobLevel()), jcr->Job);
 
-   if (jcr->getJobLevel() == L_VERIFY_VOLUME_TO_CATALOG) {
-      BSOCK *sd;
+   if (jcr->getJobLevel() == L_VERIFY_VOLUME_TO_CATALOG ||
+       jcr->getJobLevel() == L_VERIFY_DATA)
+   {
       /*
        * Start conversation with Storage daemon
        */
@@ -235,6 +259,7 @@ bool do_verify(JCR *jcr)
          return false;
       }
       sd = jcr->store_bsock;
+      jcr->sd_calls_client = jcr->client->sd_calls_client;
       /*
        * Send the bootstrap file -- what Volumes/files to restore
        */
@@ -242,17 +267,11 @@ bool do_verify(JCR *jcr)
           !response(jcr, sd, OKbootstrap, "Bootstrap", DISPLAY_ERROR)) {
          goto bail_out;
       }
-      if (!sd->fsend("run")) {
-         return false;
+      if (!jcr->sd_calls_client) {
+         if (!run_storage_and_start_message_thread(jcr, sd)) {
+            return false;
+         }
       }
-      /*
-       * Now start a Storage daemon message thread
-       */
-      if (!start_storage_daemon_message_thread(jcr)) {
-         return false;
-      }
-      Dmsg0(50, "Storage daemon connection OK\n");
-
    }
    /*
     * OK, now connect to the File daemon
@@ -288,16 +307,38 @@ bool do_verify(JCR *jcr)
    case L_VERIFY_CATALOG:
       level = "catalog";
       break;
+   case L_VERIFY_DATA:
+      send_accurate_current_files(jcr);
+      /* Fall-through wanted */
    case L_VERIFY_VOLUME_TO_CATALOG:
-      /*
-       * send Storage daemon address to the File daemon
-       */
-      if (jcr->rstore->SDDport == 0) {
-         jcr->rstore->SDDport = jcr->rstore->SDport;
+      if (jcr->sd_calls_client) {
+         if (jcr->FDVersion < 10) {
+            Jmsg(jcr, M_FATAL, 0, _("The File daemon does not support SDCallsClient.\n"));
+            goto bail_out;
+         }
+
+         if (!send_client_addr_to_sd(jcr)) {
+            goto bail_out;
+         }
+
+         if (!run_storage_and_start_message_thread(jcr, jcr->store_bsock)) {
+            return false;
+         }
+         store_address = jcr->rstore->address;  /* dummy */
+         store_port = 0;           /* flag that SD calls FD */
+      } else {
+         /*
+          * send Storage daemon address to the File daemon
+          */
+         if (jcr->rstore->SDDport == 0) {
+            jcr->rstore->SDDport = jcr->rstore->SDport;
+         }
+
+         store_address = get_storage_address(jcr->client, jcr->rstore);
+         store_port = jcr->rstore->SDDport;
       }
-      bnet_fsend(fd, storaddr, jcr->rstore->address, 
-                 jcr->rstore->SDDport, jcr->sd_auth_key);
-      if (!response(jcr, fd, OKstore, "Storage", DISPLAY_ERROR)) {
+
+      if (!send_store_addr_to_fd(jcr, jcr->rstore, store_address, store_port)) {
          goto bail_out;
       }
 
@@ -305,18 +346,19 @@ bool do_verify(JCR *jcr)
          Jmsg0(jcr, M_FATAL, 0, _("Deprecated feature ... use bootstrap.\n"));
          goto bail_out;
       }
-
-      level = "volume";
-      break;
-   case L_VERIFY_DATA:
-      level = "data";
+      if (jcr->getJobLevel() == L_VERIFY_VOLUME_TO_CATALOG) {
+         level = "volume";
+      } else {
+         level = "data";
+      }
       break;
    case L_VERIFY_DISK_TO_CATALOG:
       level="disk_to_catalog";
       break;
    default:
-      Jmsg2(jcr, M_FATAL, 0, _("Unimplemented Verify level %d(%c)\n"), jcr->getJobLevel(),
-         jcr->getJobLevel());
+      Jmsg2(jcr, M_FATAL, 0, _("Unimplemented Verify level %d(%c)\n"),
+            jcr->getJobLevel(),
+            jcr->getJobLevel());
       goto bail_out;
    }
 
@@ -368,6 +410,10 @@ bool do_verify(JCR *jcr)
       db_write_batch_file_records(jcr);
       break;
 
+   case L_VERIFY_DATA:
+      /* Nothing special to do */
+      bget_dirmsg(fd);          /* eat EOD */
+      break;
    default:
       Jmsg1(jcr, M_FATAL, 0, _("Unimplemented verify level %d\n"), jcr->getJobLevel());
       goto bail_out;
@@ -389,17 +435,18 @@ bail_out:
 void verify_cleanup(JCR *jcr, int TermCode)
 {
    char sdt[50], edt[50];
-   char ec1[30], ec2[30];
+   char ec1[30], ec2[30], elapsed[50];
    char term_code[100], fd_term_msg[100], sd_term_msg[100];
    const char *term_msg;
    int msg_type;
+   utime_t RunTime;
    const char *Name;
 
 // Dmsg1(100, "Enter verify_cleanup() TermCod=%d\n", TermCode);
 
    Dmsg3(900, "JobLevel=%c Expected=%u JobFiles=%u\n", jcr->getJobLevel(),
       jcr->ExpectedFiles, jcr->JobFiles);
-   if (jcr->getJobLevel() == L_VERIFY_VOLUME_TO_CATALOG &&
+   if ((jcr->getJobLevel() == L_VERIFY_VOLUME_TO_CATALOG || jcr->getJobLevel() == L_VERIFY_DATA) &&
        jcr->ExpectedFiles != jcr->JobFiles) {
       TermCode = JS_ErrorTerminated;
    }
@@ -418,7 +465,11 @@ void verify_cleanup(JCR *jcr, int TermCode)
    msg_type = M_INFO;                 /* by default INFO message */
    switch (TermCode) {
    case JS_Terminated:
-      term_msg = _("Verify OK");
+      if (jcr->JobErrors || jcr->SDErrors) {
+         term_msg = _("Verify OK -- with warnings");
+      } else {
+         term_msg = _("Verify OK");
+      }
       break;
    case JS_FatalError:
    case JS_ErrorTerminated:
@@ -442,6 +493,7 @@ void verify_cleanup(JCR *jcr, int TermCode)
    }
    bstrftimes(sdt, sizeof(sdt), jcr->jr.StartTime);
    bstrftimes(edt, sizeof(edt), jcr->jr.EndTime);
+   RunTime = jcr->jr.EndTime - jcr->jr.StartTime;
    if (jcr->verify_job) {
       Name = jcr->verify_job->hdr.name;
    } else {
@@ -449,7 +501,11 @@ void verify_cleanup(JCR *jcr, int TermCode)
    }
 
    jobstatus_to_ascii(jcr->FDJobStatus, fd_term_msg, sizeof(fd_term_msg));
-   if (jcr->getJobLevel() == L_VERIFY_VOLUME_TO_CATALOG) {
+   if (jcr->getJobLevel() == L_VERIFY_VOLUME_TO_CATALOG || jcr->getJobLevel() == L_VERIFY_DATA) {
+      const char *accurate = "yes";
+      if (jcr->is_JobLevel(L_VERIFY_DATA)) {
+         accurate = jcr->accurate ? "yes": "no";
+      }
       jobstatus_to_ascii(jcr->SDJobStatus, sd_term_msg, sizeof(sd_term_msg));
       Jmsg(jcr, msg_type, 0, _("%s %s %s (%s):\n"
 "  Build OS:               %s %s %s\n"
@@ -462,9 +518,12 @@ void verify_cleanup(JCR *jcr, int TermCode)
 "  Verify Job:             %s\n"
 "  Start time:             %s\n"
 "  End time:               %s\n"
+"  Elapsed time:           %s\n"
+"  Accurate:               %s\n"
 "  Files Expected:         %s\n"
 "  Files Examined:         %s\n"
 "  Non-fatal FD errors:    %d\n"
+"  SD Errors:              %d\n"
 "  FD termination status:  %s\n"
 "  SD termination status:  %s\n"
 "  Termination:            %s\n\n"),
@@ -479,15 +538,18 @@ void verify_cleanup(JCR *jcr, int TermCode)
            Name,
            sdt,
            edt,
+           edit_utime(RunTime, elapsed, sizeof(elapsed)),
+           accurate,
            edit_uint64_with_commas(jcr->ExpectedFiles, ec1),
            edit_uint64_with_commas(jcr->JobFiles, ec2),
            jcr->JobErrors,
+           jcr->SDErrors,
            fd_term_msg,
            sd_term_msg,
            term_msg);
    } else {
       Jmsg(jcr, msg_type, 0, _("%s %s %s (%s):\n"
-"  Build:                  %s %s %s\n"
+"  Build OS:               %s %s %s\n"
 "  JobId:                  %d\n"
 "  Job:                    %s\n"
 "  FileSet:                %s\n"
@@ -497,6 +559,7 @@ void verify_cleanup(JCR *jcr, int TermCode)
 "  Verify Job:             %s\n"
 "  Start time:             %s\n"
 "  End time:               %s\n"
+"  Elapsed time:           %s\n"
 "  Files Examined:         %s\n"
 "  Non-fatal FD errors:    %d\n"
 "  FD termination status:  %s\n"
@@ -512,6 +575,7 @@ void verify_cleanup(JCR *jcr, int TermCode)
            Name,
            sdt,
            edt,
+           edit_utime(RunTime, elapsed, sizeof(elapsed)),
            edit_uint64_with_commas(jcr->JobFiles, ec1),
            jcr->JobErrors,
            fd_term_msg,
@@ -552,22 +616,27 @@ void get_attributes_and_compare_to_catalog(JCR *jcr, JobId_t JobId)
     *   Link name  ???
     */
    while ((n=bget_dirmsg(fd)) >= 0 && !job_canceled(jcr)) {
-      int stream;
+      int32_t stream, full_stream;
       char *attr, *p, *fn;
       char Opts_Digest[MAXSTRING];        /* Verify Opts or MD5/SHA1 digest */
 
       if (job_canceled(jcr)) {
-         goto bail_out;
+         free_pool_memory(fname);
+         return;
       }
       fname = check_pool_memory_size(fname, fd->msglen);
       jcr->fname = check_pool_memory_size(jcr->fname, fd->msglen);
       Dmsg1(200, "Atts+Digest=%s\n", fd->msg);
-      if ((len = sscanf(fd->msg, "%ld %d %100s", &file_index, &stream,
+      if ((len = sscanf(fd->msg, "%ld %d %100s", &file_index, &full_stream,
             fname)) != 3) {
          Jmsg3(jcr, M_FATAL, 0, _("bird<filed: bad attributes, expected 3 fields got %d\n"
 " mslen=%d msg=%s\n"), len, fd->msglen, fd->msg);
-         goto bail_out;
+         free_pool_memory(fname);
+         return;
       }
+      stream = full_stream & STREAMMASK_TYPE;
+      Dmsg4(30, "Got hdr: FilInx=%d FullStream=%d Stream=%d fname=%s.\n", file_index, full_stream, stream, fname);
+
       /*
        * We read the Options or Signature into fname
        *  to prevent overrun, now copy it to proper location.
@@ -737,7 +806,8 @@ void get_attributes_and_compare_to_catalog(JCR *jcr, JobId_t JobId)
          if (jcr->FileIndex != (uint32_t)file_index) {
             Jmsg2(jcr, M_FATAL, 0, _("MD5/SHA1 index %d not same as attributes %d\n"),
                file_index, jcr->FileIndex);
-            goto bail_out;
+            free_pool_memory(fname);
+            return;
          }
          if (do_Digest != CRYPTO_DIGEST_NONE) {
             db_escape_string(jcr, jcr->db, buf, Opts_Digest, strlen(Opts_Digest));
@@ -752,11 +822,12 @@ void get_attributes_and_compare_to_catalog(JCR *jcr, JobId_t JobId)
       }
       jcr->JobFiles = file_index;
    }
-   if (is_bnet_error(fd)) {
+   if (fd->is_error()) {
       berrno be;
       Jmsg2(jcr, M_FATAL, 0, _("bdird<filed: bad attributes from filed n=%d : %s\n"),
                         n, be.bstrerror());
-      goto bail_out;
+      free_pool_memory(fname);
+      return;
    }
 
    /* Now find all the files that are missing -- i.e. all files in
@@ -764,18 +835,16 @@ void get_attributes_and_compare_to_catalog(JCR *jcr, JobId_t JobId)
     */
    jcr->fn_printed = false;
    bsnprintf(buf, sizeof(buf),
-      "SELECT Path.Path,Filename.Name FROM File,Path,Filename "
-      "WHERE File.JobId=%d AND File.FileIndex > 0 "
-      "AND File.MarkId!=%d AND File.PathId=Path.PathId "
-      "AND File.FilenameId=Filename.FilenameId",
-         JobId, jcr->JobId);
+    "SELECT Path.Path,Filename.Name FROM File,Path,Filename "
+    "WHERE File.JobId=%d AND File.FileIndex > 0 "
+    "AND File.MarkId!=%d AND File.PathId=Path.PathId "
+    "AND File.FilenameId=Filename.FilenameId",
+       JobId, jcr->JobId);
    /* missing_handler is called for each file found */
    db_sql_query(jcr->db, buf, missing_handler, (void *)jcr);
    if (jcr->fn_printed) {
       jcr->setJobStatus(JS_Differences);
    }
-
-bail_out:
    free_pool_memory(fname);
 }
 
