@@ -1,37 +1,26 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2011 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2015 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation and included
-   in the file LICENSE.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   This notice must be preserved when any source code is 
+   conveyed and/or propagated.
 
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  *  Create a file, and reset the modes
  *
  *    Kern Sibbald, November MM
  *
- *   Version $Id$
- *
  */
 
 #include "bacula.h"
@@ -46,6 +35,13 @@
 #define O_CTG 0
 #endif
 
+#ifndef O_EXCL
+#define O_EXCL 0
+#endif
+#ifndef O_NOFOLLOW
+#define O_NOFOLLOW 0
+#endif
+
 static int separate_path_and_file(JCR *jcr, char *fname, char *ofile);
 static int path_already_seen(JCR *jcr, char *path, int pnl);
 
@@ -153,7 +149,7 @@ int create_file(JCR *jcr, ATTR *attr, BFILE *bfd, int replace)
    case FT_SPEC:                      /* fifo, ... to be backed up */
    case FT_REGE:                      /* empty file */
    case FT_REG:                       /* regular file */
-      /* 
+      /*
        * Note, we do not delete FT_RAW because these are device files
        *  or FIFOs that should already exist. If we blow it away,
        *  we may blow away a FIFO that is being used to read the
@@ -166,7 +162,7 @@ int create_file(JCR *jcr, ATTR *attr, BFILE *bfd, int replace)
             berrno be;
             Qmsg(jcr, M_ERROR, 0, _("File %s already exists and could not be replaced. ERR=%s.\n"),
                attr->ofname, be.bstrerror());
-            /* Continue despite error */
+            return CF_ERROR;
          }
       }
       /*
@@ -210,7 +206,7 @@ int create_file(JCR *jcr, ATTR *attr, BFILE *bfd, int replace)
       case FT_REGE:
       case FT_REG:
          Dmsg1(100, "Create=%s\n", attr->ofname);
-         flags =  O_WRONLY | O_CREAT | O_TRUNC | O_BINARY; /*  O_NOFOLLOW; */
+         flags =  O_WRONLY | O_CREAT | O_BINARY | O_EXCL;
          if (IS_CTG(attr->statp.st_mode)) {
             flags |= O_CTG;              /* set contiguous bit if needed */
          }
@@ -218,8 +214,8 @@ int create_file(JCR *jcr, ATTR *attr, BFILE *bfd, int replace)
             Qmsg1(jcr, M_ERROR, 0, _("bpkt already open fid=%d\n"), bfd->fid);
             bclose(bfd);
          }
-      
 
+         set_fattrs(bfd, &attr->statp);
          if ((bopen(bfd, attr->ofname, flags, S_IRUSR | S_IWUSR)) < 0) {
             berrno be;
             be.set_errno(bfd->berrno);
@@ -265,7 +261,7 @@ int create_file(JCR *jcr, ATTR *attr, BFILE *bfd, int replace)
           * Here we are going to attempt to restore to a FIFO, which
           *   means that the FIFO must already exist, AND there must
           *   be some process already attempting to read from the
-          *   FIFO, so we open it write-only. 
+          *   FIFO, so we open it write-only.
           */
          if (attr->type == FT_RAW || attr->type == FT_FIFO) {
             btimer_t *tid;
@@ -282,6 +278,7 @@ int create_file(JCR *jcr, ATTR *attr, BFILE *bfd, int replace)
                Qmsg1(jcr, M_ERROR, 0, _("bpkt already open fid=%d\n"), bfd->fid);
             }
             Dmsg2(400, "open %s flags=0x%x\n", attr->ofname, flags);
+            set_fattrs(bfd, &attr->statp);
             if ((bopen(bfd, attr->ofname, flags, 0)) < 0) {
                berrno be;
                be.set_errno(bfd->berrno);
@@ -313,14 +310,12 @@ int create_file(JCR *jcr, ATTR *attr, BFILE *bfd, int replace)
             berrno be;
 #ifdef HAVE_CHFLAGS
             struct stat s;
-
-        /*
+           /*
             * If using BSD user flags, maybe has a file flag
             * preventing this. So attempt to disable, retry link,
             * and reset flags.
             * Note that BSD securelevel may prevent disabling flag.
-        */
-
+            */
             if (stat(attr->olname, &s) == 0 && s.st_flags != 0) {
                if (chflags(attr->olname, 0) == 0) {
                   if (link(attr->olname, attr->ofname) != 0) {
@@ -377,13 +372,14 @@ int create_file(JCR *jcr, ATTR *attr, BFILE *bfd, int replace)
          if (is_bopen(bfd)) {
             Qmsg1(jcr, M_ERROR, 0, _("bpkt already open fid=%d\n"), bfd->fid);
          }
+         set_fattrs(bfd, &attr->statp);
          if ((bopen(bfd, attr->ofname, O_WRONLY|O_BINARY, 0)) < 0) {
             berrno be;
             be.set_errno(bfd->berrno);
 #ifdef HAVE_WIN32
             /* Check for trying to create a drive, if so, skip */
-            if (attr->ofname[1] == ':' && 
-                IsPathSeparator(attr->ofname[2]) && 
+            if (attr->ofname[1] == ':' &&
+                IsPathSeparator(attr->ofname[2]) &&
                 attr->ofname[3] == '\0') {
                return CF_SKIP;
             }
@@ -414,6 +410,7 @@ int create_file(JCR *jcr, ATTR *attr, BFILE *bfd, int replace)
       break;
    default:
       Qmsg2(jcr, M_ERROR, 0, _("Unknown file type %d; not restored: %s\n"), attr->type, attr->fname);
+      Pmsg2(000, "Unknown file type %d; not restored: %s\n", attr->type, attr->fname);
       break;
    }
    return CF_ERROR;
