@@ -1,29 +1,20 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2011 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2017 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation and included
-   in the file LICENSE.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   This notice must be preserved when any source code is
+   conveyed and/or propagated.
 
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  *  daemon.c by Kern Sibbald 2000
@@ -40,24 +31,22 @@
 
 
 #include "bacula.h"
-extern int debug_level;
 
 void
 daemon_start()
 {
-#if !defined(HAVE_WIN32)
-   int i;
+#ifndef HAVE_WIN32
+   int i, fd, next_fd;
    pid_t cpid;
    mode_t oldmask;
 #ifdef DEVELOPER
-   int low_fd = 2;
+   next_fd = 3;
 #else
-   int low_fd = -1;
+   next_fd = 0;
 #endif
    /*
     *  Become a daemon.
     */
-
    Dmsg0(900, "Enter daemon_start\n");
    if ( (cpid = fork() ) < 0) {
       berrno be;
@@ -65,19 +54,26 @@ daemon_start()
    } else if (cpid > 0) {
       exit(0);              /* parent exits */
    }
-   /* Child continues */
 
+   /* Child continues */
    setsid();
 
    /* In the PRODUCTION system, we close ALL
     * file descriptors except stdin, stdout, and stderr.
     */
    if (debug_level > 0) {
-      low_fd = 2;                     /* don't close debug output */
+      next_fd = 3;                    /* don't close debug output */
    }
-   for (i=sysconf(_SC_OPEN_MAX)-1; i > low_fd; i--) {
+
+#if defined(HAVE_FCNTL_F_CLOSEM)
+   fcntl(next_fd, F_CLOSEM);
+#elif defined(HAVE_CLOSEFROM)
+   closefrom(next_fd);
+#else
+   for (i=sysconf(_SC_OPEN_MAX); i >= next_fd; i--) {
       close(i);
    }
+#endif
 
    /* Move to root directory. For debug we stay
     * in current directory so dumps go there.
@@ -102,7 +98,6 @@ daemon_start()
     *  send total garbage to our socket.
     *
     */
-   int fd;
    fd = open("/dev/null", O_RDONLY, 0644);
    if (fd > 2) {
       close(fd);
