@@ -1,29 +1,20 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2001-2011 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2016 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation and included
-   in the file LICENSE.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   This notice must be preserved when any source code is 
+   conveyed and/or propagated.
 
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  * Bacula Thread Read/Write locking code. It permits
@@ -38,7 +29,7 @@
  *
  */
 
-#define _LOCKMGR_COMPLIANT
+#define LOCKMGR_COMPLIANT
 #include "bacula.h"
 #include "devlock.h"
 
@@ -49,7 +40,7 @@
  *           errno on failure
  */
 
-devlock *new_devlock()                          
+devlock *new_devlock()
 {
    devlock *lock;
    lock = (devlock *)malloc(sizeof (devlock));
@@ -57,14 +48,14 @@ devlock *new_devlock()
    return lock;
 }
 
-int devlock::init(int initial_priority)
+int devlock::init(int init_priority)
 {
    int stat;
    devlock *rwl = this;
 
    rwl->r_active = rwl->w_active = 0;
    rwl->r_wait = rwl->w_wait = 0;
-   rwl->priority = initial_priority;
+   rwl->priority = init_priority;
    if ((stat = pthread_mutex_init(&rwl->mutex, NULL)) != 0) {
       return stat;
    }
@@ -273,7 +264,7 @@ int devlock::writelock(int areason, bool acan_take)
       rwl->w_active++;                /* we are running */
       rwl->writer_id = pthread_self(); /* save writer thread's id */
       lmgr_post_lock();
-   } 
+   }
    rwl->reason = areason;
    rwl->can_take = acan_take;
    pthread_mutex_unlock(&rwl->mutex);
@@ -513,14 +504,12 @@ int main (int argc, char *argv[])
     int thread_writes = 0;
     int data_writes = 0;
 
-#ifdef USE_THR_SETCONCURRENCY
     /*
-     * On Solaris 2.5,2.6,7 and 8 threads are not timesliced. To ensure
-     * that our threads can run concurrently, we need to
-     * increase the concurrency level to THREADS.
+     * For Solaris 2.5,2.6,7 and 8 threads are not timesliced.
+     *  To ensure our threads can run concurrently, we specifically
+     *  set the concurrency level to THREADS.
      */
-    thr_setconcurrency (THREADS);
-#endif
+    thr_setconcurrency(THREADS); /* Turned on only for Solaris */
 
     /*
      * Initialize the shared data.
@@ -596,7 +585,7 @@ int main (int argc, char *argv[])
  *
  * Demonstrate use of non-blocking read-write locks.
  *
- * Special notes: On older Solaris system, call thr_setconcurrency()
+ * On older Solaris system, call thr_setconcurrency()
  * to allow interleaved thread execution, since threads are not
  * timesliced.
  */
@@ -686,15 +675,13 @@ int main (int argc, char *argv[])
     int thread_updates = 0, data_updates = 0;
     int status;
 
-#ifdef USE_THR_SETCONCURRENCY
     /*
-     * On Solaris 2.5,2.6,7 and 8 threads are not timesliced. To ensure
-     * that our threads can run concurrently, we need to
-     * increase the concurrency level to THREADS.
+     * For Solaris 2.5,2.6,7 and 8 threads are not timesliced.
+     *  To ensure our threads can run concurrently, we specifically
+     *  set the concurrency level to THREADS.
      */
     DPRINTF (("Setting concurrency level to %d\n", THREADS));
-    thr_setconcurrency (THREADS);
-#endif
+    thr_setconcurrency(THREADS); /* Turned on only for Solaris */
 
     /*
      * Initialize the shared data.
