@@ -1,35 +1,26 @@
 /*
-   Copyright (C) 2011-2011 Bacula Systems(R) SA
+   Bacula(R) - The Network Backup Solution
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can modify it under the terms of
-   version three of the GNU Affero General Public License as published by the 
-   Free Software Foundation, which is listed in the file LICENSE.
+   Copyright (C) 2000-2017 Kern Sibbald
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   Bacula® is a registered trademark of Kern Sibbald.
-   Bacula Systems(R) is a trademark of Bacula Systems SA.
-   Bacula Enterprise(TM) is a trademark of Bacula Systems SA.
+   This notice must be preserved when any source code is 
+   conveyed and/or propagated.
 
-   The licensor of Bacula Enterprise(TM) is Bacula Systems(R) SA,
-   Rue Galilee 5, 1400 Yverdon-les-Bains, Switzerland.
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
-
 /*
  * Handle simple configuration file such as "ini" files.
  * key1 = val     # comment
  * key2 = val     # <type>
- * 
+ *
  */
 
 #include "bacula.h"
@@ -54,11 +45,13 @@ static struct ini_store funcs[] = {
    {"@STR@",    "String",           ini_store_str},
    {"@BOOL@",   "on/off",           ini_store_bool},
    {"@ALIST@",  "String list",      ini_store_alist_str},
+   {"@DATE@",   "Date",             ini_store_date},
+/* TODO: Add protocol for the FD @ASKFD@ */
    {NULL,       NULL,               NULL}
 };
 
-/* 
- * Get handler code from handler @ 
+/*
+ * Get handler code from handler @
  */
 const char *ini_get_store_code(INI_ITEM_HANDLER *handler)
 {
@@ -70,8 +63,8 @@ const char *ini_get_store_code(INI_ITEM_HANDLER *handler)
    return NULL;
 }
 
-/* 
- * Get handler function from handler name 
+/*
+ * Get handler function from handler name
  */
 INI_ITEM_HANDLER *ini_get_store_handler(const char *key)
 {
@@ -79,7 +72,7 @@ INI_ITEM_HANDLER *ini_get_store_handler(const char *key)
       if (!strcmp(funcs[i].key, key)) {
          return funcs[i].handler;
       }
-   }   
+   }
    return NULL;
 }
 
@@ -108,13 +101,13 @@ static void s_err(const char *file, int line, LEX *lc, const char *msg, ...)
          buf, lc->line_no, lc->col_no, lc->fname, lc->line);
 
 //   } else if (ini->ctx) {       /* called from plugin */
-//      ini->bfuncs->JobMessage(ini->ctx, __FILE__, __LINE__, M_FATAL, 0, 
+//      ini->bfuncs->JobMessage(ini->ctx, __FILE__, __LINE__, M_FATAL, 0,
 //                    _("Config file error: %s\n"
 //                      "            : Line %d, col %d of file %s\n%s\n"),
 //                            buf, lc->line_no, lc->col_no, lc->fname, lc->line);
 //
    } else {                     /* called from ??? */
-      e_msg(file, line, M_ERROR, 0, 
+      e_msg(file, line, M_ERROR, 0,
             _("Config file error: %s\n"
               "            : Line %d, col %d of file %s\n%s\n"),
             buf, lc->line_no, lc->col_no, lc->fname, lc->line);
@@ -137,7 +130,7 @@ void ConfigFile::clear_items()
 
          } else if (items[i].handler == ini_store_alist_str) {
             delete items[i].val.alistval;
-            items[i].val.alistval = NULL;            
+            items[i].val.alistval = NULL;
          }
          items[i].found = false;
       }
@@ -150,7 +143,10 @@ void ConfigFile::free_items()
       for (int i=0; items[i].name; i++) {
          bfree_and_null_const(items[i].name);
          bfree_and_null_const(items[i].comment);
+         bfree_and_null_const(items[i].default_value);
       }
+   }
+   if (items) {
       free(items);
    }
    items = NULL;
@@ -184,8 +180,8 @@ bool ConfigFile::dump_string(const char *buf, int32_t len)
       out_fname = get_pool_memory(PM_FNAME);
       make_unique_filename(&out_fname, (int)(intptr_t)this, (char*)"configfile");
    }
-   
-   fp = fopen(out_fname, "wb");
+
+   fp = bfopen(out_fname, "wb");
    if (!fp) {
       return ret;
    }
@@ -209,8 +205,8 @@ bool ConfigFile::serialize(const char *fname)
    if (!items) {
       return ret;
    }
-   
-   fp = fopen(fname, "w");
+
+   fp = bfopen(fname, "w");
    if (!fp) {
       return ret;
    }
@@ -230,23 +226,25 @@ bool ConfigFile::serialize(const char *fname)
 int ConfigFile::serialize(POOLMEM **buf)
 {
    int len;
-   POOLMEM *tmp;
+   POOLMEM *tmp, *tmp2;
    if (!items) {
       **buf = 0;
       return 0;
    }
-   
+
    len = Mmsg(buf, "# Plugin configuration file\n# Version %d\n", version);
 
    tmp = get_pool_memory(PM_MESSAGE);
+   tmp2 = get_pool_memory(PM_MESSAGE);
 
    for (int i=0; items[i].name ; i++) {
       if (items[i].comment) {
-         Mmsg(tmp, "OptPrompt=%s\n", items[i].comment);
+         Mmsg(tmp, "OptPrompt=%s\n", quote_string(tmp2, items[i].comment));
          pm_strcat(buf, tmp);
       }
       if (items[i].default_value) {
-         Mmsg(tmp, "OptDefault=%s\n", items[i].default_value);
+         Mmsg(tmp, "OptDefault=%s\n",
+              quote_string(tmp2, items[i].default_value));
          pm_strcat(buf, tmp);
       }
       if (items[i].required) {
@@ -254,12 +252,14 @@ int ConfigFile::serialize(POOLMEM **buf)
          pm_strcat(buf, tmp);
       }
 
-      /* variable = @INT64@ */
-      Mmsg(tmp, "%s=%s\n\n", 
+      Mmsg(tmp, "%s=%s\n\n",
            items[i].name, ini_get_store_code(items[i].handler));
+
+      /* variable = @INT64@ */
       len = pm_strcat(buf, tmp);
    }
    free_pool_memory(tmp);
+   free_pool_memory(tmp2);
 
    return len ;
 }
@@ -268,7 +268,7 @@ int ConfigFile::serialize(POOLMEM **buf)
 int ConfigFile::dump_results(POOLMEM **buf)
 {
    int len;
-   POOLMEM *tmp;
+   POOLMEM *tmp, *tmp2;
    if (!items) {
       **buf = 0;
       return 0;
@@ -276,71 +276,83 @@ int ConfigFile::dump_results(POOLMEM **buf)
    len = Mmsg(buf, "# Plugin configuration file\n# Version %d\n", version);
 
    tmp = get_pool_memory(PM_MESSAGE);
+   tmp2 = get_pool_memory(PM_MESSAGE);
 
    for (int i=0; items[i].name ; i++) {
+      bool process= items[i].found;
       if (items[i].found) {
          items[i].handler(NULL, this, &items[i]);
+      }
+      if (!items[i].found && items[i].required && items[i].default_value) {
+         pm_strcpy(this->edit, items[i].default_value);
+         process = true;
+      }
+      if (process) {
          if (items[i].comment && *items[i].comment) {
             Mmsg(tmp, "# %s\n", items[i].comment);
             pm_strcat(buf, tmp);
          }
-         Mmsg(tmp, "%s=%s\n\n", items[i].name, this->edit);
+         if (items[i].handler == ini_store_str  ||
+             items[i].handler == ini_store_name ||
+             items[i].handler == ini_store_date)
+         {
+            Mmsg(tmp, "%s=%s\n\n",
+                 items[i].name,
+                 quote_string(tmp2, this->edit));
+
+         } else {
+            Mmsg(tmp, "%s=%s\n\n", items[i].name, this->edit);
+         }
          len = pm_strcat(buf, tmp);
       }
    }
    free_pool_memory(tmp);
+   free_pool_memory(tmp2);
 
    return len ;
 }
 
-/* Parse a config file used by Plugin/Director */
-bool ConfigFile::parse(const char *fname)
+bool ConfigFile::parse()
 {
    int token, i;
-   bool ret=false;
+   bool ret = false;
+   bool found;
 
-   if (!items) {
-      return false;
-   }
-
-   if ((lc = lex_open_file(lc, fname, s_err)) == NULL) {
-      berrno be;
-      Emsg2(M_ERROR, 0, _("Cannot open config file %s: %s\n"),
-            fname, be.bstrerror());
-      return false;
-   }
    lc->options |= LOPT_NO_EXTERN;
    lc->caller_ctx = (void *)this;
 
    while ((token=lex_get_token(lc, T_ALL)) != T_EOF) {
-      Dmsg1(dbglevel, "parse got token=%s\n", lex_tok_to_str(token));
       if (token == T_EOL) {
          continue;
       }
+      found = false;
       for (i=0; items[i].name; i++) {
          if (strcasecmp(items[i].name, lc->str) == 0) {
             if ((token = lex_get_token(lc, T_EQUALS)) == T_ERROR) {
-               Dmsg1(dbglevel, "in T_IDENT got token=%s\n",
-                     lex_tok_to_str(token));
+               Dmsg2(dbglevel, "in T_IDENT got token=%s str=%s\n", lex_tok_to_str(token), lc->str);
                break;
             }
-
+            Dmsg2(dbglevel, "parse got token=%s str=%s\n", lex_tok_to_str(token), lc->str);
             Dmsg1(dbglevel, "calling handler for %s\n", items[i].name);
             /* Call item handler */
             ret = items[i].found = items[i].handler(lc, this, &items[i]);
-            i = -1;
+            found = true;
             break;
          }
       }
-      if (i >= 0) {
-         Dmsg1(dbglevel, "Keyword = %s\n", lc->str);
+      if (!found) {
+         Dmsg1(dbglevel, "Unfound keyword=%s\n", lc->str);
          scan_err1(lc, "Keyword %s not found", lc->str);
          /* We can raise an error here */
          break;
+      } else {
+         Dmsg1(dbglevel, "Found keyword=%s\n", items[i].name);
       }
       if (!ret) {
+         Dmsg1(dbglevel, "Error getting value for keyword=%s\n", items[i].name);
          break;
       }
+      Dmsg0(dbglevel, "Continue with while(token) loop\n");
    }
 
    for (i=0; items[i].name; i++) {
@@ -351,10 +363,40 @@ bool ConfigFile::parse(const char *fname)
    }
 
    lc = lex_close_file(lc);
-
    return ret;
 }
 
+/* Parse a config file used by Plugin/Director */
+bool ConfigFile::parse(const char *fname)
+{
+   if (!items) {
+      return false;
+   }
+
+   if ((lc = lex_open_file(lc, fname, s_err)) == NULL) {
+      berrno be;
+      Emsg2(M_ERROR, 0, _("Cannot open config file %s: %s\n"),
+            fname, be.bstrerror());
+      return false;
+   }
+   return parse();          /* Parse file */
+}
+
+/* Parse a config buffer used by Plugin/Director */
+bool ConfigFile::parse_buf(const char *buffer)
+{
+   if (!items) {
+      return false;
+   }
+
+   if ((lc = lex_open_buf(lc, buffer, s_err)) == NULL) {
+      Emsg0(M_ERROR, 0, _("Cannot open lex\n"));
+      return false;
+   }
+   return parse();         /* Parse memory buffer */
+}
+
+
 /* Analyse the content of a ini file to build the item list
  * It uses special syntax for datatype. Used by Director on Restore object
  *
@@ -407,7 +449,7 @@ bool ConfigFile::unserialize(const char *fname)
       } else if (strcasecmp("optdefault", lc->str) == 0) {
          assign = &(items[nb].default_value);
 
-      } else if (strcasecmp("optrequired", lc->str) == 0) { 
+      } else if (strcasecmp("optrequired", lc->str) == 0) {
          items[nb].required = true;               /* Don't use argument */
          scan_to_eol(lc);
          continue;
@@ -486,8 +528,10 @@ bool ini_store_name(LEX *lc, ConfigFile *inifile, ini_items *item)
       return true;
    }
    if (lex_get_token(lc, T_NAME) == T_ERROR) {
+      Dmsg0(dbglevel, "Want token=T_NAME got T_ERROR\n");
       return false;
    }
+   Dmsg1(dbglevel, "ini_store_name: %s\n", lc->str);
    strncpy(item->val.nameval, lc->str, sizeof(item->val.nameval));
    scan_to_eol(lc);
    return true;
@@ -495,26 +539,33 @@ bool ini_store_name(LEX *lc, ConfigFile *inifile, ini_items *item)
 
 bool ini_store_alist_str(LEX *lc, ConfigFile *inifile, ini_items *item)
 {
-   alist *list;
+   alist *list = item->val.alistval;
    if (!lc) {
       /* TODO, write back the alist to edit buffer */
       return true;
    }
-   if (lex_get_token(lc, T_STRING) == T_ERROR) {
-      return false;
+
+   for (;;) {
+      if (lex_get_token(lc, T_STRING) == T_ERROR) {
+         return false;
+      }
+
+      if (list == NULL) {
+         list = New(alist(10, owned_by_alist));
+      }
+      list->append(bstrdup(lc->str));
+
+      if (lc->ch != ',') {         /* if no other item follows */
+         if (!lex_check_eol(lc)) {
+            /* found garbage at the end of the line */
+            return false;
+         }
+         break;                    /* get out */
+      }
+      lex_get_token(lc, T_ALL);    /* eat comma */
    }
 
-   if (item->val.alistval == NULL) {
-      list = New(alist(10, owned_by_alist));
-   } else {
-      list = item->val.alistval;    
-   }
-
-   Dmsg4(900, "Append %s to alist %p size=%d %s\n", 
-         lc->str, list, list->size(), item->name);
-   list->append(bstrdup(lc->str));
    item->val.alistval = list;
-
    scan_to_eol(lc);
    return true;
 }
@@ -584,13 +635,40 @@ bool ini_store_bool(LEX *lc, ConfigFile *inifile, ini_items *item)
    if (lex_get_token(lc, T_NAME) == T_ERROR) {
       return false;
    }
-   if (strcasecmp(lc->str, "yes") == 0 || strcasecmp(lc->str, "true") == 0) {
+   if (strcasecmp(lc->str, "yes")  == 0 ||
+       strcasecmp(lc->str, "true") == 0 ||
+       strcasecmp(lc->str, "on")   == 0 ||
+       strcasecmp(lc->str, "1")    == 0)
+   {
       item->val.boolval = true;
-   } else if (strcasecmp(lc->str, "no") == 0 || strcasecmp(lc->str, "false") == 0) {
+
+   } else if (strcasecmp(lc->str, "no")    == 0 ||
+              strcasecmp(lc->str, "false") == 0 ||
+              strcasecmp(lc->str, "off")   == 0 ||
+              strcasecmp(lc->str, "0")     == 0)
+   {
       item->val.boolval = false;
+
    } else {
       /* YES and NO must not be translated */
-      scan_err2(lc, _("Expect %s, got: %s"), "YES, NO, TRUE, or FALSE", lc->str);
+      scan_err2(lc, _("Expect %s, got: %s"), "YES, NO, ON, OFF, 0, 1, TRUE, or FALSE", lc->str);
+      return false;
+   }
+   scan_to_eol(lc);
+   return true;
+}
+
+bool ini_store_date(LEX *lc, ConfigFile *inifile, ini_items *item)
+{
+   if (!lc) {
+      bstrutime(inifile->edit,sizeof_pool_memory(inifile->edit),item->val.btimeval);
+      return true;
+   }
+   if (lex_get_token(lc, T_STRING) == T_ERROR) {
+      return false;
+   }
+   item->val.btimeval = str_to_utime(lc->str);
+   if (item->val.btimeval == 0) {
       return false;
    }
    scan_to_eol(lc);
@@ -640,6 +718,15 @@ int report()
    return err>0;
 }
 
+struct ini_items membuf_items[] = {
+   /* name          handler           comment             req */
+   {"client",      ini_store_name,    "Client name",       0},
+   {"serial",      ini_store_int32,   "Serial number",     1},
+   {"max_clients", ini_store_int32,   "Max Clients",       0},
+   {NULL,          NULL,              NULL,                0}
+};
+
+
 struct ini_items test_items[] = {
    /* name          handler           comment             req */
    {"datastore",   ini_store_name,    "Target Datastore", 0},
@@ -650,16 +737,63 @@ struct ini_items test_items[] = {
    {"pint64",      ini_store_pint64,  "pint",             0},
    {"int32",       ini_store_int32,   "int 32bit",        0},
    {"plugin.test", ini_store_str,     "test with .",      0},
+   {"adate",       ini_store_date,    "test with date",   0},
    {NULL,          NULL,              NULL,               0}
 };
 
+/* In order to link with libbaccfg */
+int32_t r_last;
+int32_t r_first;
+RES_HEAD **res_head;
+bool save_resource(RES_HEAD **rhead, int type, RES_ITEM *items, int pass){return false;}
+bool save_resource(CONFIG*, int, RES_ITEM*, int) {return false;}
+void dump_resource(int type, RES *ares, void sendit(void *sock, const char *fmt, ...), void *sock){}
+void free_resource(RES *rres, int type){}
+union URES {
+};
+RES_TABLE resources[] = {};
+URES res_all;
+
 int main()
 {
    FILE *fp;
-   int pos;
+   int pos, size;
    ConfigFile *ini = new ConfigFile();
    POOLMEM *buf = get_pool_memory(PM_BSOCK);
+   char buffer[2000];
 
+   //debug_level=500;
+   printf("Begin Memory buffer Test\n");
+   ok(ini->register_items(membuf_items, sizeof(struct ini_items)), "Check sizeof ini_items");
+
+   if ((fp = fopen("test.cfg", "w")) == NULL) {
+      exit (1);
+   }
+   fprintf(fp, "client=JohnDoe\n");
+   fprintf(fp, "serial=2\n");
+   fprintf(fp, "max_clients=3\n");
+   fflush(fp);
+   fseek(fp, 0, SEEK_END);
+   size = ftell(fp);
+   fclose(fp);
+
+   if ((fp = fopen("test.cfg", "rb")) == NULL) {
+      printf("Could not open file test.cfg\n");
+      exit (1);
+   }
+
+   size = fread(buffer, 1, size, fp);
+   buffer[size] = 0;
+   //printf("size of read buffer is: %d\n", size);
+   //printf("====buf=%s\n", buffer);
+
+   ok(ini->parse_buf(buffer), "Test memory read with all members");
+
+   ini->clear_items();
+   fclose(fp);
+
+   //debug_level = 0;
+   printf("\n\nBegin Original Full Tests\n");
    nok(ini->register_items(test_items, 5), "Check bad sizeof ini_items");
    ok(ini->register_items(test_items, sizeof(struct ini_items)), "Check sizeof ini_items");
 
@@ -677,6 +811,7 @@ int main()
    fprintf(fp, "int32=100\n");
    fprintf(fp, "bool=yes\n");
    fprintf(fp, "plugin.test=parameter\n");
+   fprintf(fp, "adate=\"1970-01-02 12:00:00\"\n");
 
    fflush(fp);
 
@@ -690,11 +825,12 @@ int main()
    ok(ini->items[2].val.int64val == 10, "Test int");
    ok(ini->items[4].val.boolval == true, "Test bool");
    ok(ini->items[6].val.int32val == 100, "Test int 32");
+   ok(ini->items[6].val.btimeval != 126000, "Test btime");
 
    alist *list = ini->items[3].val.alistval;
    nok(ini->items[3].found, "Test presence of alist");
 
-   fprintf(fp, "list=a\nlist=b\nlist=c\n");
+   fprintf(fp, "list=a\nlist=b\nlist=c,d,e\n");
    fflush(fp);
 
    ini->clear_items();
@@ -703,7 +839,7 @@ int main()
    list = ini->items[3].val.alistval;
    ok(ini->items[3].found, "Test presence of alist");
    ok(list != NULL, "Test list member");
-   ok(list->size() == 3, "Test list size");
+   ok(list->size() == 5, "Test list size");
 
    ok(!strcmp((char *)list->get(0), "a"), "Testing alist[0]");
    ok(!strcmp((char *)list->get(1), "b"), "Testing alist[1]");
@@ -723,12 +859,12 @@ int main()
    fclose(fp);
    ini->clear_items();
    ini->free_items();
-   
+
    /* Test  */
    if ((fp = fopen("test2.cfg", "w")) == NULL) {
       exit (1);
    }
-   fprintf(fp, 
+   fprintf(fp,
            "# this is a comment\n"
            "optprompt=\"Datastore Name\"\n"
            "datastore=@NAME@\n"
@@ -743,6 +879,7 @@ int main()
            "pouet=@STR@\n"
            "int32=@INT32@\n"
            "plugin.test=@STR@\n"
+           "adate=@DATE@\n"
       );
    fclose(fp);
 
@@ -767,16 +904,19 @@ int main()
    ok(ini->items[pos].val.int64val == 10, "Test int");
    ok(ini->items[pos].required == true, "Check required");
 
-   ok((pos = ini->get_item("bool")) == 4, "Check bool definition");   
+   ok((pos = ini->get_item("bool")) == 4, "Check bool definition");
    ok(ini->items[pos].val.boolval == true, "Test bool");
 
+   ok((pos = ini->get_item("adate")) == 9, "Check adate definition");
+   ok(ini->items[pos].val.btimeval == 126000, "Test date");
+
    ok(ini->dump_results(&buf), "Test to dump results");
    printf("<%s>\n", buf);
 
    ini->clear_items();
    ini->free_items();
    report();
-   
+
    free_pool_memory(buf);
    exit (0);
 }
