@@ -1,17 +1,20 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2014 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2017 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from many
-   others, a complete list can be found in the file AUTHORS.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
    You may use this file and others of this release according to the
    license defined in the LICENSE file, which includes the Affero General
    Public License, v3.0 ("AGPLv3") and some additional permissions and
    terms pursuant to its AGPLv3 Section 7.
 
-   Bacula® is a registered trademark of Kern Sibbald.
+   This notice must be preserved when any source code is 
+   conveyed and/or propagated.
+
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  * Manipulation routines for Job Control Records and
@@ -75,7 +78,7 @@ static pthread_key_t jcr_key;         /* Pointer to jcr for each thread */
 
 pthread_once_t key_once = PTHREAD_ONCE_INIT;
 
-static char Job_status[]     = "Status Job=%s JobStatus=%d\n";
+static char Job_status[] = "Status JobId=%ld JobStatus=%d\n";
 
 
 void lock_jobs()
@@ -277,6 +280,23 @@ bool JCR::JobReads()
    return false;
 }
 
+/* We can stop only Backup jobs connected to a client. It doesn't make sens at
+ * this time to stop a copy, migraton, restore or a verify job. The specific
+ * code should be implemented first.
+ */
+bool JCR::can_be_stopped()
+{
+   bool ok=true;
+   if (getJobType() == JT_BACKUP) {          /* Is a Backup */
+      if (getJobLevel() == L_VIRTUAL_FULL) { /* Is a VirtualFull */
+         ok = false;
+      }
+   } else {   /* Is not a backup (so, copy, migration, admin, verify, ... */
+      ok = false;
+   }
+   return ok;
+}
+
 /*
  * Push a subroutine address into the job end callback stack
  */
@@ -286,6 +306,9 @@ void job_end_push(JCR *jcr, void job_end_cb(JCR *jcr,void *), void *ctx)
    jcr->job_end_push.append(ctx);
 }
 
+/* DELETE ME when bugs in MA1512, MA1632 MA1639 are fixed */
+void (*MA1512_reload_job_end_cb)(JCR *,void *) = NULL;
+
 /* Pop each job_end subroutine and call it */
 static void job_end_pop(JCR *jcr)
 {
@@ -294,7 +317,20 @@ static void job_end_pop(JCR *jcr)
    for (int i=jcr->job_end_push.size()-1; i > 0; ) {
       ctx = jcr->job_end_push.get(i--);
       job_end_cb = (void (*)(JCR *,void *))jcr->job_end_push.get(i--);
-      job_end_cb(jcr, ctx);
+      /* check for bug MA1512, MA1632 MA1639,
+       * today, job_end_cb can only be reload_job_end_cb() from DIR */
+      if (job_end_cb != MA1512_reload_job_end_cb && MA1512_reload_job_end_cb != NULL) {
+         Tmsg2(0, "Bug 'job_end_pop' detected, skip ! job_end_cb=0x%p ctx=0x%p\n", job_end_cb, ctx);
+         Tmsg0(0, "Display job_end_push list\n");
+         for (int j=jcr->job_end_push.size()-1; j > 0; ) {
+            void *ctx2 = jcr->job_end_push.get(j--);
+            void *job_end_cb2 = jcr->job_end_push.get(j--);
+            Tmsg3(0, "Bug 'job_end_pop' entry[%d] job_end_cb=0x%p ctx=0x%p\n", j+1, job_end_cb2, ctx2);
+         }
+      } else
+      {
+         job_end_cb(jcr, ctx);
+      }
    }
 }
 
@@ -321,7 +357,6 @@ JCR *new_jcr(int size, JCR_free_HANDLER *daemon_free_jcr)
 {
    JCR *jcr;
    MQUEUE_ITEM *item = NULL;
-   struct sigaction sigtimer;
    int status;
 
    Dmsg0(dbglvl, "Enter new_jcr\n");
@@ -331,7 +366,9 @@ JCR *new_jcr(int size, JCR_free_HANDLER *daemon_free_jcr)
       Jmsg1(NULL, M_ABORT, 0, _("pthread_once failed. ERR=%s\n"), be.bstrerror(status));
    }
    jcr = (JCR *)malloc(size);
-   memset(jcr, 0, size);
+   bmemzero(jcr, size);
+   /* Note for the director, this value is changed in jobq.c */
+   jcr->my_thread_id = pthread_self();
    jcr->msg_queue = New(dlist(item, &item->link));
    if ((status = pthread_mutex_init(&jcr->msg_queue_mutex, NULL)) != 0) {
       berrno be;
@@ -350,16 +387,22 @@ JCR *new_jcr(int size, JCR_free_HANDLER *daemon_free_jcr)
    jcr->errmsg[0] = 0;
    jcr->comment = get_pool_memory(PM_FNAME);
    jcr->comment[0] = 0;
+   jcr->StatusErrMsg = get_pool_memory(PM_FNAME);
+   jcr->StatusErrMsg[0] = 0;
+   jcr->job_uid = -1;
    /* Setup some dummy values */
    bstrncpy(jcr->Job, "*System*", sizeof(jcr->Job));
    jcr->JobId = 0;
    jcr->setJobType(JT_SYSTEM);           /* internal job until defined */
    jcr->setJobLevel(L_NONE);
    jcr->setJobStatus(JS_Created);        /* ready to run */
+#ifndef HAVE_WIN32
+   struct sigaction sigtimer;
    sigtimer.sa_flags = 0;
    sigtimer.sa_handler = timeout_handler;
    sigfillset(&sigtimer.sa_mask);
    sigaction(TIMEOUT_SIGNAL, &sigtimer, NULL);
+#endif
 
    /*
     * Locking jobs is a global lock that is needed
@@ -414,31 +457,20 @@ static void free_common_jcr(JCR *jcr)
    }
 
    /* do this after closing messages */
-   if (jcr->client_name) {
-      free_pool_memory(jcr->client_name);
-      jcr->client_name = NULL;
-   }
-
-   if (jcr->attr) {
-      free_pool_memory(jcr->attr);
-      jcr->attr = NULL;
-   }
+   free_and_null_pool_memory(jcr->JobIds);
+   free_and_null_pool_memory(jcr->client_name);
+   free_and_null_pool_memory(jcr->attr);
+   free_and_null_pool_memory(jcr->VolumeName);
+   free_and_null_pool_memory(jcr->errmsg);
+   free_and_null_pool_memory(jcr->StatusErrMsg);
 
    if (jcr->sd_auth_key) {
       free(jcr->sd_auth_key);
       jcr->sd_auth_key = NULL;
    }
-   if (jcr->VolumeName) {
-      free_pool_memory(jcr->VolumeName);
-      jcr->VolumeName = NULL;
-   }
 
    free_bsock(jcr->dir_bsock);
 
-   if (jcr->errmsg) {
-      free_pool_memory(jcr->errmsg);
-      jcr->errmsg = NULL;
-   }
    if (jcr->where) {
       free(jcr->where);
       jcr->where = NULL;
@@ -491,10 +523,10 @@ void free_jcr(JCR *jcr)
 
    lock_jcr_chain();
    jcr->dec_use_count();              /* decrement use count */
-   if (jcr->use_count() < 0) {
-      Jmsg2(jcr, M_ERROR, 0, _("JCR use_count=%d JobId=%d\n"),
-         jcr->use_count(), jcr->JobId);
-   }
+   ASSERT2(jcr->use_count() >= 0, "JCR use_count < 0");
+   //    Jmsg2(jcr, M_ERROR, 0, _("JCR use_count=%d JobId=%d\n"),
+   //      jcr->use_count(), jcr->JobId);
+   //}
    if (jcr->JobId > 0) {
       Dmsg3(dbglvl, "Dec free_jcr jid=%u use_count=%d Job=%s\n",
          jcr->JobId, jcr->use_count(), jcr->Job);
@@ -507,6 +539,7 @@ void free_jcr(JCR *jcr)
       Dmsg3(dbglvl, "remove jcr jid=%u use_count=%d Job=%s\n",
             jcr->JobId, jcr->use_count(), jcr->Job);
    }
+   jcr->exiting = true;
    remove_jcr(jcr);                   /* remove Jcr from chain */
    unlock_jcr_chain();
 
@@ -564,6 +597,7 @@ void free_jcr(JCR *jcr)
    }
 
    free_common_jcr(jcr);
+   close_msg(NULL);                   /* flush any daemon messages */
    Dmsg0(dbglvl, "Exit free_jcr\n");
 }
 
@@ -581,10 +615,9 @@ void remove_jcr_from_tsd(JCR *jcr)
 
 void JCR::set_killable(bool killable)
 {
-   JCR *jcr = this;
-   jcr->lock();
-   jcr->my_thread_killable = killable;
-   jcr->unlock();
+   lock();
+   my_thread_killable = killable;
+   unlock();
 }
 
 /*
@@ -604,17 +637,24 @@ void set_jcr_in_tsd(JCR *jcr)
 
 void JCR::my_thread_send_signal(int sig)
 {
+   lock_jcr_chain();   /* use global lock */
    this->lock();
+   if (this->exiting) {
+      goto get_out;
+   }
    if (this->is_killable() &&
        !pthread_equal(this->my_thread_id, pthread_self()))
    {
       Dmsg1(800, "Send kill to jid=%d\n", this->JobId);
       pthread_kill(this->my_thread_id, sig);
+      this->exiting = true;
 
    } else if (!this->is_killable()) {
-      Dmsg1(10, "Warning, can't send kill to jid=%d\n", this->JobId);
+      Dmsg1(10, "Warning, cannot send kill to jid=%d marked not killable.\n", this->JobId);
    }
+get_out:
    this->unlock();
+   unlock_jcr_chain();
 }
 
 /*
@@ -828,6 +868,9 @@ static int get_status_priority(int JobStatus)
 {
    int priority = 0;
    switch (JobStatus) {
+   case JS_Incomplete:
+      priority = 10;
+      break;
    case JS_ErrorTerminated:
    case JS_FatalError:
    case JS_Canceled:
@@ -848,9 +891,8 @@ static int get_status_priority(int JobStatus)
  */
 bool JCR::sendJobStatus()
 {
-   JCR *jcr = this;
-   if (jcr->dir_bsock) {
-      return jcr->dir_bsock->fsend(Job_status, jcr->Job, jcr->JobStatus);
+   if (dir_bsock) {
+      return dir_bsock->fsend(Job_status, JobId, JobStatus);
    }
    return true;
 }
@@ -858,13 +900,12 @@ bool JCR::sendJobStatus()
 /*
  * Set and send Job status to Director
  */
-bool JCR::sendJobStatus(int newJobStatus)
+bool JCR::sendJobStatus(int aJobStatus)
 {
-   JCR *jcr = this;
-   if (!jcr->is_JobStatus(newJobStatus)) {
-      setJobStatus(newJobStatus);
-      if (jcr->dir_bsock) {
-         return jcr->dir_bsock->fsend(Job_status, jcr->Job, jcr->JobStatus);
+   if (!is_JobStatus(aJobStatus)) {
+      setJobStatus(aJobStatus);
+      if (dir_bsock) {
+         return dir_bsock->fsend(Job_status, JobId, JobStatus);
       }
    }
    return true;
@@ -872,29 +913,31 @@ bool JCR::sendJobStatus(int newJobStatus)
 
 void JCR::setJobStarted()
 {
-   JCR *jcr = this;
-   jcr->job_started = true;
-   jcr->job_started_time = time(NULL);
+   job_started = true;
+   job_started_time = time(NULL);
 }
 
+static pthread_mutex_t status_lock = PTHREAD_MUTEX_INITIALIZER;
+
 void JCR::setJobStatus(int newJobStatus)
 {
-   JCR *jcr = this;
    int priority, old_priority;
-   int oldJobStatus = jcr->JobStatus;
+   int oldJobStatus = JobStatus;
+
+   P(status_lock);
    priority = get_status_priority(newJobStatus);
    old_priority = get_status_priority(oldJobStatus);
 
-   Dmsg2(800, "set_jcr_job_status(%s, %c)\n", Job, newJobStatus);
+   Dmsg2(800, "set_jcr_job_status(%ld, %c)\n", JobId, newJobStatus);
 
    /* Update wait_time depending on newJobStatus and oldJobStatus */
-   update_wait_time(jcr, newJobStatus);
+   update_wait_time(this, newJobStatus);
 
    /*
     * For a set of errors, ... keep the current status
     *   so it isn't lost. For all others, set it.
     */
-   Dmsg2(800, "OnEntry JobStatus=%c newJobstatus=%c\n", oldJobStatus, newJobStatus);
+   Dmsg2(800, "OnEntry JobStatus=%c newJobstatus=%c\n", (oldJobStatus==0)?'0':oldJobStatus, newJobStatus);
    /*
     * If status priority is > than proposed new status, change it.
     * If status priority == new priority and both are zero, take
@@ -905,14 +948,15 @@ void JCR::setJobStatus(int newJobStatus)
    if (priority > old_priority || (
        priority == 0 && old_priority == 0)) {
       Dmsg4(800, "Set new stat. old: %c,%d new: %c,%d\n",
-         jcr->JobStatus, old_priority, newJobStatus, priority);
-      jcr->JobStatus = newJobStatus;     /* replace with new status */
+            (oldJobStatus==0)?'0':oldJobStatus, old_priority, newJobStatus, priority);
+      JobStatus = newJobStatus;     /* replace with new status */
    }
 
-   if (oldJobStatus != jcr->JobStatus) {
-      Dmsg2(800, "leave setJobStatus old=%c new=%c\n", oldJobStatus, newJobStatus);
-//    generate_plugin_event(jcr, bEventStatusChange, NULL);
+   if (oldJobStatus != JobStatus) {
+      Dmsg2(800, "leave setJobStatus old=%c new=%c\n", (oldJobStatus==0)?'0':oldJobStatus, newJobStatus);
+//    generate_plugin_event(this, bEventStatusChange, NULL);
    }
+   V(status_lock);
 }
 
 #ifdef TRACE_JCR_CHAIN
@@ -1157,11 +1201,11 @@ extern "C" void timeout_handler(int sig)
 }
 
 /* Used to display specific daemon information after a fatal signal
- * (like B_DB in the director)
+ * (like BDB in the director)
  */
 #define MAX_DBG_HOOK 10
 static dbg_jcr_hook_t *dbg_jcr_hooks[MAX_DBG_HOOK];
-static int dbg_jcr_handler_count;
+static int dbg_jcr_handler_count=0;
 
 void dbg_jcr_add_hook(dbg_jcr_hook_t *hook)
 {
@@ -1184,6 +1228,7 @@ void dbg_jcr_add_hook(dbg_jcr_hook_t *hook)
 void dbg_print_jcr(FILE *fp)
 {
    char buf1[128], buf2[128], buf3[128], buf4[128];
+
    if (!jcrs) {
       return;
    }
@@ -1193,11 +1238,8 @@ void dbg_print_jcr(FILE *fp)
    for (JCR *jcr = (JCR *)jcrs->first(); jcr ; jcr = (JCR *)jcrs->next(jcr)) {
       fprintf(fp, "threadid=%p JobId=%d JobStatus=%c jcr=%p name=%s\n",
               get_threadid(jcr->my_thread_id), (int)jcr->JobId, jcr->JobStatus, jcr, jcr->Job);
-      fprintf(fp, "threadid=%p killable=%d JobId=%d JobStatus=%c "
-                  "jcr=%p name=%s\n",
-              get_threadid(jcr->my_thread_id), jcr->is_killable(),
-              (int)jcr->JobId, jcr->JobStatus, jcr, jcr->Job);
-      fprintf(fp, "\tuse_count=%i\n", jcr->use_count());
+      fprintf(fp, "\tuse_count=%i killable=%d\n",
+              jcr->use_count(), jcr->is_killable());
       fprintf(fp, "\tJobType=%c JobLevel=%c\n",
               jcr->getJobType(), jcr->getJobLevel());
       bstrftime(buf1, sizeof(buf1), jcr->sched_time);
