@@ -1,29 +1,20 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2010 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2017 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation and included
-   in the file LICENSE.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   This notice must be preserved when any source code is 
+   conveyed and/or propagated.
 
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
 
@@ -44,6 +35,9 @@ extern uint32_t DLL_IMP_EXP sm_buffers;
 #define SMARTALLOC SMARTALLOC
 
 
+/* Avoid aggressive GCC optimization */
+extern void *bmemset(void *s, int c, size_t n);
+
 extern void *sm_malloc(const char *fname, int lineno, unsigned int nbytes),
             *sm_calloc(const char *fname, int lineno,
                 unsigned int nelem, unsigned int elsize),
@@ -70,8 +64,8 @@ extern int sm_check_rtn(const char *fname, int lineno, bool bufdump);
 /* Redefine standard memory allocator calls to use our routines
    instead. */
 
-#define free(x)        sm_free(__FILE__, __LINE__, (x))
-#define cfree(x)       sm_free(__FILE__, __LINE__, (x))
+#define free(x)        sm_free(__FILE__, __LINE__, (void *)(x))
+#define cfree(x)       sm_free(__FILE__, __LINE__, (void *)(x))
 #define malloc(x)      sm_malloc(__FILE__, __LINE__, (x))
 #define calloc(n,e)    sm_calloc(__FILE__, __LINE__, (n), (e))
 #define realloc(p,x)   sm_realloc(__FILE__, __LINE__, (p), (x))
@@ -85,7 +79,7 @@ extern int sm_check_rtn(const char *fname, int lineno, bool bufdump);
 #define actuallymalloc(x)    malloc(x)
 #define actuallycalloc(x,y)  calloc(x,y)
 #define actuallyrealloc(x,y) realloc(x,y)
-#define sm_dump(x)
+inline void sm_dump(int x, int y=0) {} /* with default arguments, we can't use a #define */
 #define sm_static(x)
 #define sm_new_owner(a, b, c)
 #define sm_malloc(f, l, n)     malloc(n)
@@ -93,9 +87,12 @@ extern int sm_check_rtn(const char *fname, int lineno, bool bufdump);
 #define sm_check(f, l, fl)
 #define sm_check_rtn(f, l, fl) 1
 
-extern void *b_malloc();
+extern void *b_malloc(const char *file, int line, size_t size);
 #define malloc(x) b_malloc(__FILE__, __LINE__, (x))
 
+#define Dsm_check(lvl)
+#define sm_check(f, l, fl)
+#define sm_check_rtn(f, l, fl) 1
 
 #endif
 
@@ -115,15 +112,13 @@ void *operator new(size_t s, const char *fname, int line)
 {
    size_t size =  s > sizeof(int) ? (unsigned int)s : sizeof(int);
    void *p = sm_malloc(fname, line, size);
-   memset(p, 0, size);
-   return p;
+   return bmemset(p, 0, size);   /* return memset() result to avoid GCC 6.1 issue */
 }
 void *operator new[](size_t s, const char *fname, int line)
 {
    size_t size =  s > sizeof(int) ? (unsigned int)s : sizeof(int);
    void *p = sm_malloc(fname, line, size);
-   memset(p, 0, size);
-   return p;
+   return bmemset(p, 0, size);  /* return memset() result to avoid GCC 6.1 issue */
 }
 
 void  operator delete(void *ptr)
@@ -139,7 +134,7 @@ void  operator delete(void *ptr, const char * /*fname*/, int /*line*/)
 {
    free(ptr);
 }
-void  operator delete[](void *ptr, size_t /*i*/, 
+void  operator delete[](void *ptr, size_t /*i*/,
                         const char * /*fname*/, int /*line*/)
 {
    free(ptr);
@@ -159,12 +154,12 @@ class SMARTALLOC
    public:
       void *operator new(size_t s) {
          void *p = malloc(s);
-         memset(p, 0, s);
+         bmemset(p, 0, s);
          return p;
       }
       void *operator new[](size_t s) {
          void *p = malloc(s);
-         memset(p, 0, s);
+         bmemset(p, 0, s);
          return p;
       }
       void  operator delete(void *ptr) {
@@ -175,5 +170,4 @@ class SMARTALLOC
       }
 };
 #endif  /* SMARTALLOC */
-   
 #endif  /* !SMARTALLOC_H */
