@@ -1,29 +1,20 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2002-2012 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2017 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation and included
-   in the file LICENSE.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   This notice must be preserved when any source code is 
+   conveyed and/or propagated.
 
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  * Directory tree build/traverse routines
@@ -102,6 +93,9 @@ TREE_ROOT *new_tree(int count)
    root->cached_path = get_pool_memory(PM_FNAME);
    root->type = TN_ROOT;
    root->fname = "";
+   root->can_access = 1;
+   HL_ENTRY* entry = NULL;
+   root->hardlinks.init(entry, &entry->link, 0);
    return root;
 }
 
@@ -115,6 +109,7 @@ static TREE_NODE *new_tree_node(TREE_ROOT *root)
    node = (TREE_NODE *)tree_alloc(root, size);
    memset(node, 0, size);
    node->delta_seq = -1;
+   node->can_access = 1;
    return node;
 }
 
@@ -172,6 +167,7 @@ void free_tree(TREE_ROOT *root)
    struct s_mem *mem, *rel;
    uint32_t freed_blocks = 0;
 
+   root->hardlinks.destroy();
    for (mem=root->mem; mem; ) {
       rel = mem;
       mem = mem->next;
@@ -192,7 +188,7 @@ void free_tree(TREE_ROOT *root)
 void tree_add_delta_part(TREE_ROOT *root, TREE_NODE *node,
                          JobId_t JobId, int32_t FileIndex)
 {
-   struct delta_list *elt = 
+   struct delta_list *elt =
       (struct delta_list*) tree_alloc(root, sizeof(struct delta_list));
 
    elt->next = node->delta_list;
@@ -334,7 +330,6 @@ static TREE_NODE *search_and_insert_tree_node(char *fname, int type,
    strcpy(node->fname, fname);
    node->parent = parent;
    node->type = type;
-
    /* Maintain a linear chain of nodes */
    if (!root->first) {
       root->first = node;
@@ -386,7 +381,7 @@ TREE_NODE *tree_cwd(char *path, TREE_ROOT *root, TREE_NODE *node)
    /* Handle relative path */
    if (path[0] == '.' && path[1] == '.' && (IsPathSeparator(path[2]) || path[2] == '\0')) {
       TREE_NODE *parent = node->parent ? node->parent : node;
-      if (path[2] == 0) { 
+      if (path[2] == 0) {
          return parent;
       } else {
          return tree_cwd(path+3, root, parent);
@@ -440,6 +435,9 @@ TREE_NODE *tree_relcwd(char *path, TREE_ROOT *root, TREE_NODE *node)
    if (!cd || (cd->type == TN_FILE && !tree_node_has_child(cd))) {
       return NULL;
    }
+   if (!cd->can_access) {       /* Will display permission denied */
+      return cd;
+   }
    if (!p) {
       Dmsg0(100, "tree_relcwd: no more to lookup. found.\n");
       return cd;
