@@ -1,32 +1,23 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2002-2011 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2017 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation and included
-   in the file LICENSE.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   This notice must be preserved when any source code is 
+   conveyed and/or propagated.
 
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
- * Bacula thread watchdog routine. General routine that 
+ * Bacula thread watchdog routine. General routine that
  *  allows setting a watchdog timer with a callback that is
  *  called when the timer goes off.
  *
@@ -52,7 +43,7 @@ static void wd_lock();
 static void wd_unlock();
 
 /* Static globals */
-static bool quit = false;;
+static bool quit = false;
 static bool wd_is_init = false;
 static brwlock_t lock;                /* watchdog lock */
 
@@ -60,7 +51,7 @@ static pthread_t wd_tid;
 static dlist *wd_queue;
 static dlist *wd_inactive;
 
-/* 
+/*
  * Returns: 0 if the current thread is NOT the watchdog
  *          1 if the current thread is the watchdog
  */
@@ -265,19 +256,6 @@ extern "C" void *watchdog_thread(void *arg)
    while (!quit) {
       watchdog_t *p;
 
-      /*
-       *
-       *  NOTE. lock_jcr_chain removed, but the message below
-       *   was left until we are sure there are no deadlocks.
-       *  
-       * We lock the jcr chain here because a good number of the
-       *   callback routines lock the jcr chain. We need to lock
-       *   it here *before* the watchdog lock because the SD message
-       *   thread first locks the jcr chain, then when closing the
-       *   job locks the watchdog chain. If the two threads do not
-       *   lock in the same order, we get a deadlock -- each holds
-       *   the other's needed lock.
-       */
       wd_lock();
 
 walk_list:
