@@ -1,14 +1,24 @@
 /*
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2007-2010 Kern Sibbald
+   Copyright (C) 2000-2015 Kern Sibbald
 
-   You may freely use this code to create your own plugin provided
-   it is to write a plugin for Bacula licensed under AGPLv3
-   (as Bacula is), and in that case, you may also remove
-   the above Copyright and this notice as well as modify 
-   the code in any way. 
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
+
+   This notice must be preserved when any source code is
+   conveyed and/or propagated.
+
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
+#define BUILD_PLUGIN
+#define BUILDING_DLL            /* required for Windows plugin */
+
 #include "bacula.h"
 #include "fd_plugins.h"
 
@@ -18,7 +28,7 @@ extern "C" {
 
 #define PLUGIN_LICENSE      "AGPLv3"
 #define PLUGIN_AUTHOR       "Your name"
-#define PLUGIN_DATE         "January 2008"
+#define PLUGIN_DATE         "January 2010"
 #define PLUGIN_VERSION      "1"
 #define PLUGIN_DESCRIPTION  "Test File Daemon Plugin"
 
@@ -35,7 +45,8 @@ static bRC startRestoreFile(bpContext *ctx, const char *cmd);
 static bRC endRestoreFile(bpContext *ctx);
 static bRC createFile(bpContext *ctx, struct restore_pkt *rp);
 static bRC setFileAttributes(bpContext *ctx, struct restore_pkt *rp);
-
+static bRC checkFile(bpContext *ctx, char *fname);
+static bRC handleXACLdata(bpContext *ctx, struct xacl_pkt *xacl);
 
 /* Pointers to Bacula functions */
 static bFuncs *bfuncs = NULL;
@@ -68,13 +79,16 @@ static pFuncs pluginFuncs = {
    endRestoreFile,
    pluginIO,
    createFile,
-   setFileAttributes
+   setFileAttributes,
+   checkFile,
+   handleXACLdata
 };
 
 /*
  * Plugin called here when it is first loaded
  */
-bRC loadPlugin(bInfo *lbinfo, bFuncs *lbfuncs, pInfo **pinfo, pFuncs **pfuncs)
+bRC DLL_IMP_EXP
+loadPlugin(bInfo *lbinfo, bFuncs *lbfuncs, pInfo **pinfo, pFuncs **pfuncs)
 {
    bfuncs = lbfuncs;                  /* set Bacula funct pointers */
    binfo  = lbinfo;
@@ -90,7 +104,8 @@ bRC loadPlugin(bInfo *lbinfo, bFuncs *lbfuncs, pInfo **pinfo, pFuncs **pfuncs)
  * Plugin called here when it is unloaded, normally when
  *  Bacula is going to exit.
  */
-bRC unloadPlugin() 
+bRC DLL_IMP_EXP
+unloadPlugin()
 {
    printf("plugin: Unloaded\n");
    return bRC_OK;
@@ -113,7 +128,7 @@ static bRC newPlugin(bpContext *ctx)
 }
 
 /*
- * Release everything concerning a particular instance of a 
+ * Release everything concerning a particular instance of a
  *  plugin. Normally called when the Job terminates.
  */
 static bRC freePlugin(bpContext *ctx)
@@ -128,17 +143,17 @@ static bRC freePlugin(bpContext *ctx)
  * Called by core code to get a variable from the plugin.
  *   Not currently used.
  */
-static bRC getPluginValue(bpContext *ctx, pVariable var, void *value) 
+static bRC getPluginValue(bpContext *ctx, pVariable var, void *value)
 {
 // printf("plugin: getPluginValue var=%d\n", var);
    return bRC_OK;
 }
 
-/* 
+/*
  * Called by core code to set a plugin variable.
  *  Not currently used.
  */
-static bRC setPluginValue(bpContext *ctx, pVariable var, void *value) 
+static bRC setPluginValue(bpContext *ctx, pVariable var, void *value)
 {
 // printf("plugin: setPluginValue var=%d\n", var);
    return bRC_OK;
@@ -178,16 +193,18 @@ static bRC handlePluginEvent(bpContext *ctx, bEvent *event, void *value)
    case bEventEndRestoreJob:
       printf("plugin: EndRestoreJob\n");
       break;
-
    /* Plugin command e.g. plugin = <plugin-name>:<name-space>:command */
    case bEventRestoreCommand:
       printf("plugin: backup command=%s\n", NPRT((char *)value));
       break;
-
    case bEventBackupCommand:
       printf("plugin: backup command=%s\n", NPRT((char *)value));
       break;
 
+   case bEventComponentInfo:
+      printf("plugin: Component=%s\n", NPRT((char *)value));
+      break;
+
    default:
       printf("plugin: unknown event=%d\n", event->eventType);
    }
@@ -214,13 +231,13 @@ static bRC startBackupFile(bpContext *ctx, struct save_pkt *sp)
  * Done backing up a file.
  */
 static bRC endBackupFile(bpContext *ctx)
-{ 
+{
    return bRC_OK;
 }
 
 /*
  * Do actual I/O.  Bacula calls this after startBackupFile
- *   or after startRestoreFile to do the actual file 
+ *   or after startRestoreFile to do the actual file
  *   input or output.
  */
 static bRC pluginIO(bpContext *ctx, struct io_pkt *io)
@@ -275,6 +292,19 @@ static bRC setFileAttributes(bpContext *ctx, struct restore_pkt *rp)
    return bRC_OK;
 }
 
+/* When using Incremental dump, all previous dumps are necessary */
+static bRC checkFile(bpContext *ctx, char *fname)
+{
+   return bRC_OK;
+}
+
+/*
+ * New Bacula Plugin API require this
+ */
+static bRC handleXACLdata(bpContext *ctx, struct xacl_pkt *xacl)
+{
+   return bRC_OK;
+}
 
 #ifdef __cplusplus
 }
