@@ -1,29 +1,20 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2007-2011 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2016 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation and included
-   in the file LICENSE.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   This notice must be preserved when any source code is 
+   conveyed and/or propagated.
 
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  *  DirComm, Director communications,class
@@ -70,8 +61,7 @@ void DirComm::terminate()
       }
       if (mainWin->m_connDebug)
          Pmsg2(000, "DirComm %i terminating connections %s\n", m_conn, m_console->m_dir->name());
-      m_sock->close();
-      m_sock = NULL;
+      free_bsock(m_sock);
    }
 }
 
@@ -84,10 +74,16 @@ bool DirComm::connect_dir()
    utime_t heart_beat;
    char buf[1024];
    CONRES *cons;
-      
+   int numcon = 0;
+   int i = 0;
+
    buf[0] = 0;
 
-   if (m_sock) {
+   foreach_res(cons, R_CONSOLE) {
+      numcon++;
+   }
+
+   if (m_sock && !is_bsock_open(m_sock)) {
       mainWin->set_status( tr("Already connected."));
       m_console->display_textf(_("Already connected\"%s\".\n"),
             m_console->m_dir->name());
@@ -110,7 +106,33 @@ bool DirComm::connect_dir()
    
    LockRes();
    /* If cons==NULL, default console will be used */
-   cons = (CONRES *)GetNextRes(R_CONSOLE, NULL);
+   for (i=0; i<numcon; i++) {
+      cons = (CONRES *)GetNextRes(R_CONSOLE, (RES *)cons);
+      if (cons->director && strcasecmp(cons->director, m_console->m_dir->name()) == 0) {
+         break;
+      }
+      if (i == (numcon - 1)) {
+         cons = NULL;
+      }
+   }
+
+   /* Look for the first non-linked console */
+   if (cons == NULL) {
+      for (i=0; i<numcon; i++) {
+         cons = (CONRES *)GetNextRes(R_CONSOLE, (RES *)cons);
+         if (cons->director == NULL) {
+            break;
+         }
+         if (i == (numcon - 1)) {
+            cons = NULL;
+         }
+      }
+   }
+
+   /* If no console, take first one */
+   if (!cons) {
+      cons = (CONRES *)GetNextRes(R_CONSOLE, (RES *)NULL);
+   }
    UnlockRes();
 
    /* Initialize Console TLS context once */
@@ -169,9 +191,15 @@ bool DirComm::connect_dir()
       heart_beat = 0;
    }        
 
-   m_sock = bnet_connect(NULL, 5, 15, heart_beat,
+   if (!m_sock) {
+      m_sock = new_bsock();
+   }
+   if (!m_sock->connect(NULL, 5, 15, heart_beat,
                           _("Director daemon"), m_console->m_dir->address,
-                          NULL, m_console->m_dir->DIRport, 0);
+                          NULL, m_console->m_dir->DIRport, 0)) {
+      m_sock->destroy();
+      m_sock = NULL;
+   }
    if (m_sock == NULL) {
       mainWin->set_status("Connection failed");
       if (mainWin->m_connDebug) {
@@ -301,9 +329,10 @@ int DirComm::read()
       return -1;
    }
    while (m_sock) {
+      /* Poll DIR every 50 milliseconds */
       for (;;) {
          if (!m_sock) break;
-         stat = m_sock->wait_data_intr(0, 50000);
+         stat = m_sock->wait_data_intr(0, 50); /* wait 50 milliseconds */
          if (stat > 0) {
             break;
          } 
@@ -372,9 +401,7 @@ int DirComm::read()
          if (mainWin->m_commDebug) Pmsg4(000, "conn %i TEXT_INPUT at_prompt=%d  m_in_select=%d notify=%d\n", 
                m_conn, m_at_prompt, m_in_select, is_notify_enabled());
          if (!m_in_select && is_notify_enabled()) {
-            mainWin->waitExit();
             new textInputDialog(m_console, m_conn);
-         } else {
             if (mainWin->m_commDebug) Pmsg0(000, "!m_in_select && is_notify_enabled\n");
             m_at_prompt = true;
             m_at_main_prompt = false;
@@ -444,11 +471,10 @@ int DirComm::read()
          stat = BNET_HARDEOF;
          return stat;
       }
-      if (is_bnet_stop(m_sock)) {         /* error or term request */
+      if (m_sock->is_stop()) {         /* error or term request */
          if (mainWin->m_commDebug) Pmsg1(000, "conn %i BNET STOP\n", m_conn);
          m_console->stopTimer();
-         m_sock->close();
-         m_sock = NULL;
+         free_bsock(m_sock);
          mainWin->actionConnect->setIcon(QIcon(":images/disconnected.png"));
          QBrush redBrush(Qt::red);
          QTreeWidgetItem *item = mainWin->getFromHash(m_console);
