@@ -1,34 +1,25 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2007-2012 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2016 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation and included
-   in the file LICENSE.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   This notice must be preserved when any source code is 
+   conveyed and/or propagated.
 
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  *  Main program for bat (qt-console)
  *
- *   Kern Sibbald, January MMVII
+ *   Written by Kern Sibbald, January MMVII
  *
  */ 
 
@@ -38,11 +29,11 @@
 #include <QTranslator>
 
 /*
- * We need Qt version 4.7.4 or later to be able to comple correctly
+ * We need Qt version 4.8.4 or later to be able to comple correctly
  */
-#if QT_VERSION < 0x040704
+#if QT_VERSION < 0x040804
 #error "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
-#error "You need Qt version 4.7.4 or later to build Bat"
+#error "You need Qt version 4.8.4 or later to build Bat"
 #error "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
 #endif
 
@@ -73,14 +64,17 @@ int main(int argc, char *argv[])
 
    app = new QApplication(argc, argv);        
    app->setQuitOnLastWindowClosed(true);
+#if QT_VERSION < 0x050000
+   app->setStyle(new QPlastiqueStyle());
    QTextCodec::setCodecForCStrings(QTextCodec::codecForName("UTF-8"));
+#endif
      
    QTranslator qtTranslator;
-   qtTranslator.load(QString("qt_") + QLocale::system().name());
+   qtTranslator.load(QString("qt_") + QLocale::system().name(),QLibraryInfo::location(QLibraryInfo::TranslationsPath));
    app->installTranslator(&qtTranslator);
 
    QTranslator batTranslator;
-   batTranslator.load(QString("bat_") + QLocale::system().name());
+   batTranslator.load(QString("bat_") + QLocale::system().name(),QLibraryInfo::location(QLibraryInfo::TranslationsPath));
    app->installTranslator(&batTranslator);
 
    register_message_callback(message_callback);
@@ -101,13 +95,14 @@ int main(int argc, char *argv[])
    init_msg(NULL, NULL);
    working_directory  = "/tmp";
 
+#ifndef HAVE_WIN32
    struct sigaction sigignore;
    sigignore.sa_flags = 0;
    sigignore.sa_handler = SIG_IGN;
    sigfillset(&sigignore.sa_mask);
    sigaction(SIGPIPE, &sigignore, NULL);
    sigaction(SIGUSR2, &sigignore, NULL);
-
+#endif
 
    while ((ch = getopt(argc, argv, "bc:d:r:st?")) != -1) {
       switch (ch) {
@@ -158,7 +153,7 @@ int main(int argc, char *argv[])
       configfile = bstrdup(CONFIG_FILE);
    }
 
-   config = new_config_parser();
+   config = New(CONFIG());
    parse_bat_config(config, configfile, M_ERROR_TERM);
 
    if (init_crypto() != 0) {
@@ -180,7 +175,9 @@ int main(int argc, char *argv[])
 
 void terminate_console(int /*sig*/)
 {
-// WSA_Cleanup();                  /* TODO: check when we have to call it */
+#ifdef HAVE_WIN32
+   WSACleanup();                  /* TODO: check when we have to call it */
+#endif
    exit(0);
 }
 
