@@ -1,45 +1,44 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2002-2011 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2017 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation and included
-   in the file LICENSE.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   This notice must be preserved when any source code is
+   conveyed and/or propagated.
 
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  *
  *  Routines for handling the autochanger.
  *
- *   Kern Sibbald, August MMII
- *                            
+ *   Written by Kern Sibbald, August MMII
  */
 
 #include "bacula.h"                   /* pull in global headers */
 #include "stored.h"                   /* pull in Storage Deamon headers */
 
+static const int dbglvl = 60;
+
 /* Forward referenced functions */
 static void lock_changer(DCR *dcr);
 static void unlock_changer(DCR *dcr);
-static bool unload_other_drive(DCR *dcr, int slot);
+static bool unload_other_drive(DCR *dcr, int slot, bool writing);
+
+bool DCR::is_virtual_autochanger()
+{
+   return device->changer_command &&
+      (device->changer_command[0] == 0 ||
+       strcmp(device->changer_command, "/dev/null") == 0);
+}
 
 /* Init all the autochanger resources found */
 bool init_autochangers()
@@ -52,7 +51,7 @@ bool init_autochangers()
       foreach_alist(device, changer->device) {
          /*
           * If the device does not have a changer name or changer command
-          *   defined, used the one from the Autochanger resource 
+          *   defined, use the one from the Autochanger resource
           */
          if (!device->changer_name && changer->changer_name) {
             device->changer_name = bstrdup(changer->changer_name);
@@ -61,32 +60,17 @@ bool init_autochangers()
             device->changer_command = bstrdup(changer->changer_command);
          }
          if (!device->changer_name) {
-            Jmsg(NULL, M_ERROR, 0, 
+            Jmsg(NULL, M_ERROR, 0,
                _("No Changer Name given for device %s. Cannot continue.\n"),
                device->hdr.name);
             OK = false;
-         }   
+         }
          if (!device->changer_command) {
-            Jmsg(NULL, M_ERROR, 0, 
+            Jmsg(NULL, M_ERROR, 0,
                _("No Changer Command given for device %s. Cannot continue.\n"),
                device->hdr.name);
             OK = false;
-         }   
-
-#ifdef xxx_needed
-         if (media_type == NULL) {
-            media_type = device->media_type;     /* get Media Type of first device */
-            continue;
-         }     
-         /* Ensure that other devices Media Types are the same */
-         if (strcmp(media_type, device->media_type) != 0) {
-            Jmsg(NULL, M_ERROR, 0, 
-               _("Media Type not the same for all devices in changer %s. Cannot continue.\n"),
-               changer->hdr.name);
-            OK = false;
-            continue;
          }
-#endif
       }
    }
    return OK;
@@ -107,29 +91,28 @@ bool init_autochangers()
  *           0 on failure (no changer available)
  *          -1 on error on autochanger
  */
-int autoload_device(DCR *dcr, int writing, BSOCK *dir)
+int autoload_device(DCR *dcr, bool writing, BSOCK *dir)
 {
    JCR *jcr = dcr->jcr;
    DEVICE * volatile dev = dcr->dev;
+   char *new_vol_name = dcr->VolumeName;
    int slot;
    int drive = dev->drive_index;
    int rtn_stat = -1;                 /* error status */
    POOLMEM *changer;
 
    if (!dev->is_autochanger()) {
-      Dmsg1(100, "Device %s is not an autochanger\n", dev->print_name());
+      Dmsg1(dbglvl, "Device %s is not an autochanger\n", dev->print_name());
       return 0;
    }
 
    /* An empty ChangerCommand => virtual disk autochanger */
-   if (dcr->device->changer_command && dcr->device->changer_command[0] == 0) {
-      Dmsg0(100, "ChangerCommand=0, virtual disk changer\n");
+   if (dcr->is_virtual_autochanger()) {
+      Dmsg0(dbglvl, "ChangerCommand=0, virtual disk changer\n");
       return 1;                       /* nothing to load */
    }
 
    slot = dcr->VolCatInfo.InChanger ? dcr->VolCatInfo.Slot : 0;
-   Dmsg3(100, "autoload: slot=%d InChgr=%d Vol=%s\n", dcr->VolCatInfo.Slot,
-         dcr->VolCatInfo.InChanger, dcr->getVolCatName());
    /*
     * Handle autoloaders here.  If we cannot autoload it, we
     *  will return 0 so that the sysop will be asked to load it.
@@ -143,15 +126,18 @@ int autoload_device(DCR *dcr, int writing, BSOCK *dir)
          slot = dcr->VolCatInfo.InChanger ? dcr->VolCatInfo.Slot : 0;
       } else {
          slot = 0;
+         dev->clear_wait();
       }
    }
-   Dmsg1(400, "Want changer slot=%d\n", slot);
+   Dmsg4(dbglvl, "Want slot=%d drive=%d InChgr=%d Vol=%s\n",
+         dcr->VolCatInfo.Slot, drive,
+         dcr->VolCatInfo.InChanger, dcr->getVolCatName());
 
    changer = get_pool_memory(PM_FNAME);
    if (slot <= 0) {
       /* Suppress info when polling */
       if (!dev->poll) {
-         Jmsg(jcr, M_INFO, 0, _("No slot defined in catalog (slot=%d) for Volume \"%s\" on %s.\n"), 
+         Jmsg(jcr, M_INFO, 0, _("No slot defined in catalog (slot=%d) for Volume \"%s\" on %s.\n"),
               slot, dcr->getVolCatName(), dev->print_name());
          Jmsg(jcr, M_INFO, 0, _("Cartridge change or \"update slots\" may be required.\n"));
       }
@@ -172,22 +158,25 @@ int autoload_device(DCR *dcr, int writing, BSOCK *dir)
       rtn_stat = 0;
   } else {
       /* Attempt to load the Volume */
-
       uint32_t timeout = dcr->device->max_changer_wait;
       int loaded, status;
 
       loaded = get_autochanger_loaded_slot(dcr);
+      if (loaded < 0) {   /* Autochanger error, try again */
+         loaded = get_autochanger_loaded_slot(dcr);
+      }
+      Dmsg2(dbglvl, "Found loaded=%d drive=%d\n", loaded, drive);
 
-      if (loaded != slot) {
+      if (loaded <= 0 || loaded != slot) {
          POOL_MEM results(PM_MESSAGE);
 
          /* Unload anything in our drive */
          if (!unload_autochanger(dcr, loaded)) {
             goto bail_out;
          }
-            
+
          /* Make sure desired slot is unloaded */
-         if (!unload_other_drive(dcr, slot)) {
+         if (!unload_other_drive(dcr, slot, writing)) {
             goto bail_out;
          }
 
@@ -195,38 +184,48 @@ int autoload_device(DCR *dcr, int writing, BSOCK *dir)
           * Load the desired cassette
           */
          lock_changer(dcr);
-         Dmsg2(100, "Doing changer load slot %d %s\n", slot, dev->print_name());
+         Dmsg2(dbglvl, "Doing changer load slot %d %s\n", slot, dev->print_name());
          Jmsg(jcr, M_INFO, 0,
-              _("3304 Issuing autochanger \"load slot %d, drive %d\" command.\n"),
-              slot, drive);
+            _("3304 Issuing autochanger \"load Volume %s, Slot %d, Drive %d\" command.\n"),
+            new_vol_name, slot, drive);
+         Dmsg3(dbglvl,
+            "3304 Issuing autochanger \"load Volume %s, Slot %d, Drive %d\" command.\n",
+            new_vol_name, slot, drive);
+
          dcr->VolCatInfo.Slot = slot;    /* slot to be loaded */
          changer = edit_device_codes(dcr, changer, dcr->device->changer_command, "load");
-         dev->close();
-         Dmsg1(200, "Run program=%s\n", changer);
+         dev->close(dcr);
+         Dmsg1(dbglvl, "Run program=%s\n", changer);
          status = run_program_full_output(changer, timeout, results.addr());
          if (status == 0) {
-            Jmsg(jcr, M_INFO, 0, _("3305 Autochanger \"load slot %d, drive %d\", status is OK.\n"),
-                    slot, drive);
-            Dmsg2(100, "load slot %d, drive %d, status is OK.\n", slot, drive);
+            Jmsg(jcr, M_INFO, 0, _("3305 Autochanger \"load Volume %s, Slot %d, Drive %d\", status is OK.\n"),
+                    new_vol_name, slot, drive);
+            Dmsg3(dbglvl, "OK: load volume %s, slot %d, drive %d.\n", new_vol_name,  slot, drive);
+            bstrncpy(dev->LoadedVolName, new_vol_name, sizeof(dev->LoadedVolName));
             dev->set_slot(slot);      /* set currently loaded slot */
+            if (dev->vol) {
+               /* We just swapped this Volume so it cannot be swapping any more */
+               dev->vol->clear_swapping();
+            }
          } else {
             berrno be;
             be.set_errno(status);
-            Dmsg3(100, "load slot %d, drive %d, bad stats=%s.\n", slot, drive,
-               be.bstrerror());
-            Jmsg(jcr, M_FATAL, 0, _("3992 Bad autochanger \"load slot %d, drive %d\": "
+            Dmsg5(dbglvl, "Error: load Volume %s, Slot %d, Drive %d, bad stats=%s.\nResults=%s\n",
+                new_vol_name, slot, drive,
+               be.bstrerror(), results.c_str());
+            Jmsg(jcr, M_FATAL, 0, _("3992 Bad autochanger \"load Volume %s Slot %d, Drive %d\": "
                  "ERR=%s.\nResults=%s\n"),
-                    slot, drive, be.bstrerror(), results.c_str());
+                    new_vol_name, slot, drive, be.bstrerror(), results.c_str());
             rtn_stat = -1;            /* hard error */
-            dev->set_slot(-1);        /* mark unknown */
+            dev->clear_slot();        /* mark unknown */
          }
-         Dmsg2(100, "load slot %d status=%d\n", slot, status);
          unlock_changer(dcr);
       } else {
          status = 0;                  /* we got what we want */
          dev->set_slot(slot);         /* set currently loaded slot */
+         bstrncpy(dev->LoadedVolName, new_vol_name, sizeof(dev->LoadedVolName));
       }
-      Dmsg1(100, "After changer, status=%d\n", status);
+      Dmsg1(dbglvl, "After changer, status=%d\n", status);
       if (status == 0) {              /* did we succeed? */
          rtn_stat = 1;                /* tape loaded by changer */
       }
@@ -260,15 +259,16 @@ int get_autochanger_loaded_slot(DCR *dcr)
       return -1;
    }
    if (!dcr->device->changer_command) {
-//    Jmsg(jcr, M_FATAL, 0, _("3992 Missing Changer command.\n"));
       return -1;
    }
-   if (dev->get_slot() > 0) {
+
+   if (dev->get_slot() > 0 && dev->has_cap(CAP_ALWAYSOPEN)) {
+      Dmsg1(dbglvl, "Return cached slot=%d\n", dev->get_slot());
       return dev->get_slot();
    }
 
    /* Virtual disk autochanger */
-   if (dcr->device->changer_command[0] == 0) {
+   if (dcr->is_virtual_autochanger()) {
       return 1;
    }
 
@@ -276,26 +276,26 @@ int get_autochanger_loaded_slot(DCR *dcr)
    changer = get_pool_memory(PM_FNAME);
    lock_changer(dcr);
    /* Suppress info when polling */
-   if (!dev->poll) {
+   if (!dev->poll && chk_dbglvl(1)) {
       Jmsg(jcr, M_INFO, 0, _("3301 Issuing autochanger \"loaded? drive %d\" command.\n"),
            drive);
    }
    changer = edit_device_codes(dcr, changer, dcr->device->changer_command, "loaded");
-   Dmsg1(100, "Run program=%s\n", changer);
+   Dmsg1(dbglvl, "Run program=%s\n", changer);
    status = run_program_full_output(changer, timeout, results.addr());
-   Dmsg3(100, "run_prog: %s stat=%d result=%s", changer, status, results.c_str());
+   Dmsg3(dbglvl, "run_prog: %s stat=%d result=%s", changer, status, results.c_str());
    if (status == 0) {
       loaded = str_to_int32(results.c_str());
       if (loaded > 0) {
          /* Suppress info when polling */
-         if (!dev->poll) {
+         if (!dev->poll && chk_dbglvl(1)) {
             Jmsg(jcr, M_INFO, 0, _("3302 Autochanger \"loaded? drive %d\", result is Slot %d.\n"),
                  drive, loaded);
          }
          dev->set_slot(loaded);
       } else {
          /* Suppress info when polling */
-         if (!dev->poll) {
+         if (!dev->poll && chk_dbglvl(1)) {
             Jmsg(jcr, M_INFO, 0, _("3302 Autochanger \"loaded? drive %d\", result: nothing loaded.\n"),
                  drive);
          }
@@ -310,7 +310,10 @@ int get_autochanger_loaded_slot(DCR *dcr)
       be.set_errno(status);
       Jmsg(jcr, M_INFO, 0, _("3991 Bad autochanger \"loaded? drive %d\" command: "
            "ERR=%s.\nResults=%s\n"), drive, be.bstrerror(), results.c_str());
+      Dmsg3(dbglvl, "Error: autochanger loaded? drive %d "
+           "ERR=%s.\nResults=%s\n", drive, be.bstrerror(), results.c_str());
       loaded = -1;              /* force unload */
+      dev->clear_slot();        /* slot unknown */
    }
    unlock_changer(dcr);
    free_pool_memory(changer);
@@ -322,7 +325,7 @@ static void lock_changer(DCR *dcr)
    AUTOCHANGER *changer_res = dcr->device->changer_res;
    if (changer_res) {
       int errstat;
-      Dmsg1(200, "Locking changer %s\n", changer_res->hdr.name);
+      Dmsg1(dbglvl, "Locking changer %s\n", changer_res->hdr.name);
       if ((errstat=rwl_writelock(&changer_res->changer_lock)) != 0) {
          berrno be;
          Jmsg(dcr->jcr, M_ERROR_TERM, 0, _("Lock failure on autochanger. ERR=%s\n"),
@@ -336,7 +339,7 @@ static void unlock_changer(DCR *dcr)
    AUTOCHANGER *changer_res = dcr->device->changer_res;
    if (changer_res) {
       int errstat;
-      Dmsg1(200, "Unlocking changer %s\n", changer_res->hdr.name);
+      Dmsg1(dbglvl, "Unlocking changer %s\n", changer_res->hdr.name);
       if ((errstat=rwl_writeunlock(&changer_res->changer_lock)) != 0) {
          berrno be;
          Jmsg(dcr->jcr, M_ERROR_TERM, 0, _("Unlock failure on autochanger. ERR=%s\n"),
@@ -355,6 +358,7 @@ bool unload_autochanger(DCR *dcr, int loaded)
 {
    DEVICE *dev = dcr->dev;
    JCR *jcr = dcr->jcr;
+   const char *old_vol_name;
    int slot;
    uint32_t timeout = dcr->device->max_changer_wait;
    bool ok = true;
@@ -369,52 +373,63 @@ bool unload_autochanger(DCR *dcr, int loaded)
    }
 
    /* Virtual disk autochanger */
-   if (dcr->device->changer_command[0] == 0) {
+   if (dcr->is_virtual_autochanger()) {
       dev->clear_unload();
       return true;
    }
 
    lock_changer(dcr);
+   if (dev->LoadedVolName[0]) {
+      old_vol_name = dev->LoadedVolName;
+   } else {
+      old_vol_name = "*Unknown*";
+   }
    if (loaded < 0) {
       loaded = get_autochanger_loaded_slot(dcr);
+      if (loaded < 0) {   /* try again, maybe autochanger error */
+         loaded = get_autochanger_loaded_slot(dcr);
+      }
    }
 
    if (loaded > 0) {
       POOL_MEM results(PM_MESSAGE);
       POOLMEM *changer = get_pool_memory(PM_FNAME);
       Jmsg(jcr, M_INFO, 0,
-           _("3307 Issuing autochanger \"unload slot %d, drive %d\" command.\n"),
-           loaded, dev->drive_index);
+         _("3307 Issuing autochanger \"unload Volume %s, Slot %d, Drive %d\" command.\n"),
+         old_vol_name, loaded, dev->drive_index);
+      Dmsg3(dbglvl,
+         "3307 Issuing autochanger \"unload Volume %s, Slot %d, Drive %d\" command.\n",
+         old_vol_name, loaded, dev->drive_index);
       slot = dcr->VolCatInfo.Slot;
       dcr->VolCatInfo.Slot = loaded;
-      changer = edit_device_codes(dcr, changer, 
+      changer = edit_device_codes(dcr, changer,
                    dcr->device->changer_command, "unload");
-      dev->close();
-      Dmsg1(100, "Run program=%s\n", changer);
+      dev->close(dcr);
+      Dmsg1(dbglvl, "Run program=%s\n", changer);
       int stat = run_program_full_output(changer, timeout, results.addr());
       dcr->VolCatInfo.Slot = slot;
       if (stat != 0) {
          berrno be;
          be.set_errno(stat);
-         Jmsg(jcr, M_INFO, 0, _("3995 Bad autochanger \"unload slot %d, drive %d\": "
+         Jmsg(jcr, M_INFO, 0, _("3995 Bad autochanger \"unload Volume %s, Slot %d, Drive %d\": "
               "ERR=%s\nResults=%s\n"),
-                 loaded, dev->drive_index, be.bstrerror(), results.c_str());
+              old_vol_name, loaded, dev->drive_index, be.bstrerror(), results.c_str());
+         Dmsg5(dbglvl, "Error: unload Volume %s, Slot %d, Drive %d, bad stats=%s.\nResults=%s\n",
+               old_vol_name, loaded, dev->drive_index,
+               be.bstrerror(), results.c_str());
          ok = false;
          dev->clear_slot();        /* unknown */
       } else {
-         dev->set_slot(0);         /* nothing loaded */
+         dev->set_slot(0);         /* unload is OK, mark nothing loaded */
+         dev->clear_unload();
+         dev->LoadedVolName[0] = 0; /* clear loaded volume name */
       }
-
       free_pool_memory(changer);
    }
    unlock_changer(dcr);
 
-   if (loaded > 0) {           /* free_volume outside from changer lock */
-      free_volume(dev);        /* Free any volume associated with this drive */
-   }
-
    if (ok) {
-      dev->clear_unload();
+      free_volume(dev);
    }
    return ok;
 }
@@ -422,7 +437,7 @@ bool unload_autochanger(DCR *dcr, int loaded)
 /*
  * Unload the slot if mounted in a different drive
  */
-static bool unload_other_drive(DCR *dcr, int slot)
+static bool unload_other_drive(DCR *dcr, int slot, bool writing)
 {
    DEVICE *dev = NULL;
    DEVICE *dev_save;
@@ -430,8 +445,10 @@ static bool unload_other_drive(DCR *dcr, int slot)
    AUTOCHANGER *changer = dcr->dev->device->changer_res;
    DEVRES *device;
    int retries = 0;                /* wait for device retries */
+   int loaded;
+   int i;
 
-   if (!changer) {
+   if (!changer || !changer->device) {
       return false;
    }
    if (changer->device->size() == 1) {
@@ -443,52 +460,77 @@ static bool unload_other_drive(DCR *dcr, int slot)
     *   we want in other drives, and if possible, unload
     *   it.
     */
-   Dmsg0(100, "Wiffle through devices looking for slot\n");
-   foreach_alist(device, changer->device) {
+   Dmsg1(dbglvl, "Begin wiffle through devices looking for slot=%d\n", slot);
+   /*
+    *  foreach_alist(device, changer->device) {
+    *
+    * The above fails to loop through all devices. It is
+    * probably a compiler bug.
+    */
+   for (i=0; i < changer->device->size(); i++) {
+      device = (DEVRES *)changer->device->get(i);
       dev = device->dev;
       if (!dev) {
+         Dmsg0(dbglvl, "No dev attached to device\n");
          continue;
       }
+
       dev_save = dcr->dev;
-      dcr->dev = dev;
-      if (dev->get_slot() <= 0 && get_autochanger_loaded_slot(dcr) <= 0) {
-         dcr->dev = dev_save;
-         continue;
-      }
-      dcr->dev = dev_save;
-      if (dev->get_slot() == slot) {
-         found = true;
-         break;
+      dcr->set_dev(dev);
+      loaded = get_autochanger_loaded_slot(dcr);
+      dcr->set_dev(dev_save);
+
+      if (loaded > 0) {
+         Dmsg4(dbglvl, "Want slot=%d, drive=%d loaded=%d dev=%s\n",
+            slot, dev->drive_index, loaded, dev->print_name());
+         if (loaded == slot) {
+            found = true;
+            break;
+         }
+      } else {
+         Dmsg4(dbglvl, "After slot=%d drive=%d loaded=%d dev=%s\n",
+            slot, dev->drive_index, loaded, dev->print_name());
       }
    }
+   Dmsg1(dbglvl, "End wiffle through devices looking for slot=%d\n", slot);
    if (!found) {
-      Dmsg1(100, "Slot=%d not found in another device\n", slot);
+      Dmsg1(dbglvl, "Slot=%d not found in another device\n", slot);
       return true;
    } else {
-      Dmsg1(100, "Slot=%d found in another device\n", slot);
-   }   
-
-   /* The Volume we want is on another device. */
-   if (dev->is_busy()) {
-      Dmsg4(100, "Vol %s for dev=%s in use dev=%s slot=%d\n",
-           dcr->VolumeName, dcr->dev->print_name(),
-           dev->print_name(), slot);
-   }   
-   for (int i=0; i < 3; i++) {
-      if (dev->is_busy()) {
-         wait_for_device(dcr->jcr, retries);
-         continue;
-      }
-      break;
+      Dmsg3(dbglvl, "Slot=%d drive=%d found in dev=%s\n", slot, dev->drive_index, dev->print_name());
    }
-   if (dev->is_busy()) {
-      Jmsg(dcr->jcr, M_WARNING, 0, _("Volume \"%s\" wanted on %s is in use by device %s\n"),
-           dcr->VolumeName, dcr->dev->print_name(), dev->print_name());
-      Dmsg4(100, "Vol %s for dev=%s is busy dev=%s slot=%d\n",
-           dcr->VolumeName, dcr->dev->print_name(), dev->print_name(), dev->get_slot());
-      Dmsg2(100, "num_writ=%d reserv=%d\n", dev->num_writers, dev->num_reserved());
-      volume_unused(dcr);
-      return false;
+
+   /*
+    * The Volume we want is on another device.
+    * If we want the Volume to read it, and the other device where the
+    *   Volume is currently is not open, we simply unload the Volume then
+    *   then the subsequent code will load it in the desired drive.
+    * If want to write or  the device is open, we attempt to wait for
+    *   the Volume to become available.
+    */
+   if (writing || dev->is_open()) {
+      if (dev->is_busy()) {
+         Dmsg4(dbglvl, "Vol %s for dev=%s in use dev=%s slot=%d\n",
+              dcr->VolumeName, dcr->dev->print_name(),
+              dev->print_name(), slot);
+      }
+      for (int i=0; i < 3; i++) {
+         if (dev->is_busy()) {
+            Dmsg0(40, "Device is busy. Calling wait_for_device()\n");
+            wait_for_device(dcr, retries);
+            continue;
+         }
+         break;
+      }
+      if (dev->is_busy()) {
+         Jmsg(dcr->jcr, M_WARNING, 0, _("Volume \"%s\" wanted on %s is in use by device %s\n"),
+              dcr->VolumeName, dcr->dev->print_name(), dev->print_name());
+         Dmsg4(dbglvl, "Vol %s for dev=%s is busy dev=%s slot=%d\n",
+              dcr->VolumeName, dcr->dev->print_name(), dev->print_name(), dev->get_slot());
+         Dmsg2(dbglvl, "num_writ=%d reserv=%d\n", dev->num_writers, dev->num_reserved());
+         volume_unused(dcr);
+         return false;
+      }
    }
    return unload_dev(dcr, dev);
 }
@@ -502,6 +544,7 @@ bool unload_dev(DCR *dcr, DEVICE *dev)
    bool ok = true;
    uint32_t timeout = dcr->device->max_changer_wait;
    AUTOCHANGER *changer = dcr->dev->device->changer_res;
+   const char *old_vol_name = dcr->VolumeName;
    DEVICE *save_dev;
    int save_slot;
 
@@ -510,58 +553,70 @@ bool unload_dev(DCR *dcr, DEVICE *dev)
    }
 
    save_dev = dcr->dev;               /* save dcr device */
-   dcr->dev = dev;                    /* temporarily point dcr at other device */
+   dcr->set_dev(dev);                 /* temporarily point dcr at other device */
 
-   if (dev->get_slot() <= 0 && get_autochanger_loaded_slot(dcr) <= 0) {
-      dcr->dev = save_dev;
+   get_autochanger_loaded_slot(dcr);
+
+   /* Fail if we have no slot to unload */
+   if (dev->get_slot() <= 0) {
+      if (dev->get_slot() < 0) {
+         Dmsg1(dbglvl, "Cannot unload, slot not defined. dev=%s\n",
+            dev->print_name());
+      }
+      dcr->set_dev(save_dev);
       return false;
    }
+
    save_slot = dcr->VolCatInfo.Slot;
    dcr->VolCatInfo.Slot = dev->get_slot();
 
-//   dev->dlock();
-
    POOLMEM *changer_cmd = get_pool_memory(PM_FNAME);
    POOL_MEM results(PM_MESSAGE);
+   if (old_vol_name[0] == 0) {
+      if (dev->LoadedVolName[0]) {
+         old_vol_name = dev->LoadedVolName;
+      } else {
+         old_vol_name = "*Unknown*";
+      }
+   }
    lock_changer(dcr);
    Jmsg(jcr, M_INFO, 0,
-        _("3307 Issuing autochanger \"unload slot %d, drive %d\" command.\n"),
-        dev->get_slot(), dev->drive_index);
+      _("3307 Issuing autochanger \"unload Volume %s, Slot %d, Drive %d\" command.\n"),
+      old_vol_name, dev->get_slot(), dev->drive_index);
+   Dmsg3(0/*dbglvl*/, "Issuing autochanger \"unload Volume %s, Slot %d, Drive %d\" command.\n",
+        old_vol_name, dev->get_slot(), dev->drive_index);
 
-   Dmsg2(100, "Issuing autochanger \"unload slot %d, drive %d\" command.\n",
-        dev->get_slot(), dev->drive_index);
-
-   changer_cmd = edit_device_codes(dcr, changer_cmd, 
+   changer_cmd = edit_device_codes(dcr, changer_cmd,
                 dcr->device->changer_command, "unload");
-   dev->close();
-   Dmsg2(200, "close dev=%s reserve=%d\n", dev->print_name(), 
+   dev->close(dcr);
+   Dmsg2(dbglvl, "close dev=%s reserve=%d\n", dev->print_name(),
       dev->num_reserved());
-   Dmsg1(100, "Run program=%s\n", changer_cmd);
+   Dmsg1(dbglvl, "Run program=%s\n", changer_cmd);
    int stat = run_program_full_output(changer_cmd, timeout, results.addr());
    dcr->VolCatInfo.Slot = save_slot;
-   dcr->dev = save_dev;
    if (stat != 0) {
       berrno be;
       be.set_errno(stat);
-      Jmsg(jcr, M_INFO, 0, _("3995 Bad autochanger \"unload slot %d, drive %d\": ERR=%s.\n"),
-              dev->get_slot(), dev->drive_index, be.bstrerror());
-
-      Dmsg3(100, "Bad autochanger \"unload slot %d, drive %d\": ERR=%s.\n",
-              dev->get_slot(), dev->drive_index, be.bstrerror());
+      Jmsg(jcr, M_INFO, 0, _("3997 Bad autochanger \"unload Volume %s, Slot %d, Drive %d\": ERR=%s.\n"),
+           old_vol_name, dev->get_slot(), dev->drive_index, be.bstrerror());
+      Dmsg5(dbglvl, "Error: unload Volume %s, Slot %d, Drive %d bad stats=%s.\nResults=%s\n",
+            old_vol_name, dev->get_slot(), dev->drive_index,
+            be.bstrerror(), results.c_str());
       ok = false;
       dev->clear_slot();          /* unknown */
    } else {
-      Dmsg2(100, "Slot %d unloaded %s\n", dev->get_slot(), dev->print_name());
-      dev->set_slot(0);           /* nothing loaded */
-   }
-   if (ok) {
+      Dmsg3(dbglvl, "Volume %s, Slot %d unloaded %s\n",
+            old_vol_name, dev->get_slot(), dev->print_name());
+      dev->set_slot(0);           /* unload OK, mark nothing loaded */
       dev->clear_unload();
+      dev->LoadedVolName[0] = 0;
    }
    unlock_changer(dcr);
 
-//   dev->dunlock();
-
-   free_volume(dev);               /* Free any volume associated with this drive */
+   if (ok) {
+      free_volume(dev);
+   }
+   dcr->set_dev(save_dev);
    free_pool_memory(changer_cmd);
    return ok;
 }
@@ -573,7 +628,7 @@ bool unload_dev(DCR *dcr, DEVICE *dev)
  *   with their barcodes.
  *   We assume that it is always the Console that is calling us.
  */
-bool autochanger_cmd(DCR *dcr, BSOCK *dir, const char *cmd)  
+bool autochanger_cmd(DCR *dcr, BSOCK *dir, const char *cmd)
 {
    DEVICE *dev = dcr->dev;
    uint32_t timeout = dcr->device->max_changer_wait;
@@ -584,7 +639,7 @@ bool autochanger_cmd(DCR *dcr, BSOCK *dir, const char *cmd)
 
    if (!dev->is_autochanger() || !dcr->device->changer_name ||
        !dcr->device->changer_command) {
-      if (strcmp(cmd, "drives") == 0) {
+      if (strcasecmp(cmd, "drives") == 0) {
          dir->fsend("drives=1\n");
       }
       dir->fsend(_("3993 Device %s not an autochanger device.\n"),
@@ -592,21 +647,27 @@ bool autochanger_cmd(DCR *dcr, BSOCK *dir, const char *cmd)
       return false;
    }
 
-   if (strcmp(cmd, "drives") == 0) {
+   if (strcasecmp(cmd, "drives") == 0) {
       AUTOCHANGER *changer_res = dcr->device->changer_res;
       int drives = 1;
-      if (changer_res) {
+      if (changer_res && changer_res->device) {
          drives = changer_res->device->size();
       }
       dir->fsend("drives=%d\n", drives);
-      Dmsg1(100, "drives=%d\n", drives);
+      Dmsg1(dbglvl, "drives=%d\n", drives);
       return true;
    }
 
+   /* If listing, reprobe changer */
+   if (bstrcasecmp(cmd, "list") || bstrcasecmp(cmd, "listall")) {
+      dcr->dev->set_slot(0);
+      get_autochanger_loaded_slot(dcr);
+   }
+
    changer = get_pool_memory(PM_FNAME);
    lock_changer(dcr);
    /* Now issue the command */
-   changer = edit_device_codes(dcr, changer, 
+   changer = edit_device_codes(dcr, changer,
                  dcr->device->changer_command, cmd);
    dir->fsend(_("3306 Issuing autochanger \"%s\" command.\n"), cmd);
    bpipe = open_bpipe(changer, timeout, "r");
@@ -614,14 +675,14 @@ bool autochanger_cmd(DCR *dcr, BSOCK *dir, const char *cmd)
       dir->fsend(_("3996 Open bpipe failed.\n"));
       goto bail_out;            /* TODO: check if we need to return false */
    }
-   if (bstrcmp(cmd, "list") || bstrcmp(cmd, "listall")) {
+   if (bstrcasecmp(cmd, "list") || bstrcasecmp(cmd, "listall")) {
       /* Get output from changer */
       while (fgets(dir->msg, len, bpipe->rfd)) {
          dir->msglen = strlen(dir->msg);
-         Dmsg1(100, "<stored: %s\n", dir->msg);
-         bnet_send(dir);
+         Dmsg1(dbglvl, "<stored: %s\n", dir->msg);
+         dir->send();
       }
-   } else if (strcmp(cmd, "slots") == 0 ) {
+   } else if (strcasecmp(cmd, "slots") == 0 ) {
       char buf[100], *p;
       /* For slots command, read a single line */
       buf[0] = 0;
@@ -631,16 +692,15 @@ bool autochanger_cmd(DCR *dcr, BSOCK *dir, const char *cmd)
       for (p=buf; B_ISSPACE(*p); p++)
         { }
       dir->fsend("slots=%s", p);
-      Dmsg1(100, "<stored: %s", dir->msg);
-   } 
-                 
+      Dmsg1(dbglvl, "<stored: %s", dir->msg);
+   }
+
    stat = close_bpipe(bpipe);
    if (stat != 0) {
       berrno be;
       be.set_errno(stat);
       dir->fsend(_("Autochanger error: ERR=%s\n"), be.bstrerror());
    }
-   bnet_sig(dir, BNET_EOD);
 
 bail_out:
    unlock_changer(dcr);
@@ -657,6 +717,7 @@ bail_out:
  *  %d = changer drive index
  *  %f = Client's name
  *  %j = Job name
+ *  %l = archive control channel name
  *  %o = command
  *  %s = Slot base 0
  *  %S = Slot base 1
@@ -688,6 +749,9 @@ char *edit_device_codes(DCR *dcr, char *omsg, const char *imsg, const char *cmd)
          case 'c':
             str = NPRT(dcr->device->changer_name);
             break;
+         case 'l':
+            str = NPRT(dcr->device->control_name);
+            break;
          case 'd':
             sprintf(add, "%d", dcr->dev->drive_index);
             str = add;
