@@ -1,36 +1,26 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2011 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2017 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation and included
-   in the file LICENSE.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   This notice must be preserved when any source code is
+   conveyed and/or propagated.
 
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  *
  *  Dumb program to extract files from a Bacula backup.
  *
  *   Kern E. Sibbald, MM
- *
  */
 
 #include "bacula.h"
@@ -38,6 +28,11 @@
 #include "ch.h"
 #include "findlib/find.h"
 
+#ifdef HAVE_LZO
+#include <lzo/lzoconf.h>
+#include <lzo/lzo1x.h>
+#endif
+
 extern bool parse_sd_config(CONFIG *config, const char *configfile, int exit_code);
 
 static void do_extract(char *fname);
@@ -53,7 +48,10 @@ static bool extract = false;
 static int non_support_data = 0;
 static long total = 0;
 static ATTR *attr;
+static POOLMEM *curr_fname;
 static char *where;
+static uint64_t num_errors = 0;
+static uint64_t num_records = 0;
 static uint32_t num_files = 0;
 static uint32_t compress_buf_size = 70000;
 static POOLMEM *compress_buf;
@@ -67,28 +65,29 @@ static uint64_t fileAddr = 0;         /* file write address */
 
 static CONFIG *config;
 #define CONFIG_FILE "bacula-sd.conf"
+
+void *start_heap;
 char *configfile = NULL;
-STORES *me = NULL;                    /* our Global resource */
-bool forge_on = false;
-pthread_mutex_t device_release_mutex = PTHREAD_MUTEX_INITIALIZER;
-pthread_cond_t wait_device_release = PTHREAD_COND_INITIALIZER;
+bool skip_extract = false;
 
 static void usage()
 {
    fprintf(stderr, _(
 PROG_COPYRIGHT
-"\nVersion: %s (%s)\n\n"
+"\n%sVersion: %s (%s)\n\n"
 "Usage: bextract <options> <bacula-archive-device-name> <directory-to-store-files>\n"
 "       -b <file>       specify a bootstrap file\n"
 "       -c <file>       specify a Storage configuration file\n"
 "       -d <nn>         set debug level to <nn>\n"
 "       -dt             print timestamp in debug output\n"
+"       -T              send debug traces to trace file (stored in /tmp)\n"
 "       -e <file>       exclude list\n"
 "       -i <file>       include list\n"
 "       -p              proceed inspite of I/O errors\n"
+"       -t              read data from volume, do not write anything\n"
 "       -v              verbose\n"
 "       -V <volumes>    specify Volume names (separated by |)\n"
-"       -?              print this message\n\n"), 2000, VERSION, BDATE);
+"       -?              print this message\n\n"), 2000, "", VERSION, BDATE);
    exit(1);
 }
 
@@ -99,7 +98,9 @@ int main (int argc, char *argv[])
    FILE *fd;
    char line[1000];
    bool got_inc = false;
+   BtoolsAskDirHandler askdir_handler;
 
+   init_askdir_handler(&askdir_handler);
    setlocale(LC_ALL, "");
    bindtextdomain("bacula", LOCALEDIR);
    textdomain("bacula");
@@ -115,11 +116,18 @@ int main (int argc, char *argv[])
    ff = init_find_files();
    binit(&bfd);
 
-   while ((ch = getopt(argc, argv, "b:c:d:e:i:pvV:?")) != -1) {
+   while ((ch = getopt(argc, argv, "Ttb:c:d:e:i:pvV:?")) != -1) {
       switch (ch) {
+      case 't':
+         skip_extract = true;
+         break;
+
       case 'b':                    /* bootstrap file */
          bsr = parse_bsr(NULL, optarg);
-//       dump_bsr(bsr, true);
+         break;
+
+      case 'T':                 /* Send debug to trace file */
+         set_trace(1);
          break;
 
       case 'c':                    /* specify config file */
@@ -133,10 +141,18 @@ int main (int argc, char *argv[])
          if (*optarg == 't') {
             dbg_timestamp = true;
          } else {
+            char *p;
+            /* We probably find a tag list -d 10,sql,bvfs */
+            if ((p = strchr(optarg, ',')) != NULL) {
+               *p = 0;
+            }
             debug_level = atoi(optarg);
             if (debug_level <= 0) {
                debug_level = 1;
             }
+            if (p) {
+               debug_parse_tags(p+1, &debug_level_tags);
+            }
          }
          break;
 
@@ -201,8 +217,10 @@ int main (int argc, char *argv[])
       configfile = bstrdup(CONFIG_FILE);
    }
 
-   config = new_config_parser();
+   config = New(CONFIG());
    parse_sd_config(config, configfile, M_ERROR_TERM);
+   setup_me();
+   load_sd_plugins(me->plugin_directory);
 
    if (!got_inc) {                            /* If no include file, */
       add_fname_to_include_list(ff, 0, "/");  /*   include everything */
@@ -229,11 +247,12 @@ int main (int argc, char *argv[])
 
 static void do_extract(char *devname)
 {
+   char ed1[50];
    struct stat statp;
 
    enable_backup_privileges(NULL, 1);
 
-   jcr = setup_jcr("bextract", devname, bsr, VolumeName, 1); /* acquire for read */
+   jcr = setup_jcr("bextract", devname, bsr, VolumeName, SD_READ, true/*read dedup data*/);
    if (!jcr) {
       exit(1);
    }
@@ -258,6 +277,8 @@ static void do_extract(char *devname)
    attr = new_attr(jcr);
 
    compress_buf = get_memory(compress_buf_size);
+   curr_fname = get_pool_memory(PM_FNAME);
+   *curr_fname = 0;
 
    read_records(dcr, record_cb, mount_next_read_volume);
    /* If output file is still open, it was the last one in the
@@ -269,9 +290,13 @@ static void do_extract(char *devname)
    release_device(dcr);
    free_attr(attr);
    free_jcr(jcr);
-   dev->term();
+   dev->term(NULL);
+   free_pool_memory(curr_fname);
 
    printf(_("%u files restored.\n"), num_files);
+   if (num_errors) {
+      printf(_("Found %s error%s\n"), edit_uint64(num_errors, ed1), num_errors>1? "s":"");
+   }
    return;
 }
 
@@ -300,13 +325,47 @@ static bool store_data(BFILE *bfd, char *data, const int32_t length)
  */
 static bool record_cb(DCR *dcr, DEV_RECORD *rec)
 {
-   int stat;
+   int stat, ret=true;
    JCR *jcr = dcr->jcr;
+   char ed1[50];
+
+   bool     restoredatap = false;
+   POOLMEM *orgdata = NULL;
+   uint32_t orgdata_len = 0;
 
    if (rec->FileIndex < 0) {
       return true;                    /* we don't want labels */
    }
 
+   /* In this mode, we do not create any file on disk, just read
+    * everything from the volume.
+    */
+   if (skip_extract) {
+      switch (rec->maskedStream) {
+      case STREAM_UNIX_ATTRIBUTES:
+      case STREAM_UNIX_ATTRIBUTES_EX:
+         if (!unpack_attributes_record(jcr, rec->Stream, rec->data, rec->data_len, attr)) {
+            Emsg0(M_ERROR_TERM, 0, _("Cannot continue.\n"));
+         }
+         if (verbose) {
+            attr->data_stream = decode_stat(attr->attr, &attr->statp, sizeof(attr->statp), &attr->LinkFI);
+            build_attr_output_fnames(jcr, attr);
+            print_ls_output(jcr, attr);
+         }
+         pm_strcpy(curr_fname, attr->fname);
+         num_files++;
+         break;
+      }
+      num_records++;
+
+      /* We display some progress information if verbose not set or set to 2 */
+      if (verbose != 1 && (num_records % 200000) == 0L) {
+         fprintf(stderr, "\rfiles=%d records=%s\n", num_files, edit_uint64(num_records, ed1));
+      }
+      ret = true;
+      goto bail_out;
+   }
+
    /* File Attributes stream */
 
    switch (rec->maskedStream) {
@@ -328,6 +387,9 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
          Emsg0(M_ERROR_TERM, 0, _("Cannot continue.\n"));
       }
 
+      /* Keep the name of the current file if we find a bad block */
+      pm_strcpy(curr_fname, attr->fname);
+
       if (file_is_included(ff, attr->fname) && !file_is_excluded(ff, attr->fname)) {
          attr->data_stream = decode_stat(attr->attr, &attr->statp, sizeof(attr->statp), &attr->LinkFI);
          if (!is_restore_stream_supported(attr->data_stream)) {
@@ -336,7 +398,7 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
                   stream_to_ascii(attr->data_stream));
             }
             extract = false;
-            return true;
+            goto bail_out;
          }
 
          build_attr_output_fnames(jcr, attr);
@@ -344,11 +406,12 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
          if (attr->type == FT_DELETED) { /* TODO: choose the right fname/ofname */
             Jmsg(jcr, M_INFO, 0, _("%s was deleted.\n"), attr->fname);
             extract = false;
-            return true;
+            goto bail_out;
          }
 
          extract = false;
          stat = create_file(jcr, attr, &bfd, REPLACE_ALWAYS);
+
          switch (stat) {
          case CF_ERROR:
          case CF_SKIP:
@@ -377,7 +440,6 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
    case STREAM_FILE_DATA:
    case STREAM_SPARSE_DATA:
    case STREAM_WIN32_DATA:
-
       if (extract) {
          if (rec->maskedStream == STREAM_SPARSE_DATA) {
             ser_declare;
@@ -386,12 +448,13 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
             wsize = rec->data_len - OFFSET_FADDR_SIZE;
             ser_begin(rec->data, OFFSET_FADDR_SIZE);
             unser_uint64(faddr);
-            if (fileAddr != faddr) {
+            /* We seek only for real SPARSE data, not for OFFSET option */
+            if ((rec->Stream & STREAM_BIT_OFFSETS) == 0 && fileAddr != faddr) {
                fileAddr = faddr;
                if (blseek(&bfd, (boffset_t)fileAddr, SEEK_SET) < 0) {
                   berrno be;
-                  Emsg2(M_ERROR_TERM, 0, _("Seek error on %s: %s\n"),
-                     attr->ofname, be.bstrerror());
+                  Emsg3(M_ERROR_TERM, 0, _("Seek error Addr=%llu on %s: %s\n"),
+                     fileAddr, attr->ofname, be.bstrerror());
                }
             }
          } else {
@@ -414,7 +477,7 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
          uLong compress_len = compress_buf_size;
          int stat = Z_BUF_ERROR;
 
-         if (rec->maskedStream == STREAM_SPARSE_GZIP_DATA) {
+         if (rec->maskedStream == STREAM_SPARSE_DATA) {
             ser_declare;
             uint64_t faddr;
             char ec1[50];
@@ -422,14 +485,14 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
             wsize = rec->data_len - OFFSET_FADDR_SIZE;
             ser_begin(rec->data, OFFSET_FADDR_SIZE);
             unser_uint64(faddr);
-            if (fileAddr != faddr) {
+            if ((rec->Stream & STREAM_BIT_OFFSETS) == 0 && fileAddr != faddr) {
                fileAddr = faddr;
                if (blseek(&bfd, (boffset_t)fileAddr, SEEK_SET) < 0) {
                   berrno be;
                   Emsg3(M_ERROR, 0, _("Seek to %s error on %s: ERR=%s\n"),
                      edit_uint64(fileAddr, ec1), attr->ofname, be.bstrerror());
                   extract = false;
-                  return true;
+                  goto bail_out;
                }
             }
          } else {
@@ -447,7 +510,7 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
          if (stat != Z_OK) {
             Emsg1(M_ERROR, 0, _("Uncompression error. ERR=%d\n"), stat);
             extract = false;
-            return true;
+            goto bail_out;
          }
 
          Dmsg2(100, "Write uncompressed %d bytes, total before write=%d\n", compress_len, total);
@@ -461,7 +524,7 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
       if (extract) {
          Emsg0(M_ERROR, 0, _("GZIP data stream found, but GZIP not configured!\n"));
          extract = false;
-         return true;
+         goto bail_out;
       }
 #endif
       break;
@@ -479,7 +542,7 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
          int r, real_compress_len;
 #endif
 
-         if (rec->maskedStream == STREAM_SPARSE_COMPRESSED_DATA) {
+         if (rec->maskedStream == STREAM_SPARSE_DATA) {
             ser_declare;
             uint64_t faddr;
             char ec1[50];
@@ -487,14 +550,14 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
             wsize = rec->data_len - OFFSET_FADDR_SIZE;
             ser_begin(rec->data, OFFSET_FADDR_SIZE);
             unser_uint64(faddr);
-            if (fileAddr != faddr) {
+            if ((rec->Stream & STREAM_BIT_OFFSETS) == 0 && fileAddr != faddr) {
                fileAddr = faddr;
                if (blseek(&bfd, (boffset_t)fileAddr, SEEK_SET) < 0) {
                   berrno be;
                   Emsg3(M_ERROR, 0, _("Seek to %s error on %s: ERR=%s\n"),
                      edit_uint64(fileAddr, ec1), attr->ofname, be.bstrerror());
                   extract = false;
-                  return true;
+                  goto bail_out;
                }
             }
          } else {
@@ -515,13 +578,15 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
          /* version check */
          if (comp_version != COMP_HEAD_VERSION) {
             Emsg1(M_ERROR, 0, _("Compressed header version error. version=0x%x\n"), comp_version);
-            return false;
+            ret = false;
+            goto bail_out;
          }
          /* size check */
          if (comp_len + sizeof(comp_stream_header) != wsize) {
             Emsg2(M_ERROR, 0, _("Compressed header size error. comp_len=%d, msglen=%d\n"),
                  comp_len, wsize);
-            return false;
+            ret = false;
+            goto bail_out;
          }
 
           switch(comp_magic) {
@@ -543,7 +608,7 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
                if (r != LZO_E_OK) {
                   Emsg1(M_ERROR, 0, _("LZO uncompression error. ERR=%d\n"), r);
                   extract = false;
-                  return true;
+                  goto bail_out;
                }
                Dmsg2(100, "Write uncompressed %d bytes, total before write=%d\n", compress_len, total);
                store_data(&bfd, compress_buf, compress_len);
@@ -555,7 +620,7 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
             default:
                Emsg1(M_ERROR, 0, _("Compression algorithm 0x%x found, but not supported!\n"), comp_magic);
                extract = false;
-               return true;
+               goto bail_out;
          }
 
       }
@@ -579,7 +644,9 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
          prog_name_msg++;
       }
       break;
-
+   case STREAM_PLUGIN_NAME:
+      /* Just ignore the plugin name */
+      break;
    default:
       /* If extracting, weird stream (not 1 or 2), close output file anyway */
       if (extract) {
@@ -594,33 +661,10 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
       break;
 
    } /* end switch */
-   return true;
-}
-
-/* Dummies to replace askdir.c */
-bool    dir_find_next_appendable_volume(DCR *dcr) { return 1;}
-bool    dir_update_volume_info(DCR *dcr, bool relabel, bool update_LastWritten) { return 1; }
-bool    dir_create_jobmedia_record(DCR *dcr, bool zero) { return 1; }
-bool    dir_ask_sysop_to_create_appendable_volume(DCR *dcr) { return 1; }
-bool    dir_update_file_attributes(DCR *dcr, DEV_RECORD *rec) { return 1;}
-bool    dir_send_job_status(JCR *jcr) {return 1;}
-
-
-bool dir_ask_sysop_to_mount_volume(DCR *dcr, int /*mode*/)
-{
-   DEVICE *dev = dcr->dev;
-   fprintf(stderr, _("Mount Volume \"%s\" on device %s and press return when ready: "),
-      dcr->VolumeName, dev->print_name());
-   dev->close();
-   getchar();
-   return true;
-}
-
-bool dir_get_volume_info(DCR *dcr, enum get_vol_info_rw  writing)
-{
-   Dmsg0(100, "Fake dir_get_volume_info\n");
-   dcr->setVolCatName(dcr->VolumeName);
-   dcr->VolCatInfo.VolCatParts = find_num_dvd_parts(dcr);
-   Dmsg2(500, "Vol=%s num_parts=%d\n", dcr->getVolCatName(), dcr->VolCatInfo.VolCatParts);
-   return 1;
+bail_out:
+   if (restoredatap) {
+      rec->data = orgdata;
+      rec->data_len = orgdata_len;
+   }
+   return ret;
 }
