@@ -1,29 +1,32 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2014 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2017 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from many
-   others, a complete list can be found in the file AUTHORS.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
    You may use this file and others of this release according to the
    license defined in the LICENSE file, which includes the Affero General
    Public License, v3.0 ("AGPLv3") and some additional permissions and
    terms pursuant to its AGPLv3 Section 7.
 
-   Bacula® is a registered trademark of Kern Sibbald.
+   This notice must be preserved when any source code is
+   conveyed and/or propagated.
+
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  *
  *  Dumb program to do an "ls" of a Bacula 1.0 mortal file.
  *
  *  Kern Sibbald, MM
- *
  */
 
 #include "bacula.h"
 #include "stored.h"
 #include "findlib/find.h"
+#include "lib/cmd_parser.h"
 
 extern bool parse_sd_config(CONFIG *config, const char *configfile, int exit_code);
 
@@ -49,11 +52,8 @@ static CONFIG *config;
 void *start_heap;
 #define CONFIG_FILE "bacula-sd.conf"
 char *configfile = NULL;
-STORES *me = NULL;                    /* our Global resource */
-bool forge_on = false;
-pthread_mutex_t device_release_mutex = PTHREAD_MUTEX_INITIALIZER;
-pthread_cond_t wait_device_release = PTHREAD_COND_INITIALIZER;
-
+bool detect_errors = false;
+int  errors = 0;
 
 static FF_PKT *ff;
 
@@ -63,22 +63,23 @@ static void usage()
 {
    fprintf(stderr, _(
 PROG_COPYRIGHT
-"\nVersion: %s (%s)\n\n"
+"\n%sVersion: %s (%s)\n\n"
 "Usage: bls [options] <device-name>\n"
-"       -b <file>       specify a bootstrap file\n"
-"       -c <file>       specify a Storage configuration file\n"
-"       -d <nn>         set debug level to <nn>\n"
-"       -dt             print timestamp in debug output\n"
-"       -e <file>       exclude list\n"
-"       -i <file>       include list\n"
-"       -j              list jobs\n"
-"       -k              list blocks\n"
-"    (no j or k option) list saved files\n"
-"       -L              dump label\n"
-"       -p              proceed inspite of errors\n"
-"       -v              be verbose\n"
-"       -V              specify Volume names (separated by |)\n"
-"       -?              print this message\n\n"), 2000, VERSION, BDATE);
+"     -b <file>          specify a bootstrap file\n"
+"     -c <file>          specify a Storage configuration file\n"
+"     -d <nn>            set debug level to <nn>\n"
+"     -dt                print timestamp in debug output\n"
+"     -e <file>          exclude list\n"
+"     -i <file>          include list\n"
+"     -j                 list jobs\n"
+"     -k                 list blocks\n"
+"  (no j or k option)    list saved files\n"
+"     -L                 dump label\n"
+"     -p                 proceed inspite of errors\n"
+"     -V                 specify Volume names (separated by |)\n"
+"     -E                 Check records to detect errors\n"
+"     -v                 be verbose\n"
+"     -?                 print this message\n\n"), 2000, "", VERSION, BDATE);
    exit(1);
 }
 
@@ -91,7 +92,9 @@ int main (int argc, char *argv[])
    char *VolumeName= NULL;
    char *bsrName = NULL;
    bool ignore_label_errors = false;
+   BtoolsAskDirHandler askdir_handler;
 
+   init_askdir_handler(&askdir_handler);
    setlocale(LC_ALL, "");
    bindtextdomain("bacula", LOCALEDIR);
    textdomain("bacula");
@@ -106,12 +109,16 @@ int main (int argc, char *argv[])
 
    ff = init_find_files();
 
-   while ((ch = getopt(argc, argv, "b:c:d:e:i:jkLpvV:?")) != -1) {
+   while ((ch = getopt(argc, argv, "b:c:d:e:i:jkLpvV:?EDF:")) != -1) {
       switch (ch) {
       case 'b':
          bsrName = optarg;
          break;
 
+      case 'E':
+         detect_errors = true;
+         break;
+
       case 'c':                    /* specify config file */
          if (configfile != NULL) {
             free(configfile);
@@ -123,10 +130,18 @@ int main (int argc, char *argv[])
          if (*optarg == 't') {
             dbg_timestamp = true;
          } else {
+            char *p;
+            /* We probably find a tag list -d 10,sql,bvfs */
+            if ((p = strchr(optarg, ',')) != NULL) {
+               *p = 0;
+            }
             debug_level = atoi(optarg);
             if (debug_level <= 0) {
                debug_level = 1;
             }
+            if (p) {
+               debug_parse_tags(p+1, &debug_level_tags);
+            }
          }
          break;
 
@@ -203,7 +218,7 @@ int main (int argc, char *argv[])
       configfile = bstrdup(CONFIG_FILE);
    }
 
-   config = new_config_parser();
+   config = New(CONFIG());
    parse_sd_config(config, configfile, M_ERROR_TERM);
    setup_me();
    load_sd_plugins(me->plugin_directory);
@@ -252,17 +267,20 @@ int main (int argc, char *argv[])
    }
    term_include_exclude_files(ff);
    term_find_files(ff);
+
+   if (detect_errors) {
+      return (errors > 0)? 1 : 0;
+   }
    return 0;
 }
 
-
 static void do_close(JCR *jcr)
 {
    release_device(jcr->dcr);
    free_attr(attr);
    free_record(rec);
    free_jcr(jcr);
-   dev->term();
+   dev->term(NULL);
 }
 
 
@@ -273,7 +291,7 @@ static void do_blocks(char *infname)
    char buf1[100], buf2[100];
    for ( ;; ) {
       if (!dcr->read_block_from_device(NO_BLOCK_NUMBER_CHECK)) {
-         Dmsg1(100, "!read_block(): ERR=%s\n", dev->bstrerror());
+         Dmsg1(100, "!read_block(): ERR=%s\n", dev->print_errmsg());
          if (dev->at_eot()) {
             if (!mount_next_read_volume(dcr)) {
                Jmsg(jcr, M_INFO, 0, _("Got EOM at file %u on device %s, Volume \"%s\"\n"),
@@ -294,10 +312,11 @@ static void do_blocks(char *infname)
             Dmsg0(20, "read_record got eof. try again\n");
             continue;
          } else if (dev->is_short_block()) {
-            Jmsg(jcr, M_INFO, 0, "%s", dev->errmsg);
+            Jmsg(jcr, M_INFO, 0, "%s", dev->print_errmsg());
             continue;
          } else {
             /* I/O error */
+            errors++;
             display_tape_error_status(jcr, dev);
             break;
          }
@@ -313,16 +332,19 @@ static void do_blocks(char *infname)
         block->VolSessionId, block->VolSessionTime);
       if (verbose == 1) {
          read_record_from_block(dcr, rec);
-         Pmsg9(-1, _("File:blk=%u:%u blk_num=%u blen=%u First rec FI=%s SessId=%u SessTim=%u Strm=%s rlen=%d\n"),
-              dev->file, dev->block_num,
+         Pmsg8(-1, "Addr=%llu blk_num=%u blen=%u First rec FI=%s SessId=%u SessTim=%u Strm=%s rlen=%d\n",
+              dev->get_full_addr(),
               block->BlockNumber, block->block_len,
               FI_to_ascii(buf1, rec->FileIndex), rec->VolSessionId, rec->VolSessionTime,
-              stream_to_ascii(buf2, rec->Stream, rec->FileIndex), rec->data_len);
+              stream_to_ascii_ex(buf2, rec->Stream, rec->FileIndex), rec->data_len);
          rec->remainder = 0;
-      } else if (verbose > 1) {
-         dump_block(block, "");
+      } else if (verbose > 1) {   /* detailed block dump */
+         Pmsg5(-1, "Blk=%u blen=%u bVer=%d SessId=%u SessTim=%u\n",
+           block->BlockNumber, block->block_len, block->BlockVer,
+           block->VolSessionId, block->VolSessionTime);
+         dump_block(dcr->dev, block, "", true);
       } else {
-         printf(_("Block: %d size=%d\n"), block->BlockNumber, block->block_len);
+         printf("Block: %d size=%d\n", block->BlockNumber, block->block_len);
       }
 
    }
@@ -335,7 +357,7 @@ static void do_blocks(char *infname)
 static bool jobs_cb(DCR *dcr, DEV_RECORD *rec)
 {
    if (rec->FileIndex < 0) {
-      dump_label_record(dcr->dev, rec, verbose);
+      dump_label_record(dcr->dev, rec, verbose, detect_errors);
    }
    rec->remainder = 0;
    return true;
@@ -344,36 +366,34 @@ static bool jobs_cb(DCR *dcr, DEV_RECORD *rec)
 /* Do list job records */
 static void do_jobs(char *infname)
 {
-   read_records(dcr, jobs_cb, mount_next_read_volume);
+   if (!read_records(dcr, jobs_cb, mount_next_read_volume)) {
+      errors++;
+   }
 }
 
 /* Do an ls type listing of an archive */
 static void do_ls(char *infname)
 {
    if (dump_label) {
-      dump_volume_label(dev);
+      dev->dump_volume_label();
       return;
    }
-   read_records(dcr, record_cb, mount_next_read_volume);
+   if (!read_records(dcr, record_cb, mount_next_read_volume)) {
+      errors++;
+   }
    printf("%u files found.\n", num_files);
 }
 
+
 /*
  * Called here for each record from read_records()
  */
 static bool record_cb(DCR *dcr, DEV_RECORD *rec)
 {
    if (verbose && rec->FileIndex < 0) {
-      dump_label_record(dcr->dev, rec, verbose);
+      dump_label_record(dcr->dev, rec, verbose, false);
       return true;
    }
-   if (verbose) {
-      char buf1[100], buf2[100];
-      Pmsg6(000, "Record: FI=%s SessId=%d Strm=%s len=%u remlen=%d data_len=%d\n",
-         FI_to_ascii(buf1, rec->FileIndex), rec->VolSessionId,
-         stream_to_ascii(buf2, rec->Stream, rec->FileIndex), rec->data_bytes, rec->remlen,
-         rec->data_len);
-   }
 
    /* File Attributes stream */
    if (rec->maskedStream == STREAM_UNIX_ATTRIBUTES ||
@@ -399,13 +419,17 @@ static bool record_cb(DCR *dcr, DEV_RECORD *rec)
          print_ls_output(jcr, attr);
          num_files++;
       }
-   } else if (rec->Stream == STREAM_PLUGIN_NAME) {
+   } else if (rec->maskedStream == STREAM_PLUGIN_NAME) {
       char data[100];
       int len = MIN(rec->data_len+1, sizeof(data));
       bstrncpy(data, rec->data, len);
       Dmsg1(100, "Plugin data: %s\n", data);
-   } else if (rec->Stream == STREAM_RESTORE_OBJECT) {
+   } else if (rec->maskedStream == STREAM_RESTORE_OBJECT) {
       Dmsg0(100, "Restore Object record\n");
+   } else if (rec->maskedStream == STREAM_ADATA_BLOCK_HEADER) {
+      Dmsg0(000, "Adata block header\n");
+   } else if (rec->maskedStream == STREAM_ADATA_RECORD_HEADER) {
+      Dmsg0(000, "Adata record header\n");
    }
 
    return true;
@@ -458,35 +482,3 @@ static void get_session_record(DEVICE *dev, DEV_RECORD *rec, SESSION_LABEL *sess
             rtype, rec->VolSessionId, rec->VolSessionTime, rec->Stream, rec->data_len);
    }
 }
-
-
-/* Dummies to replace askdir.c */
-bool    dir_find_next_appendable_volume(DCR *dcr) { return 1;}
-bool    dir_update_volume_info(DCR *dcr, bool relabel, bool update_LastWritten) { return 1; }
-bool    dir_create_jobmedia_record(DCR *dcr, bool zero) { return 1; }
-bool    dir_ask_sysop_to_create_appendable_volume(DCR *dcr) { return 1; }
-bool    dir_update_file_attributes(DCR *dcr, DEV_RECORD *rec) { return 1;}
-bool    dir_send_job_status(JCR *jcr) {return 1;}
-int     generate_job_event(JCR *jcr, const char *event) { return 1; }
-
-
-bool dir_ask_sysop_to_mount_volume(DCR *dcr, bool /*writing*/)
-{
-   DEVICE *dev = dcr->dev;
-   fprintf(stderr, _("Mount Volume \"%s\" on device %s and press return when ready: "),
-      dcr->VolumeName, dev->print_name());
-   dev->close();
-   getchar();
-   return true;
-}
-
-bool dir_get_volume_info(DCR *dcr, enum get_vol_info_rw  writing)
-{
-   Dmsg0(100, "Fake dir_get_volume_info\n");
-   dcr->setVolCatName(dcr->VolumeName);
-#ifdef BUILD_DVD
-   dcr->VolCatInfo.VolCatParts = find_num_dvd_parts(dcr);
-#endif
-   Dmsg2(500, "Vol=%s num_parts=%d\n", dcr->getVolCatName(), dcr->VolCatInfo.VolCatParts);
-   return 1;
-}
