@@ -1,29 +1,20 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2012 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2017 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation and included
-   in the file LICENSE.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   This notice must be preserved when any source code is
+   conveyed and/or propagated.
 
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  *
@@ -39,7 +30,7 @@
  *        Obviously, no zzz_dev() is allowed to call
  *        a www_device() or everything falls apart.
  *
- * Concerning the routines dev->r_lock()() and block_device()
+ * Concerning the routines dev->rLock()() and block_device()
  *  see the end of this module for details.  In general,
  *  blocking a device leaves it in a state where all threads
  *  other than the current thread block when they attempt to
@@ -53,7 +44,6 @@
  *
  *   Kern Sibbald, MM, MMI
  *
- *   Version $Id$
  */
 
 #include "bacula.h"                   /* pull in global headers */
@@ -76,7 +66,7 @@
  * We enter with device locked, and
  *     exit with device locked.
  *
- * Note, we are called only from one place in block.c for the daemons.  
+ * Note, we are called only from one place in block.c for the daemons.
  *     The btape utility calls it from btape.c.
  *
  *  Returns: true  on success
@@ -85,20 +75,27 @@
 bool fixup_device_block_write_error(DCR *dcr, int retries)
 {
    char PrevVolName[MAX_NAME_LENGTH];
-   DEV_BLOCK *label_blk;
    DEV_BLOCK *block = dcr->block;
+   DEV_BLOCK *ameta_block = dcr->ameta_block;
+   DEV_BLOCK *adata_block = dcr->adata_block;
    char b1[30], b2[30];
    time_t wait_time;
    char dt[MAX_TIME_LENGTH];
    JCR *jcr = dcr->jcr;
-   DEVICE *dev = dcr->dev;
-   int blocked = dev->blocked();         /* save any previous blocked status */
+   DEVICE *dev;
+   int blocked;              /* save any previous blocked status */
    bool ok = false;
+   bool save_adata = dcr->dev->adata;
+
+   Enter(100);
+   if (save_adata) {
+      dcr->set_ameta();      /* switch to working with ameta */
+   }
+   dev = dcr->dev;
+   blocked = dev->blocked();
 
    wait_time = time(NULL);
 
-   Dmsg0(100, "=== Enter fixup_device_block_write_error\n");
-
    /*
     * If we are blocked at entry, unblock it, and set our own block status
     */
@@ -108,13 +105,16 @@ bool fixup_device_block_write_error(DCR *dcr, int retries)
    block_device(dev, BST_DOING_ACQUIRE);
 
    /* Continue unlocked, but leave BLOCKED */
-   dev->dunlock();
+   dev->Unlock();
 
    bstrncpy(PrevVolName, dev->getVolCatName(), sizeof(PrevVolName));
    bstrncpy(dev->VolHdr.PrevVolumeName, PrevVolName, sizeof(dev->VolHdr.PrevVolumeName));
 
-   label_blk = new_block(dev);
-   dcr->block = label_blk;
+   /* create temporary block, that will be released at the end, current blocks
+    * have been saved in local DEV_BLOCK above and will be restored before to
+    * leave the function
+    */
+   dev->new_dcr_blocks(dcr);
 
    /* Inform User about end of medium */
    Jmsg(jcr, M_INFO, 0, _("End of medium on Volume \"%s\" Bytes=%s Blocks=%s at %s.\n"),
@@ -122,19 +122,32 @@ bool fixup_device_block_write_error(DCR *dcr, int retries)
         edit_uint64_with_commas(dev->VolCatInfo.VolCatBlocks, b2),
         bstrftime(dt, sizeof(dt), time(NULL)));
 
-   Dmsg1(050, "set_unload dev=%s\n", dev->print_name());
+   Dmsg1(150, "set_unload dev=%s\n", dev->print_name());
    dev->set_unload();
+
+   /* Clear DCR Start/End Block/File positions */
+   dcr->VolFirstIndex = dcr->VolLastIndex = 0;
+   dcr->StartAddr = dcr->EndAddr = 0;
+   dcr->VolMediaId = 0;
+   dcr->WroteVol = false;
+
    if (!dcr->mount_next_write_volume()) {
-      free_block(label_blk);
+      dev->free_dcr_blocks(dcr);
       dcr->block = block;
-      dev->dlock();  
+      dcr->ameta_block = ameta_block;
+      dcr->adata_block = adata_block;
+      dev->Lock();
       goto bail_out;
    }
-   Dmsg2(050, "must_unload=%d dev=%s\n", dev->must_unload(), dev->print_name());
-   dev->dlock();                    /* lock again */
+   Dmsg2(150, "must_unload=%d dev=%s\n", dev->must_unload(), dev->print_name());
+
+   dev->notify_newvol_in_attached_dcrs(dcr->VolumeName);
+   dev->Lock();                    /* lock again */
 
    dev->VolCatInfo.VolCatJobs++;              /* increment number of jobs on vol */
-   dir_update_volume_info(dcr, false, false); /* send Volume info to Director */
+   if (!dir_update_volume_info(dcr, false, false)) { /* send Volume info to Director */
+      goto bail_out;
+   }
 
    Jmsg(jcr, M_INFO, 0, _("New volume \"%s\" mounted on device %s at %s.\n"),
       dcr->VolumeName, dev->print_name(), bstrftime(dt, sizeof(dt), time(NULL)));
@@ -150,28 +163,16 @@ bool fixup_device_block_write_error(DCR *dcr, int retries)
       berrno be;
       Pmsg1(0, _("write_block_to_device Volume label failed. ERR=%s"),
         be.bstrerror(dev->dev_errno));
-      free_block(label_blk);
+      dev->free_dcr_blocks(dcr);
       dcr->block = block;
+      dcr->ameta_block = ameta_block;
+      dcr->adata_block = adata_block;
       goto bail_out;
    }
-   free_block(label_blk);
+   dev->free_dcr_blocks(dcr);
    dcr->block = block;
-
-   /*
-    * Walk through all attached jcrs indicating the volume has changed
-    */
-   Dmsg1(100, "Notify vol change. Volume=%s\n", dev->getVolCatName());
-   DCR *mdcr;
-   foreach_dlist(mdcr, dev->attached_dcrs) {
-      JCR *mjcr = mdcr->jcr;
-      if (mjcr->JobId == 0) {
-         continue;                 /* ignore console */
-      }
-      mdcr->NewVol = true;
-      if (jcr != mjcr) {
-         bstrncpy(mdcr->VolumeName, dcr->VolumeName, sizeof(mdcr->VolumeName));
-      }
-   }
+   dcr->ameta_block = ameta_block;
+   dcr->adata_block = adata_block;
 
    /* Clear NewVol now because dir_get_volume_info() already done */
    jcr->dcr->NewVol = false;
@@ -181,13 +182,16 @@ bool fixup_device_block_write_error(DCR *dcr, int retries)
 
    /* Write overflow block to device */
    Dmsg0(190, "Write overflow block to dev\n");
+   if (save_adata) {
+      dcr->set_adata();      /* try to write block we entered with */
+   }
    if (!dcr->write_block_to_dev()) {
       berrno be;
       Dmsg1(0, _("write_block_to_device overflow block failed. ERR=%s"),
         be.bstrerror(dev->dev_errno));
       /* Note: recursive call */
       if (retries-- <= 0 || !fixup_device_block_write_error(dcr, retries)) {
-         Jmsg2(jcr, M_FATAL, 0, 
+         Jmsg2(jcr, M_FATAL, 0,
               _("Catastrophic error. Cannot write overflow block to device %s. ERR=%s"),
               dev->print_name(), be.bstrerror(dev->dev_errno));
          goto bail_out;
@@ -196,6 +200,9 @@ bool fixup_device_block_write_error(DCR *dcr, int retries)
    ok = true;
 
 bail_out:
+   if (save_adata) {
+      dcr->set_ameta();   /* Do unblock ... on ameta */
+   }
    /*
     * At this point, the device is locked and blocked.
     * Unblock the device, restore any entry blocked condition, then
@@ -205,6 +212,9 @@ bail_out:
    if (blocked != BST_NOT_BLOCKED) {
       block_device(dev, blocked);
    }
+   if (save_adata) {
+      dcr->set_adata();      /* switch back to what we entered with */
+   }
    return ok;                               /* device locked */
 }
 
@@ -213,11 +223,17 @@ void set_start_vol_position(DCR *dcr)
    DEVICE *dev = dcr->dev;
    /* Set new start position */
    if (dev->is_tape()) {
-      dcr->StartBlock = dev->block_num;
-      dcr->StartFile = dev->file;
+      dcr->StartAddr = dcr->EndAddr = dev->get_full_addr();
    } else {
-      dcr->StartBlock = (uint32_t)dev->file_addr;
-      dcr->StartFile  = (uint32_t)(dev->file_addr >> 32);
+      if (dev->adata) {
+         dev = dcr->ameta_dev;
+      }
+      /*
+       * Note: we only update the DCR values for ameta blocks
+       *  because all the indexing (JobMedia) is done with
+       *  ameta blocks/records, which may point to adata.
+       */
+      dcr->StartAddr = dcr->EndAddr = dev->get_full_addr();
    }
 }
 
@@ -229,8 +245,17 @@ void set_start_vol_position(DCR *dcr)
 void set_new_volume_parameters(DCR *dcr)
 {
    JCR *jcr = dcr->jcr;
-   if (dcr->NewVol && !dir_get_volume_info(dcr, GET_VOL_INFO_FOR_WRITE)) {
-      Jmsg1(jcr, M_ERROR, 0, "%s", jcr->errmsg);
+   Dmsg1(40, "set_new_volume_parameters dev=%s\n", dcr->dev->print_name());
+   if (dcr->NewVol) {
+      while (dcr->VolumeName[0] == 0) {
+         int retries = 5;
+         wait_for_device(dcr, retries);
+      }
+      if (dir_get_volume_info(dcr, dcr->VolumeName, GET_VOL_INFO_FOR_WRITE)) {
+         dcr->dev->clear_wait();
+      } else {
+         Dmsg1(40, "getvolinfo failed. No new Vol: %s", jcr->errmsg);
+      }
    }
    set_new_file_parameters(dcr);
    jcr->NumWriteVolumes++;
@@ -247,6 +272,8 @@ void set_new_file_parameters(DCR *dcr)
    set_start_vol_position(dcr);
 
    /* Reset indicies */
+   Dmsg3(1000, "Reset indices Vol=%s were: FI=%d LI=%d\n", dcr->VolumeName,
+      dcr->VolFirstIndex, dcr->VolLastIndex);
    dcr->VolFirstIndex = 0;
    dcr->VolLastIndex = 0;
    dcr->NewFile = false;
@@ -279,7 +306,7 @@ bool first_open_device(DCR *dcr)
       return false;
    }
 
-   dev->r_dlock();
+   dev->rLock(false);
 
    /* Defer opening files */
    if (!dev->is_tape()) {
@@ -287,49 +314,15 @@ bool first_open_device(DCR *dcr)
       goto bail_out;
    }
 
-    int mode;
-    if (dev->has_cap(CAP_STREAM)) {
-       mode = OPEN_WRITE_ONLY;
-    } else {
-       mode = OPEN_READ_ONLY;
-    }
    Dmsg0(129, "Opening device.\n");
-   if (!dev->open(dcr, mode)) {
-      Emsg1(M_FATAL, 0, _("dev open failed: %s\n"), dev->errmsg);
+   if (!dev->open_device(dcr, OPEN_READ_ONLY)) {
+      Jmsg1(NULL, M_FATAL, 0, _("dev open failed: %s\n"), dev->errmsg);
       ok = false;
       goto bail_out;
    }
    Dmsg1(129, "open dev %s OK\n", dev->print_name());
 
 bail_out:
-   dev->dunlock();
+   dev->rUnlock();
    return ok;
 }
-
-/*
- * Make sure device is open, if not do so
- */
-bool open_device(DCR *dcr)
-{
-   DEVICE *dev = dcr->dev;
-   /* Open device */
-   int mode;
-   if (dev->has_cap(CAP_STREAM)) {
-      mode = OPEN_WRITE_ONLY;
-   } else {
-      mode = OPEN_READ_WRITE;
-   }
-   if (!dev->open(dcr, mode)) {
-      /* If polling, ignore the error */
-      /* If DVD, also ignore the error, very often you cannot open the device
-       * (when there is no DVD, or when the one inserted is a wrong one) */
-      if (!dev->poll && !dev->is_dvd() && !dev->is_removable()) {
-         Jmsg2(dcr->jcr, M_FATAL, 0, _("Unable to open device %s: ERR=%s\n"),
-            dev->print_name(), dev->bstrerror());
-         Pmsg2(000, _("Unable to open archive %s: ERR=%s\n"), 
-            dev->print_name(), dev->bstrerror());
-      }
-      return false;
-   }
-   return true;
-}
