@@ -1,29 +1,20 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2007-2011 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2017 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation, which is 
-   listed in the file LICENSE.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   This notice must be preserved when any source code is
+   conveyed and/or propagated.
 
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  * Main program to test loading and running Bacula plugins.
@@ -35,7 +26,7 @@
 #include "stored.h"
 #include "sd_plugins.h"
 
-const int dbglvl = 50;
+const int dbglvl = 250;
 const char *plugin_type = "-sd.so";
 
 
@@ -47,7 +38,7 @@ static bRC baculaJobMsg(bpContext *ctx, const char *file, int line,
   int type, utime_t mtime, const char *fmt, ...);
 static bRC baculaDebugMsg(bpContext *ctx, const char *file, int line,
   int level, const char *fmt, ...);
-static char *baculaEditDeviceCodes(DCR *dcr, char *omsg, 
+static char *baculaEditDeviceCodes(DCR *dcr, char *omsg,
   const char *imsg, const char *cmd);
 static bool is_plugin_compatible(Plugin *plugin);
 
@@ -67,10 +58,10 @@ static bsdFuncs bfuncs = {
    baculaSetValue,
    baculaJobMsg,
    baculaDebugMsg,
-   baculaEditDeviceCodes
+   baculaEditDeviceCodes,
 };
 
-/* 
+/*
  * Bacula private context
  */
 struct bacula_ctx {
@@ -97,18 +88,18 @@ static bool is_plugin_disabled(JCR *jcr)
 #endif
 
 /*
- * Create a plugin event 
+ * Create a plugin event
  */
 int generate_plugin_event(JCR *jcr, bsdEventType eventType, void *value)
 {
    bpContext *plugin_ctx;
    bsdEvent event;
    Plugin *plugin;
-   int i;
+   int i = 0;
    bRC rc = bRC_OK;
 
-   if (!bplugin_list) {
-      Dmsg0(dbglvl, "No bplugin_list: generate_plugin_event ignored.\n");
+   if (!b_plugin_list) {
+      Dmsg0(dbglvl, "No b_plugin_list: generate_plugin_event ignored.\n");
       return bRC_OK;
    }
    if (!jcr) {
@@ -116,12 +107,20 @@ int generate_plugin_event(JCR *jcr, bsdEventType eventType, void *value)
       return bRC_OK;
    }
    if (!jcr->plugin_ctx_list) {
-      Dmsg0(dbglvl, "No plugin_ctx_list: generate_plugin_event ignored.\n");
-      return bRC_OK;                  /* Return if no plugins loaded */
+       Dmsg0(dbglvl, "No plugin_ctx_list: generate_plugin_event ignored.\n");
+       return bRC_OK;                  /* Return if no plugins loaded */
    }
-   if (jcr->is_job_canceled()) {
-      Dmsg0(dbglvl, "Cancel return from generate_plugin_event\n");
-      return bRC_Cancel;
+
+   /* Always handle JobEnd and DeviceClose requests */
+   switch (eventType) {
+   case bsdEventJobEnd:
+   case bsdEventDeviceClose:
+      break;               /* pass these through even if canceled */
+   default:
+      if (jcr->is_job_canceled()) {
+         Dmsg0(dbglvl, "Cancel return from generate_plugin_event\n");
+         return bRC_Cancel;
+      }
    }
 
    bpContext *plugin_ctx_list = (bpContext *)jcr->plugin_ctx_list;
@@ -129,7 +128,7 @@ int generate_plugin_event(JCR *jcr, bsdEventType eventType, void *value)
 
    Dmsg2(dbglvl, "sd-plugin_ctx_list=%p JobId=%d\n", jcr->plugin_ctx_list, jcr->JobId);
 
-   foreach_alist_index(i, plugin, bplugin_list) {
+   foreach_alist_index(i, plugin, b_plugin_list) {
       plugin_ctx = &plugin_ctx_list[i];
       if (is_plugin_disabled(plugin_ctx)) {
          continue;
@@ -139,10 +138,38 @@ int generate_plugin_event(JCR *jcr, bsdEventType eventType, void *value)
          break;
       }
    }
-
    return rc;
 }
 
+
+/*
+ * Create a global plugin event -- i.e. no context
+ */
+int generate_global_plugin_event(bsdGlobalEventType eventType, void *value)
+{
+   bsdEvent event;
+   Plugin *plugin;
+   int i;
+   bRC rc = bRC_OK;
+
+   if (!b_plugin_list) {
+      Dmsg0(dbglvl, "No b_plugin_list: generate_global_plugin_event ignored.\n");
+      return bRC_OK;
+   }
+   event.eventType = eventType;
+
+   foreach_alist_index(i, plugin, b_plugin_list) {
+      if (sdplug_func(plugin)->handleGlobalPluginEvent != NULL) {
+         rc = sdplug_func(plugin)->handleGlobalPluginEvent(&event, value);
+         if (rc != bRC_OK) {
+            break;
+         }
+      }
+   }
+   return rc;
+}
+
+
 /*
  * Print to file the plugin info.
  */
@@ -175,27 +202,27 @@ void load_sd_plugins(const char *plugin_dir)
       Dmsg0(dbglvl, "No sd plugin dir!\n");
       return;
    }
-   bplugin_list = New(alist(10, not_owned_by_alist));
-   if (!load_plugins((void *)&binfo, (void *)&bfuncs, plugin_dir, plugin_type, 
+   b_plugin_list = New(alist(10, not_owned_by_alist));
+   if (!load_plugins((void *)&binfo, (void *)&bfuncs, plugin_dir, plugin_type,
                 is_plugin_compatible)) {
       /* Either none found, or some error */
-      if (bplugin_list->size() == 0) {
-         delete bplugin_list;
-         bplugin_list = NULL;
+      if (b_plugin_list->size() == 0) {
+         delete b_plugin_list;
+         b_plugin_list = NULL;
          Dmsg0(dbglvl, "No plugins loaded\n");
          return;
       }
    }
-   /* 
+   /*
     * Verify that the plugin is acceptable, and print information
     *  about it.
     */
-   foreach_alist_index(i, plugin, bplugin_list) {
+   foreach_alist_index(i, plugin, b_plugin_list) {
       Jmsg(NULL, M_INFO, 0, _("Loaded plugin: %s\n"), plugin->file);
       Dmsg1(dbglvl, "Loaded plugin: %s\n", plugin->file);
    }
 
-   Dmsg1(dbglvl, "num plugins=%d\n", bplugin_list->size());
+   Dmsg1(dbglvl, "num plugins=%d\n", b_plugin_list->size());
    dbg_plugin_add_hook(dump_sd_plugin);
 }
 
@@ -207,13 +234,13 @@ static bool is_plugin_compatible(Plugin *plugin)
 {
    psdInfo *info = (psdInfo *)plugin->pinfo;
    Dmsg0(50, "is_plugin_compatible called\n");
-   if (debug_level >= 50) {
+   if (chk_dbglvl(50)) {
       dump_sd_plugin(plugin, stdin);
    }
    if (strcmp(info->plugin_magic, SD_PLUGIN_MAGIC) != 0) {
       Jmsg(NULL, M_ERROR, 0, _("Plugin magic wrong. Plugin=%s wanted=%s got=%s\n"),
            plugin->file, SD_PLUGIN_MAGIC, info->plugin_magic);
-      Dmsg3(50, "Plugin magic wrong. Plugin=%s wanted=%s got=%s\n",
+      Dmsg3(000, "Plugin magic wrong. Plugin=%s wanted=%s got=%s\n",
            plugin->file, SD_PLUGIN_MAGIC, info->plugin_magic);
 
       return false;
@@ -221,7 +248,7 @@ static bool is_plugin_compatible(Plugin *plugin)
    if (info->version != SD_PLUGIN_INTERFACE_VERSION) {
       Jmsg(NULL, M_ERROR, 0, _("Plugin version incorrect. Plugin=%s wanted=%d got=%d\n"),
            plugin->file, SD_PLUGIN_INTERFACE_VERSION, info->version);
-      Dmsg3(50, "Plugin version incorrect. Plugin=%s wanted=%d got=%d\n",
+      Dmsg3(000, "Plugin version incorrect. Plugin=%s wanted=%d got=%d\n",
            plugin->file, SD_PLUGIN_INTERFACE_VERSION, info->version);
       return false;
    }
@@ -230,7 +257,7 @@ static bool is_plugin_compatible(Plugin *plugin)
        strcmp(info->plugin_license, "Bacula Systems(R) SA") != 0) {
       Jmsg(NULL, M_ERROR, 0, _("Plugin license incompatible. Plugin=%s license=%s\n"),
            plugin->file, info->plugin_license);
-      Dmsg2(50, "Plugin license incompatible. Plugin=%s license=%s\n",
+      Dmsg2(000, "Plugin license incompatible. Plugin=%s license=%s\n",
            plugin->file, info->plugin_license);
       return false;
    }
@@ -240,7 +267,7 @@ static bool is_plugin_compatible(Plugin *plugin)
            plugin->file, sizeof(psdInfo), info->size);
       return false;
    }
-      
+
    return true;
 }
 
@@ -251,10 +278,10 @@ static bool is_plugin_compatible(Plugin *plugin)
 void new_plugins(JCR *jcr)
 {
    Plugin *plugin;
-   int i;
+   int i = 0;;
 
    Dmsg0(dbglvl, "=== enter new_plugins ===\n");
-   if (!bplugin_list) {
+   if (!b_plugin_list) {
       Dmsg0(dbglvl, "No sd plugin list!\n");
       return;
    }
@@ -268,7 +295,7 @@ void new_plugins(JCR *jcr)
       return;
    }
 
-   int num = bplugin_list->size();
+   int num = b_plugin_list->size();
 
    Dmsg1(dbglvl, "sd-plugin-list size=%d\n", num);
    if (num == 0) {
@@ -279,7 +306,7 @@ void new_plugins(JCR *jcr)
 
    bpContext *plugin_ctx_list = jcr->plugin_ctx_list;
    Dmsg2(dbglvl, "Instantiate sd-plugin_ctx_list=%p JobId=%d\n", jcr->plugin_ctx_list, jcr->JobId);
-   foreach_alist_index(i, plugin, bplugin_list) {
+   foreach_alist_index(i, plugin, b_plugin_list) {
       /* Start a new instance of each plugin */
       bacula_ctx *b_ctx = (bacula_ctx *)malloc(sizeof(bacula_ctx));
       memset(b_ctx, 0, sizeof(bacula_ctx));
@@ -298,15 +325,15 @@ void new_plugins(JCR *jcr)
 void free_plugins(JCR *jcr)
 {
    Plugin *plugin;
-   int i;
+   int i = 0;
 
-   if (!bplugin_list || !jcr->plugin_ctx_list) {
+   if (!b_plugin_list || !jcr->plugin_ctx_list) {
       return;
    }
 
    bpContext *plugin_ctx_list = (bpContext *)jcr->plugin_ctx_list;
    Dmsg2(dbglvl, "Free instance sd-plugin_ctx_list=%p JobId=%d\n", jcr->plugin_ctx_list, jcr->JobId);
-   foreach_alist_index(i, plugin, bplugin_list) {
+   foreach_alist_index(i, plugin, b_plugin_list) {
       /* Free the plugin instance */
       sdplug_func(plugin)->freePlugin(&plugin_ctx_list[i]);
       free(plugin_ctx_list[i].bContext);     /* free Bacula private context */
@@ -322,6 +349,7 @@ void free_plugins(JCR *jcr)
  *
  * ==============================================================
  */
+
 static bRC baculaGetValue(bpContext *ctx, bsdrVariable var, void *value)
 {
    JCR *jcr;
@@ -344,6 +372,10 @@ static bRC baculaGetValue(bpContext *ctx, bsdrVariable var, void *value)
       *((char **)value) = jcr->Job;
       Dmsg1(dbglvl, "Bacula: return Job name=%s\n", jcr->Job);
       break;
+#if 0
+   case bsdVarDevTypes:
+      *((s_kw **)value) = dev_types;
+#endif
    default:
       break;
    }
@@ -352,16 +384,16 @@ static bRC baculaGetValue(bpContext *ctx, bsdrVariable var, void *value)
 
 static bRC baculaSetValue(bpContext *ctx, bsdwVariable var, void *value)
 {
-   JCR *jcr;   
+   JCR *jcr;
    if (!value || !ctx) {
       return bRC_Error;
    }
-// Dmsg1(dbglvl, "bacula: baculaGetValue var=%d\n", var);
+// Dmsg1(dbglvl, "bacula: baculaSetValue var=%d\n", var);
    jcr = ((bacula_ctx *)ctx->bContext)->jcr;
    if (!jcr) {
       return bRC_Error;
    }
-// Dmsg1(dbglvl, "Bacula: jcr=%p\n", jcr); 
+// Dmsg1(dbglvl, "Bacula: jcr=%p\n", jcr);
    /* Nothing implemented yet */
    Dmsg1(dbglvl, "sd-plugin: baculaSetValue var=%d\n", var);
    return bRC_OK;
@@ -413,7 +445,7 @@ static bRC baculaDebugMsg(bpContext *ctx, const char *file, int line,
    return bRC_OK;
 }
 
-static char *baculaEditDeviceCodes(DCR *dcr, char *omsg, 
+static char *baculaEditDeviceCodes(DCR *dcr, char *omsg,
   const char *imsg, const char *cmd)
 {
    return edit_device_codes(dcr, omsg, imsg, cmd);
@@ -429,7 +461,7 @@ int main(int argc, char *argv[])
    JCR *jcr2 = &mjcr2;
 
    strcpy(my_name, "test-dir");
-    
+
    getcwd(plugin_dir, sizeof(plugin_dir)-1);
    load_sd_plugins(plugin_dir);
 
@@ -448,7 +480,7 @@ int main(int argc, char *argv[])
 
    unload_plugins();
 
-   Dmsg0(dbglvl, "sd-plugin: OK ...\n");
+   Dmsg0(dbglvl, "sd-plugin: Test OK ...\n");
    close_memory_pool();
    sm_dump(false);
    return 0;
