@@ -1,22 +1,25 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2014 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2017 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from many
-   others, a complete list can be found in the file AUTHORS.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
    You may use this file and others of this release according to the
    license defined in the LICENSE file, which includes the Affero General
    Public License, v3.0 ("AGPLv3") and some additional permissions and
    terms pursuant to its AGPLv3 Section 7.
 
-   Bacula® is a registered trademark of Kern Sibbald.
+   This notice must be preserved when any source code is
+   conveyed and/or propagated.
+
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
- * Second generation Storage daemon.
+ * Third generation Storage daemon.
  *
- *  Kern Sibbald, MM
+ *  Written by Kern Sibbald, MM
  *
  * It accepts a number of simple commands from the File daemon
  * and acts on them. When a request to append data is made,
@@ -34,7 +37,7 @@
  */
 #include "sd_plugins.h"
 
-/* Imported functions */
+/* Imported functions and variables */
 extern bool parse_sd_config(CONFIG *config, const char *configfile, int exit_code);
 
 /* Forward referenced functions */
@@ -49,20 +52,20 @@ extern "C" void *device_initialization(void *arg);
 /* Global variables exported */
 char OK_msg[]   = "3000 OK\n";
 char TERM_msg[] = "3999 Terminate\n";
-STORES *me = NULL;                    /* our Global resource */
-bool forge_on = false;                /* proceed inspite of I/O errors */
-pthread_mutex_t device_release_mutex = PTHREAD_MUTEX_INITIALIZER;
-pthread_cond_t wait_device_release = PTHREAD_COND_INITIALIZER;
 void *start_heap;
+static bool test_config = false;
 
 
 static uint32_t VolSessionId = 0;
 uint32_t VolSessionTime;
 char *configfile = NULL;
 bool init_done = false;
+static pthread_t server_tid;
+static bool server_tid_valid = false;
 
 /* Global static variables */
-static bool foreground = 0;
+static bool foreground = false;
+static bool make_pid_file = true;     /* create pid file */
 static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
 static workq_t dird_workq;            /* queue for processing connections */
 static CONFIG *config;
@@ -71,26 +74,45 @@ static CONFIG *config;
 static void usage()
 {
    fprintf(stderr, _(
-PROG_COPYRIGHT
-"\nVersion: %s (%s)\n\n"
-"Usage: bacula-sd [options] [-c config_file] [config_file]\n"
-"        -c <file>   use <file> as configuration file\n"
-"        -d <nn>     set debug level to <nn>\n"
-"        -dt         print timestamp in debug output\n"
-"        -f          run in foreground (for debugging)\n"
-"        -g <group>  set groupid to group\n"
-"        -m          print kaboom output (for debugging)\n"
-"        -p          proceed despite I/O errors\n"
-"        -s          no signals (for debugging)\n"
-"        -t          test - read config and exit\n"
-"        -u <user>   userid to <user>\n"
-"        -v          verbose user messages\n"
-"        -?          print this message.\n"
-"\n"), 2000, VERSION, BDATE);
-
+      PROG_COPYRIGHT
+      "\n%sVersion: %s (%s)\n\n"
+      "Usage: bacula-sd [options] [-c config_file] [config_file]\n"
+      "     -c <file>         use <file> as configuration file\n"
+      "     -d <nn>[,<tags>]  set debug level to <nn>, debug tags to <tags>\n"
+      "     -dt               print timestamp in debug output\n"
+      "     -T                set trace on\n"
+      "     -f                run in foreground (for debugging)\n"
+      "     -g <group>        set groupid to group\n"
+      "     -m                print kaboom output (for debugging)\n"
+      "     -p                proceed despite I/O errors\n"
+      "     -P                do not create pid file\n"
+      "     -s                no signals (for debugging)\n"
+      "     -t                test - read config and exit\n"
+      "     -u <user>         userid to <user>\n"
+      "     -v                verbose user messages\n"
+      "     -?                print this message.\n"
+      "\n"), 2000, "", VERSION, BDATE);
    exit(1);
 }
 
+/*
+ * !!! WARNING !!! Use this function only when bacula is stopped.
+ * ie, after a fatal signal and before exiting the program
+ * Print information about a JCR
+ */
+static void sd_debug_print(JCR *jcr, FILE *fp)
+{
+   if (jcr->dcr) {
+      DCR *dcr = jcr->dcr;
+      fprintf(fp, "\tdcr=%p volumename=%s dev=%p newvol=%d reserved=%d locked=%d\n",
+              dcr, dcr->VolumeName, dcr->dev, dcr->NewVol,
+              dcr->is_reserved(),
+              dcr->is_dev_locked());
+   } else {
+      fprintf(fp, "dcr=*None*\n");
+   }
+}
+
 /*********************************************************************
  *
  *  Main Bacula Unix Storage Daemon
@@ -104,7 +126,6 @@ int main (int argc, char *argv[])
 {
    int ch;
    bool no_signals = false;
-   bool test_config = false;
    pthread_t thid;
    char *uid = NULL;
    char *gid = NULL;
@@ -118,17 +139,18 @@ int main (int argc, char *argv[])
    my_name_is(argc, argv, "bacula-sd");
    init_msg(NULL, NULL);
    daemon_start_time = time(NULL);
+   setup_daemon_message_queue();
 
    /* Sanity checks */
    if (TAPE_BSIZE % B_DEV_BSIZE != 0 || TAPE_BSIZE / B_DEV_BSIZE == 0) {
-      Emsg2(M_ABORT, 0, _("Tape block size (%d) not multiple of system size (%d)\n"),
+      Jmsg2(NULL, M_ABORT, 0, _("Tape block size (%d) not multiple of system size (%d)\n"),
          TAPE_BSIZE, B_DEV_BSIZE);
    }
    if (TAPE_BSIZE != (1 << (ffs(TAPE_BSIZE)-1))) {
-      Emsg1(M_ABORT, 0, _("Tape block size (%d) is not a power of 2\n"), TAPE_BSIZE);
+      Jmsg1(NULL, M_ABORT, 0, _("Tape block size (%d) is not a power of 2\n"), TAPE_BSIZE);
    }
 
-   while ((ch = getopt(argc, argv, "c:d:fg:mpstu:v?")) != -1) {
+   while ((ch = getopt(argc, argv, "c:d:fg:mpPstu:v?Ti")) != -1) {
       switch (ch) {
       case 'c':                    /* configuration file */
          if (configfile != NULL) {
@@ -141,13 +163,25 @@ int main (int argc, char *argv[])
          if (*optarg == 't') {
             dbg_timestamp = true;
          } else {
+            char *p;
+            /* We probably find a tag list -d 10,sql,bvfs */
+            if ((p = strchr(optarg, ',')) != NULL) {
+               *p = 0;
+            }
             debug_level = atoi(optarg);
             if (debug_level <= 0) {
                debug_level = 1;
             }
+            if (p) {
+               debug_parse_tags(p+1, &debug_level_tags);
+            }
          }
          break;
 
+      case 'T':
+         set_trace(true);
+         break;
+
       case 'f':                    /* run in foreground */
          foreground = true;
          break;
@@ -156,6 +190,11 @@ int main (int argc, char *argv[])
          gid = optarg;
          break;
 
+      /* Temp code to enable new match_bsr() code, not documented */
+      case 'i':
+         use_new_match_all = 1;
+
+         break;
       case 'm':                    /* print kaboom output */
          prt_kaboom = true;
          break;
@@ -164,6 +203,10 @@ int main (int argc, char *argv[])
          forge_on = true;
          break;
 
+      case 'P':                    /* no pid file */
+         make_pid_file = false;
+         break;
+
       case 's':                    /* no signals */
          no_signals = true;
          break;
@@ -197,8 +240,14 @@ int main (int argc, char *argv[])
       argc--;
       argv++;
    }
-   if (argc)
+   if (argc) {
       usage();
+   }
+
+   if (!foreground && !test_config) {
+      daemon_start();                 /* become daemon */
+      init_stack_dump();              /* pick up new pid */
+   }
 
    if (!no_signals) {
       init_signals(terminate_stored);
@@ -208,7 +257,7 @@ int main (int argc, char *argv[])
       configfile = bstrdup(CONFIG_FILE);
    }
 
-   config = new_config_parser();
+   config = New(CONFIG());
    parse_sd_config(config, configfile, M_ERROR_TERM);
 
    if (init_crypto() != 0) {
@@ -227,13 +276,10 @@ int main (int argc, char *argv[])
 
    my_name_is(0, (char **)NULL, me->hdr.name);     /* Set our real name */
 
-   if (!foreground) {
-      daemon_start();                 /* become daemon */
-      init_stack_dump();              /* pick up new pid */
+   if (make_pid_file) {
+      create_pid_file(me->pid_directory, "bacula-sd",
+                      get_first_port_host_order(me->sdaddrs));
    }
-
-   create_pid_file(me->pid_directory, "bacula-sd",
-                   get_first_port_host_order(me->sdaddrs));
    read_state_file(me->working_directory, "bacula-sd",
                    get_first_port_host_order(me->sdaddrs));
 
@@ -262,13 +308,16 @@ int main (int argc, char *argv[])
    create_volume_lists();             /* do before device_init */
    if (pthread_create(&thid, NULL, device_initialization, NULL) != 0) {
       berrno be;
-      Emsg1(M_ABORT, 0, _("Unable to create thread. ERR=%s\n"), be.bstrerror());
+      Jmsg1(NULL, M_ABORT, 0, _("Unable to create thread. ERR=%s\n"), be.bstrerror());
    }
 
    start_watchdog();                  /* start watchdog thread */
    init_jcr_subsystem();              /* start JCR watchdogs etc. */
+   dbg_jcr_add_hook(sd_debug_print); /* used to director variables */
 
    /* Single server used for Director and File daemon */
+   server_tid = pthread_self();
+   server_tid_valid = true;
    bnet_thread_server(me->sdaddrs, me->max_concurrent_jobs * 2 + 1,
                       &dird_workq, handle_connection_request);
    exit(1);                           /* to keep compiler quiet */
@@ -291,7 +340,8 @@ static int check_resources()
 {
    bool OK = true;
    bool tls_needed;
-
+   AUTOCHANGER *changer;
+   DEVRES *device;
 
    me = (STORES *)GetNextRes(R_STORAGE, NULL);
    if (!me) {
@@ -433,8 +483,15 @@ static int check_resources()
       }
    }
 
-   OK = init_autochangers();
+   foreach_res(changer, R_AUTOCHANGER) {
+      foreach_alist(device, changer->device) {
+         device->cap_bits |= CAP_AUTOCHANGER;
+      }
+   }
 
+   if (OK) {
+      OK = init_autochangers();
+   }
 
    if (OK) {
       close_msg(NULL);                   /* close temp message handler */
@@ -523,7 +580,6 @@ get_out2:
    free_pool_memory(basename);
 }
 
-
 /*
  * Here we attempt to init and open each device. This is done
  *  once at startup in a separate thread.
@@ -535,11 +591,13 @@ void *device_initialization(void *arg)
    DCR *dcr;
    JCR *jcr;
    DEVICE *dev;
+   struct stat statp;
 
    LockRes();
 
    pthread_detach(pthread_self());
    jcr = new_jcr(sizeof(JCR), stored_free_jcr);
+   new_plugins(jcr);  /* instantiate plugins */
    jcr->setJobType(JT_SYSTEM);
    /* Initialize FD start condition variable */
    int errstat = pthread_cond_init(&jcr->job_start_wait, NULL);
@@ -549,33 +607,59 @@ void *device_initialization(void *arg)
    }
 
    foreach_res(device, R_DEVICE) {
-      Dmsg1(90, "calling init_dev %s\n", device->device_name);
+      Dmsg1(90, "calling init_dev %s\n", device->hdr.name);
       dev = init_dev(NULL, device);
-      Dmsg1(10, "SD init done %s\n", device->device_name);
+      Dmsg1(10, "SD init done %s\n", device->hdr.name);
       if (!dev) {
-         Jmsg1(NULL, M_ERROR, 0, _("Could not initialize %s\n"), device->device_name);
+         Jmsg1(NULL, M_ERROR, 0, _("Could not initialize SD device \"%s\"\n"), device->hdr.name);
          continue;
       }
 
       jcr->dcr = dcr = new_dcr(jcr, NULL, dev);
       generate_plugin_event(jcr, bsdEventDeviceInit, dcr);
-      if (dev->is_autochanger()) {
-         /* If autochanger set slot in dev sturcture */
-         get_autochanger_loaded_slot(dcr);
+
+      if (device->control_name && stat(device->control_name, &statp) < 0) {
+         berrno be;
+         Jmsg2(jcr, M_ERROR_TERM, 0, _("Unable to stat ControlDevice %s: ERR=%s\n"),
+            device->control_name, be.bstrerror());
       }
 
+      if ((device->lock_command && device->control_name) &&
+          !me->plugin_directory) {
+         Jmsg0(jcr, M_ERROR_TERM, 0, _("No plugin directory configured for SAN shared storage\n"));
+      }
+
+
+      /*
+       * Note: be careful setting the slot here. If the drive
+       *  is shared storage, the contents can change before
+       *  the drive is used.
+       */
       if (device->cap_bits & CAP_ALWAYSOPEN) {
+         if (dev->is_autochanger()) {
+            /* If autochanger set slot in dev sturcture */
+            get_autochanger_loaded_slot(dcr);
+         }
          Dmsg1(20, "calling first_open_device %s\n", dev->print_name());
+         if (generate_plugin_event(jcr, bsdEventDeviceOpen, dcr) != bRC_OK) {
+            Jmsg(jcr, M_FATAL, 0, _("generate_plugin_event(bsdEventDeviceOpen) Failed\n"));
+            continue;
+         }
+
          if (!first_open_device(dcr)) {
             Jmsg1(NULL, M_ERROR, 0, _("Could not open device %s\n"), dev->print_name());
             Dmsg1(20, "Could not open device %s\n", dev->print_name());
+            generate_plugin_event(jcr, bsdEventDeviceClose, dcr);
             free_dcr(dcr);
             jcr->dcr = NULL;
             continue;
          }
+      } else {
+         /* If not always open, we don't know what is in the drive */
+         dev->clear_slot();
       }
       if (device->cap_bits & CAP_AUTOMOUNT && dev->is_open()) {
-         switch (read_dev_volume_label(dcr)) {
+         switch (dev->read_dev_volume_label(dcr)) {
          case VOL_OK:
             memcpy(&dev->VolCatInfo, &dcr->VolCatInfo, sizeof(dev->VolCatInfo));
             volume_unused(dcr);             /* mark volume "released" */
@@ -585,12 +669,15 @@ void *device_initialization(void *arg)
             break;
          }
       }
+
       free_dcr(dcr);
       jcr->dcr = NULL;
    }
+
+
 #ifdef xxx
    if (jcr->dcr) {
-      Dmsg1(000, "free_dcr=%p\n", jcr->dcr);
+      Pmsg1(000, "free_dcr=%p\n", jcr->dcr);
       free_dcr(jcr->dcr);
       jcr->dcr = NULL;
    }
@@ -618,7 +705,7 @@ void terminate_stored(int sig)
    debug_level = 0;                   /* turn off any debug */
    stop_watchdog();
 
-   if (sig == SIGTERM) {              /* normal shutdown request? */
+   if (sig == SIGTERM || sig == SIGINT) { /* normal shutdown request? or ^C */
       /*
        * This is a normal shutdown request. We wiffle through
        *   all open jobs canceling them and trying to wake
@@ -631,6 +718,10 @@ void terminate_stored(int sig)
             free_jcr(jcr);
             continue;                 /* ignore console */
          }
+         if (jcr->dcr) {
+            /* Make sure no device remains locked */
+            generate_plugin_event(jcr, bsdEventDeviceClose, jcr->dcr);
+         }
          jcr->setJobStatus(JS_Canceled);
          fd = jcr->file_bsock;
          if (fd) {
@@ -654,32 +745,42 @@ void terminate_stored(int sig)
       bmicrosleep(0, 500000);         /* give them 1/2 sec to clean up */
    }
 
-   write_state_file(me->working_directory, "bacula-sd", get_first_port_host_order(me->sdaddrs));
-   delete_pid_file(me->pid_directory, "bacula-sd", get_first_port_host_order(me->sdaddrs));
+   if (!test_config) {
+      write_state_file(me->working_directory,
+                       "bacula-sd", get_first_port_host_order(me->sdaddrs));
+      if (make_pid_file) {
+         delete_pid_file(me->pid_directory,
+                         "bacula-sd", get_first_port_host_order(me->sdaddrs));
+      }
+   }
 
    Dmsg1(200, "In terminate_stored() sig=%d\n", sig);
 
    unload_plugins();
    free_volume_lists();
 
+   free_daemon_message_queue();
+
    foreach_res(device, R_DEVICE) {
-      Dmsg1(10, "Term device %s\n", device->device_name);
+      Dmsg2(10, "Term device %s %s\n", device->hdr.name, device->device_name);
       if (device->dev) {
          device->dev->clear_volhdr();
-         device->dev->term();
+         device->dev->term(NULL);
          device->dev = NULL;
       } else {
-         Dmsg1(10, "No dev structure %s\n", device->device_name);
+         Dmsg2(10, "No dev structure %s %s\n", device->hdr.name, device->device_name);
       }
    }
+   if (server_tid_valid) {
+      bnet_stop_thread_server(server_tid);
+   }
 
    if (configfile) {
       free(configfile);
       configfile = NULL;
    }
    if (config) {
-      config->free_resources();
-      free(config);
+      delete config;
       config = NULL;
    }
 
@@ -689,6 +790,8 @@ void terminate_stored(int sig)
    term_msg();
    cleanup_crypto();
    term_reservations_lock();
+   free(res_head);
+   res_head = NULL;
    close_memory_pool();
    lmgr_cleanup_main();
 
