@@ -1,29 +1,20 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2000-2011 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2017 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation and included
-   in the file LICENSE.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   This notice must be preserved when any source code is
+   conveyed and/or propagated.
 
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  *  Subroutines to handle waiting for operator intervention
@@ -32,7 +23,6 @@
  *  Code for wait_for_sysop() pulled from askdir.c
  *
  *   Kern Sibbald, March 2005
- *
  */
 
 
@@ -44,7 +34,7 @@ const int dbglvl = 400;
 /*
  * Wait for SysOp to mount a tape on a specific device
  *
- *   Returns: W_ERROR, W_TIMEOUT, W_POLL, W_MOUNT, or W_WAKE 
+ *   Returns: W_ERROR, W_TIMEOUT, W_POLL, W_MOUNT, or W_WAKE
  */
 int wait_for_sysop(DCR *dcr)
 {
@@ -59,7 +49,7 @@ int wait_for_sysop(DCR *dcr)
    DEVICE *dev = dcr->dev;
    JCR *jcr = dcr->jcr;
 
-   dev->dlock();  
+   dev->Lock();
    Dmsg1(dbglvl, "Enter blocked=%s\n", dev->print_blocked());
 
    /*
@@ -100,12 +90,12 @@ int wait_for_sysop(DCR *dcr)
       timeout.tv_nsec = tv.tv_usec * 1000;
       timeout.tv_sec = tv.tv_sec + add_wait;
 
-      Dmsg4(dbglvl, "I'm going to sleep on device %s. HB=%d rem_wait=%d add_wait=%d\n", 
+      Dmsg4(dbglvl, "I'm going to sleep on device %s. HB=%d rem_wait=%d add_wait=%d\n",
          dev->print_name(), (int)me->heartbeat_interval, dev->rem_wait_sec, add_wait);
       start = time(NULL);
 
       /* Wait required time */
-      stat = pthread_cond_timedwait(&dev->wait_next_vol, &dev->m_mutex, &timeout);
+      stat = dev->next_vol_timedwait(&timeout);
 
       Dmsg2(dbglvl, "Wokeup from sleep on device stat=%d blocked=%s\n", stat,
          dev->print_blocked());
@@ -136,7 +126,7 @@ int wait_for_sysop(DCR *dcr)
       }
 
       /*
-       * Continue waiting if operator is labeling volumes 
+       * Continue waiting if operator is labeling volumes
        */
       if (dev->blocked() == BST_WRITING_LABEL) {
          continue;
@@ -155,7 +145,7 @@ int wait_for_sysop(DCR *dcr)
 
       if (!unmounted && dev->vol_poll_interval &&
           (total_waited >= dev->vol_poll_interval)) {
-         Dmsg1(dbglvl, "poll return in wait blocked=%s\n", dev->print_blocked());
+         Dmsg1(dbglvl, "Set poll=true return in wait blocked=%s\n", dev->print_blocked());
          dev->poll = true;            /* returning a poll event */
          stat = W_POLL;
          break;
@@ -171,7 +161,7 @@ int wait_for_sysop(DCR *dcr)
 
       /*
        * If we did not timeout, then some event happened, so
-       *   return to check if state changed.   
+       *   return to check if state changed.
        */
       if (stat != ETIMEDOUT) {
          berrno be;
@@ -180,7 +170,7 @@ int wait_for_sysop(DCR *dcr)
          break;
       }
 
-      /* 
+      /*
        * At this point, we know we woke up because of a timeout,
        *   that was due to a heartbeat, because any other reason would
        *   have caused us to return, so update the wait counters and continue.
@@ -192,7 +182,7 @@ int wait_for_sysop(DCR *dcr)
       /* If the user did not unmount the tape and we are polling, ensure
        *  that we poll at the correct interval.
        */
-      if (!unmounted && dev->vol_poll_interval && 
+      if (!unmounted && dev->vol_poll_interval &&
            add_wait > dev->vol_poll_interval - total_waited) {
          add_wait = dev->vol_poll_interval - total_waited;
       }
@@ -205,23 +195,23 @@ int wait_for_sysop(DCR *dcr)
       dev->set_blocked(dev->dev_prev_blocked);    /* restore entry state */
       Dmsg1(dbglvl, "set %s\n", dev->print_blocked());
    }
-   Dmsg1(dbglvl, "Exit blocked=%s\n", dev->print_blocked());
-   dev->dunlock();
+   Dmsg2(dbglvl, "Exit blocked=%s poll=%d\n", dev->print_blocked(), dev->poll);
+   dev->Unlock();
    return stat;
 }
 
 
 /*
- * Wait for any device to be released, then we return, so 
+ * Wait for any device to be released, then we return, so
  *  higher level code can rescan possible devices.  Since there
  *  could be a job waiting for a drive to free up, we wait a maximum
- *  of 1 minute then retry just in case a broadcast was lost, and 
+ *  of 1 minute then retry just in case a broadcast was lost, and
  *  we return to rescan the devices.
- * 
+ *
  * Returns: true  if a device has changed state
  *          false if the total wait time has expired.
  */
-bool wait_for_device(JCR *jcr, int &retries)
+bool wait_for_any_device(JCR *jcr, int &retries)
 {
    struct timeval tv;
    struct timezone tz;
@@ -231,12 +221,12 @@ bool wait_for_device(JCR *jcr, int &retries)
    const int max_wait_time = 1 * 60;       /* wait 1 minute */
    char ed1[50];
 
-   Dmsg0(dbglvl, "Enter wait_for_device\n");
+   Dmsg0(dbglvl, "Enter wait_for_any_device\n");
    P(device_release_mutex);
 
    if (++retries % 5 == 0) {
       /* Print message every 5 minutes */
-      Jmsg(jcr, M_MOUNT, 0, _("JobId=%s, Job %s waiting to reserve a device.\n"), 
+      Jmsg(jcr, M_MOUNT, 0, _("JobId=%s, Job %s waiting to reserve a device.\n"),
          edit_uint64(jcr->JobId, ed1), jcr->Job);
    }
 
@@ -255,23 +245,107 @@ bool wait_for_device(JCR *jcr, int &retries)
    return ok;
 }
 
-#ifdef xxx
 /*
- * The jcr timers are used for waiting on any device *
+ * Wait for a specific device to be released
+ *  We wait a maximum of 1 minute then
+ *  retry just in case a broadcast was lost.
+ *
+ * Returns: true  if the device has changed state
+ *          false if the total wait time has expired.
+ */
+bool wait_for_device(DCR *dcr, int &retries)
+{
+   struct timeval tv;
+   struct timezone tz;
+   struct timespec timeout;
+   JCR *jcr = dcr->jcr;
+   DEVICE *dev = dcr->dev;
+   int stat = 0;
+   bool ok = true;
+   const int max_wait_time = 1 * 60;       /* wait 1 minute */
+   char ed1[50];
+
+   Dmsg3(40, "Enter wait_for_device. busy=%d dcrvol=%s devvol=%s\n",
+      dev->is_busy(), dcr->VolumeName, dev->getVolCatName());
+
+   P(device_release_mutex);
+
+   if (++retries % 5 == 0) {
+      /* Print message every 5 minutes */
+      Jmsg(jcr, M_MOUNT, 0, _("JobId=%s, Job %s waiting device %s.\n"),
+         edit_uint64(jcr->JobId, ed1), jcr->Job, dcr->dev->print_name());
+   }
+
+   gettimeofday(&tv, &tz);
+   timeout.tv_nsec = tv.tv_usec * 1000;
+   timeout.tv_sec = tv.tv_sec + max_wait_time;
+
+   Dmsg0(dbglvl, "Going to wait for a device.\n");
+
+   /* Wait required time */
+   stat = pthread_cond_timedwait(&wait_device_release, &device_release_mutex, &timeout);
+   Dmsg1(dbglvl, "Wokeup from sleep on device stat=%d\n", stat);
+
+   V(device_release_mutex);
+   Dmsg1(dbglvl, "Return from wait_device ok=%d\n", ok);
+   return ok;
+}
+
+
+/*
+ * This routine initializes the device wait timers
+ */
+void init_device_wait_timers(DCR *dcr)
+{
+   DEVICE *dev = dcr->dev;
+   JCR *jcr = dcr->jcr;
+
+   /* ******FIXME******* put these on config variables */
+   dev->min_wait = 60 * 60;
+   dev->max_wait = 24 * 60 * 60;
+   dev->max_num_wait = 9;              /* 5 waits =~ 1 day, then 1 day at a time */
+   dev->wait_sec = dev->min_wait;
+   dev->rem_wait_sec = dev->wait_sec;
+   dev->num_wait = 0;
+   dev->poll = false;
+
+   jcr->min_wait = 60 * 60;
+   jcr->max_wait = 24 * 60 * 60;
+   jcr->max_num_wait = 9;              /* 5 waits =~ 1 day, then 1 day at a time */
+   jcr->wait_sec = jcr->min_wait;
+   jcr->rem_wait_sec = jcr->wait_sec;
+   jcr->num_wait = 0;
+
+}
+
+void init_jcr_device_wait_timers(JCR *jcr)
+{
+   /* ******FIXME******* put these on config variables */
+   jcr->min_wait = 60 * 60;
+   jcr->max_wait = 24 * 60 * 60;
+   jcr->max_num_wait = 9;              /* 5 waits =~ 1 day, then 1 day at a time */
+   jcr->wait_sec = jcr->min_wait;
+   jcr->rem_wait_sec = jcr->wait_sec;
+   jcr->num_wait = 0;
+}
+
+
+/*
+ * The dev timers are used for waiting on a particular device
+ *
  * Returns: true if time doubled
  *          false if max time expired
  */
-static bool double_jcr_wait_time(JCR *jcr)
+bool double_dev_wait_time(DEVICE *dev)
 {
-   jcr->wait_sec *= 2;               /* double wait time */
-   if (jcr->wait_sec > jcr->max_wait) {   /* but not longer than maxtime */
-      jcr->wait_sec = jcr->max_wait;
+   dev->wait_sec *= 2;               /* double wait time */
+   if (dev->wait_sec > dev->max_wait) {   /* but not longer than maxtime */
+      dev->wait_sec = dev->max_wait;
    }
-   jcr->num_wait++;
-   jcr->rem_wait_sec = jcr->wait_sec;
-   if (jcr->num_wait >= jcr->max_num_wait) {
+   dev->num_wait++;
+   dev->rem_wait_sec = dev->wait_sec;
+   if (dev->num_wait >= dev->max_num_wait) {
       return false;
    }
    return true;
 }
-#endif
