@@ -1,29 +1,20 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2002-2011 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2016 Kern Sibbald
 
-   The main author of Bacula is Kern Sibbald, with contributions from
-   many others, a complete list can be found in the file AUTHORS.
-   This program is Free Software; you can redistribute it and/or
-   modify it under the terms of version three of the GNU Affero General Public
-   License as published by the Free Software Foundation and included
-   in the file LICENSE.
+   The original author of Bacula is Kern Sibbald, with contributions
+   from many others, a complete list can be found in the file AUTHORS.
 
-   This program is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   General Public License for more details.
+   You may use this file and others of this release according to the
+   license defined in the LICENSE file, which includes the Affero General
+   Public License, v3.0 ("AGPLv3") and some additional permissions and
+   terms pursuant to its AGPLv3 Section 7.
 
-   You should have received a copy of the GNU Affero General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301, USA.
+   This notice must be preserved when any source code is 
+   conveyed and/or propagated.
 
-   Bacula® is a registered trademark of Kern Sibbald.
-   The licensor of Bacula is the Free Software Foundation Europe
-   (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
-   Switzerland, email:ftf@fsfeurope.org.
+   Bacula(R) is a registered trademark of Kern Sibbald.
 */
 /*
  *
@@ -36,18 +27,11 @@
 
 #include "bacula.h"
 #include "cats/cats.h"
-#include "cats/sql_glue.h"
 #include "lib/runscript.h"
 #include "dird/dird_conf.h"
 
 extern bool parse_dir_config(CONFIG *config, const char *configfile, int exit_code);
 
-/*
- * Dummy functions
- */
-int generate_daemon_event(JCR *jcr, const char *event)
-   { return 1; }
-
 typedef struct s_id_ctx {
    int64_t *Id;                       /* ids to be modified */
    int num_ids;                       /* ids stored */
@@ -64,25 +48,21 @@ typedef struct s_name_ctx {
    int tot_ids;                       /* total to process */
 } NAME_LIST;
 
-/*
- * Global variables
- */
+/* Global variables */ 
+
 static bool fix = false;
 static bool batch = false;
-static B_DB *db;
+static BDB *db;
 static ID_LIST id_list;
 static NAME_LIST name_list;
 static char buf[20000];
 static bool quit = false;
 static CONFIG *config;
-static const char *idx_tmp_name;
+static const char *idx_tmp_name; 
 
 #define MAX_ID_LIST_LEN 10000000
 
-/*
- * Forward referenced functions
- */
-static void print_catalog_details(CAT *catalog, const char *working_dir);
+/* Forward referenced functions */ 
 static int make_id_list(const char *query, ID_LIST *id_list);
 static int delete_id_list(const char *query, ID_LIST *id_list);
 static int make_name_list(const char *query, NAME_LIST *name_list);
@@ -105,15 +85,17 @@ static void repair_bad_filenames();
 static void do_interactive_mode();
 static bool yes_no(const char *prompt);
 static bool check_idx(const char *col_name);
-static bool create_tmp_idx(const char *idx_name, const char *table_name,
-              const char *col_name);
+static bool create_tmp_idx(const char *idx_name, const char *table_name, 
+               const char *col_name);
 static bool drop_tmp_idx(const char *idx_name, const char *table_name);
 static int check_idx_handler(void *ctx, int num_fields, char **row);
 
 static void usage()
 {
    fprintf(stderr,
-"Usage: dbcheck [-c config ] [-B] [-C catalog name] [-d debug_level] <working-directory> <bacula-database> <user> <password> [<dbhost>] [<dbport>]\n"
+PROG_COPYRIGHT
+"\n%sVersion: %s (%s)\n\n"
+"Usage: dbcheck [-c config ] [-B] [-C catalog name] [-d debug_level] <working-directory> <bacula-database> <user> <password> [<dbhost>] [<dbport>] [<dbport>] [<dbsslmode>] [<dbsslkey>] [<dbsslcert>] [<dbsslca>]\n"
 "       -b              batch mode\n"
 "       -C              catalog name in the director conf file\n"
 "       -c              Director conf filename\n"
@@ -123,7 +105,9 @@ static void usage()
 "       -f              fix inconsistencies\n"
 "       -t              test if client library is thread-safe\n"
 "       -v              verbose\n"
-"       -?              print this message\n\n");
+"       -?              print this message\n"
+"\n", 2002, "", VERSION, BDATE);
+
    exit(1);
 }
 
@@ -131,6 +115,8 @@ int main (int argc, char *argv[])
 {
    int ch;
    const char *user, *password, *db_name, *dbhost;
+   const char *dbsslmode = NULL, *dbsslkey = NULL, *dbsslcert = NULL, *dbsslca = NULL;
+   const char *dbsslcapath = NULL, *dbsslcipher = NULL;
    int dbport = 0;
    bool print_catalog=false;
    char *configfile = NULL;
@@ -143,12 +129,12 @@ int main (int argc, char *argv[])
    lmgr_init_thread();
 
    my_name_is(argc, argv, "dbcheck");
-   init_msg(NULL, NULL);           /* setup message handler */
+   init_msg(NULL, NULL);             /* setup message handler */
 
    memset(&id_list, 0, sizeof(id_list));
    memset(&name_list, 0, sizeof(name_list));
 
-   while ((ch = getopt(argc, argv, "bc:C:d:fvB?")) != -1) {
+   while ((ch = getopt(argc, argv, "bc:C:d:fvB?")) != -1) { 
       switch (ch) {
       case 'B':
          print_catalog = true;     /* get catalog information from config */
@@ -194,7 +180,7 @@ int main (int argc, char *argv[])
       if (argc > 0) {
          Pmsg0(0, _("Warning skipping the additional parameters for working directory/dbname/user/password/host.\n"));
       }
-      config = new_config_parser();
+      config = New(CONFIG());
       parse_dir_config(config, configfile, M_ERROR_TERM);
       LockRes();
       foreach_res(catalog, R_CATALOG) {
@@ -225,11 +211,25 @@ int main (int argc, char *argv[])
          }
          set_working_directory(director->working_directory);
 
-         /*
-          * Print catalog information and exit (-B)
-          */
+         /* Print catalog information and exit (-B) */
          if (print_catalog) {
-            print_catalog_details(catalog, director->working_directory);
+
+            POOLMEM *catalog_details = get_pool_memory(PM_MESSAGE);
+            db = db_init_database(NULL, catalog->db_driver, catalog->db_name, catalog->db_user,
+                    catalog->db_password, catalog->db_address,
+                    catalog->db_port, catalog->db_socket,
+                    catalog->db_ssl_mode,
+                    catalog->db_ssl_key, catalog->db_ssl_cert, 
+                    catalog->db_ssl_ca,
+                    catalog->db_ssl_capath, catalog->db_ssl_cipher,
+                    catalog->mult_db_connections,
+                    catalog->disable_batch_insert);
+            if (db) {
+               printf("%sdb_type=%s\nworking_dir=%s\n", catalog->display(catalog_details),
+                  db_get_engine_name(db), working_directory);
+               db_close_database(NULL, db);
+            }
+            free_pool_memory(catalog_details);
             exit(0);
          }
 
@@ -241,9 +241,15 @@ int main (int argc, char *argv[])
             dbhost = NULL;
          }
          dbport = catalog->db_port;
+         dbsslmode = catalog->db_ssl_mode;
+         dbsslkey = catalog->db_ssl_key;
+         dbsslcert = catalog->db_ssl_cert;
+         dbsslca = catalog->db_ssl_ca;
+         dbsslcapath = catalog->db_ssl_capath;
+         dbsslcipher = catalog->db_ssl_cipher;
       }
    } else {
-      if (argc > 6) {
+      if (argc > 10) {
          Pmsg0(0, _("Wrong number of arguments.\n"));
          usage();
       }
@@ -253,59 +259,60 @@ int main (int argc, char *argv[])
          usage();
       }
 
-      /*
-       * This is needed by SQLite to find the db
-       */
+      /* This is needed by SQLite to find the db */
       working_directory = argv[0];
       db_name = "bacula";
       user = db_name;
       password = "";
       dbhost = NULL;
 
-      if (argc == 2) {
+      if (argc >= 2) {
          db_name = argv[1];
          user = db_name;
-      } else if (argc == 3) {
-         db_name = argv[1];
-         user = argv[2];
-      } else if (argc == 4) {
-         db_name = argv[1];
-         user = argv[2];
-         password = argv[3];
-      } else if (argc == 5) {
-         db_name = argv[1];
-         user = argv[2];
-         password = argv[3];
-         dbhost = argv[4];
-      } else if (argc == 6) {
-         db_name = argv[1];
-         user = argv[2];
-         password = argv[3];
-         dbhost = argv[4];
-         errno = 0;
-         dbport = strtol(argv[5], &endptr, 10);
-         if (*endptr != '\0') {
-            Pmsg0(0, _("Database port must be a numeric value.\n"));
-            exit(1);
-         } else if (errno == ERANGE) {
-            Pmsg0(0, _("Database port must be a int value.\n"));
-            exit(1);
-         }
-      }
+         if (argc >= 3) {
+            user = argv[2];
+            if (argc >= 4) {
+               password = argv[3];
+               if (argc >= 5) {
+                  dbhost = argv[4];
+                  if (argc >= 6) {
+                     errno = 0;
+                     dbport = strtol(argv[5], &endptr, 10);
+                     if (*endptr != '\0') {
+                        Pmsg0(0, _("Database port must be a numeric value.\n"));
+                        exit(1);
+                     } else if (errno == ERANGE) {
+                        Pmsg0(0, _("Database port must be a int value.\n"));
+                        exit(1);
+                     }
+                     if (argc >= 7) {
+                        dbsslmode = argv[6];
+                        if (argc >= 8) {
+                           dbsslkey = argv[7];
+                           dbsslcert = argv[8];
+                           if (argc == 10) {
+                              dbsslca = argv[9];
+                           } /* if (argc == 10) */
+                        } /* if (argc >= 8) */
+                     } /* if (argc >= 7) */
+                  } /* if (argc >= 6) */
+               } /* if (argc >= 5) */
+            } /* if (argc >= 4) */
+         } /* if (argc >= 3) */
+      } /* if (argc >= 2) */
    }
 
-   /*
-    * Open database
-    */
-   db = db_init_database(NULL, NULL, db_name, user, password, dbhost, dbport, NULL, false, false);
-   if (!db_open_database(NULL, db)) {
+   /* Open database */
+   db = db_init_database(NULL, NULL, db_name, user, password, dbhost,
+          dbport, NULL, dbsslmode, dbsslkey, dbsslcert, dbsslca,
+           dbsslcapath, dbsslcipher, false, false);
+
+   if (!db || !db_open_database(NULL, db)) {
       Emsg1(M_FATAL, 0, "%s", db_strerror(db));
           return 1;
    }
 
-   /*
-    * Drop temporary index idx_tmp_name if it already exists
-    */
+   /* Drop temporary index idx_tmp_name if it already exists */
    drop_tmp_idx("idxPIchk", "File");
 
    if (batch) {
@@ -326,33 +333,34 @@ int main (int argc, char *argv[])
       do_interactive_mode();
    }
 
-   /*
-    * Drop temporary index idx_tmp_name
-    */
+   /* Drop temporary index idx_tmp_name */
    drop_tmp_idx("idxPIchk", "File");
 
-   db_close_database(NULL, db);
+   if (db) db_close_database(NULL, db);
    close_msg(NULL);
    term_msg();
    lmgr_cleanup_main();
    return 0;
 }
 
-static void print_catalog_details(CAT *catalog, const char *working_dir)
+void print_catalog_details(CAT *catalog, const char *working_dir)
 {
    POOLMEM *catalog_details = get_pool_memory(PM_MESSAGE);
 
    /*
-    * Instantiate a B_DB class and see what db_type gets assigned to it.
+    * Instantiate a BDB class and see what db_type gets assigned to it.
     */
    db = db_init_database(NULL, catalog->db_driver, catalog->db_name, catalog->db_user,
                          catalog->db_password, catalog->db_address,
                          catalog->db_port, catalog->db_socket,
+                         catalog->db_ssl_mode, catalog->db_ssl_key, 
+                         catalog->db_ssl_cert, catalog->db_ssl_ca,
+                         catalog->db_ssl_capath, catalog->db_ssl_cipher,
                          catalog->mult_db_connections,
                          catalog->disable_batch_insert);
    if (db) {
       printf("%sdb_type=%s\nworking_dir=%s\n", catalog->display(catalog_details),
-             db->db_get_type(), working_directory);
+             db_get_engine_name(db), working_directory);
       db_close_database(NULL, db);
    }
    free_pool_memory(catalog_details);
@@ -505,10 +513,9 @@ static int print_name_handler(void *ctx, int num_fields, char **row)
 
 static int get_name_handler(void *ctx, int num_fields, char **row)
 {
-   POOLMEM *name = (POOLMEM *)ctx;
-
+   POOLMEM *buf = (POOLMEM *)ctx;
    if (row[0]) {
-      pm_strcpy(&name, row[0]);
+      pm_strcpy(&buf, row[0]);
    }
    return 0;
 }
@@ -670,9 +677,7 @@ static void eliminate_duplicate_filenames()
 
    printf(_("Checking for duplicate Filename entries.\n"));
 
-   /*
-    * Make list of duplicated names
-    */
+   /* Make list of duplicated names */
    query = "SELECT Name, count(Name) as Count FROM Filename GROUP BY  Name "
            "HAVING count(Name) > 1";
 
@@ -687,13 +692,9 @@ static void eliminate_duplicate_filenames()
       return;
    }
    if (fix) {
-      /*
-       * Loop through list of duplicate names
-       */
+      /* Loop through list of duplicate names */
       for (int i=0; i<name_list.num_ids; i++) {
-         /*
-          * Get all the Ids of each name
-          */
+         /* Get all the Ids of each name */
          db_escape_string(NULL, db, esc_name, name_list.name[i], strlen(name_list.name[i]));
          bsnprintf(buf, sizeof(buf), "SELECT FilenameId FROM Filename WHERE Name='%s'", esc_name);
          if (verbose > 1) {
@@ -705,9 +706,7 @@ static void eliminate_duplicate_filenames()
          if (verbose) {
             printf(_("Found %d for: %s\n"), id_list.num_ids, name_list.name[i]);
          }
-         /*
-          * Force all records to use the first id then delete the other ids
-          */
+         /* Force all records to use the first id then delete the other ids */
          for (int j=1; j<id_list.num_ids; j++) {
             char ed1[50], ed2[50];
             bsnprintf(buf, sizeof(buf), "UPDATE File SET FilenameId=%s WHERE FilenameId=%s",
@@ -735,9 +734,7 @@ static void eliminate_duplicate_paths()
 
    printf(_("Checking for duplicate Path entries.\n"));
 
-   /*
-    * Make list of duplicated names
-    */
+   /* Make list of duplicated names */
    query = "SELECT Path, count(Path) as Count FROM Path "
            "GROUP BY Path HAVING count(Path) > 1";
 
@@ -752,14 +749,10 @@ static void eliminate_duplicate_paths()
       return;
    }
    if (fix) {
-      /*
-       * Loop through list of duplicate names
-       */
+      /* Loop through list of duplicate names */
       for (int i=0; i<name_list.num_ids; i++) {
-         /*
-          * Get all the Ids of each name
-          */
-         db_escape_string(NULL, db, esc_name, name_list.name[i], strlen(name_list.name[i]));
+         /* Get all the Ids of each name */
+         db_escape_string(NULL, db,  esc_name, name_list.name[i], strlen(name_list.name[i]));
          bsnprintf(buf, sizeof(buf), "SELECT PathId FROM Path WHERE Path='%s'", esc_name);
          if (verbose > 1) {
             printf("%s\n", buf);
@@ -770,9 +763,7 @@ static void eliminate_duplicate_paths()
          if (verbose) {
             printf(_("Found %d for: %s\n"), id_list.num_ids, name_list.name[i]);
          }
-         /*
-          * Force all records to use the first id then delete the other ids
-          */
+         /* Force all records to use the first id then delete the other ids */
          for (int j=1; j<id_list.num_ids; j++) {
             char ed1[50], ed2[50];
             bsnprintf(buf, sizeof(buf), "UPDATE File SET PathId=%s WHERE PathId=%s",
@@ -802,9 +793,7 @@ static void eliminate_orphaned_jobmedia_records()
    if (!make_id_list(query, &id_list)) {
       exit(1);
    }
-   /*
-    * Loop doing 300000 at a time
-    */
+   /* Loop doing 300000 at a time */
    while (id_list.num_ids != 0) {
       printf(_("Found %d orphaned JobMedia records.\n"), id_list.num_ids);
       if (id_list.num_ids && verbose && yes_no(_("Print them? (yes/no): "))) {
@@ -812,7 +801,7 @@ static void eliminate_orphaned_jobmedia_records()
             char ed1[50];
             bsnprintf(buf, sizeof(buf),
 "SELECT JobMedia.JobMediaId,JobMedia.JobId,Media.VolumeName FROM JobMedia,Media "
-"WHERE JobMedia.JobMediaId=%s AND Media.MediaId=JobMedia.MediaId",
+        "WHERE JobMedia.JobMediaId=%s AND Media.MediaId=JobMedia.MediaId",
                edit_int64(id_list.Id[i], ed1));
             if (!db_sql_query(db, buf, print_jobmedia_handler, NULL)) {
                printf("%s\n", db_strerror(db));
@@ -848,9 +837,7 @@ static void eliminate_orphaned_file_records()
    if (!make_id_list(query, &id_list)) {
       exit(1);
    }
-   /*
-    * Loop doing 300000 at a time
-    */
+   /* Loop doing 300000 at a time */
    while (id_list.num_ids != 0) {
       printf(_("Found %d orphaned File records.\n"), id_list.num_ids);
       if (name_list.num_ids && verbose && yes_no(_("Print them? (yes/no): "))) {
@@ -858,7 +845,7 @@ static void eliminate_orphaned_file_records()
             char ed1[50];
             bsnprintf(buf, sizeof(buf),
 "SELECT File.FileId,File.JobId,Filename.Name FROM File,Filename "
-"WHERE File.FileId=%s AND File.FilenameId=Filename.FilenameId",
+   "WHERE File.FileId=%s AND File.FilenameId=Filename.FilenameId",
                edit_int64(id_list.Id[i], ed1));
             if (!db_sql_query(db, buf, print_file_handler, NULL)) {
                printf("%s\n", db_strerror(db));
@@ -893,14 +880,10 @@ static void eliminate_orphaned_path_records()
    }
 
    idx_tmp_name = NULL;
-   /*
-    * Check the existence of the required "one column" index
-    */
+   /* Check the existence of the required "one column" index */
    if (!check_idx("PathId"))  {
       if (yes_no(_("Create temporary index? (yes/no): "))) {
-         /*
-          * create temporary index PathId
-          */
+         /* create temporary index PathId */
          create_tmp_idx("idxPIchk", "File", "PathId");
       }
    }
@@ -916,15 +899,13 @@ static void eliminate_orphaned_path_records()
    if (!make_id_list(query, &id_list)) {
       exit(1);
    }
-   /*
-    * Loop doing 300000 at a time
-    */
+   /* Loop doing 300000 at a time */
    while (id_list.num_ids != 0) {
       printf(_("Found %d orphaned Path records.\n"), id_list.num_ids);
       if (id_list.num_ids && verbose && yes_no(_("Print them? (yes/no): "))) {
          for (int i=0; i < id_list.num_ids; i++) {
             char ed1[50];
-            bsnprintf(buf, sizeof(buf), "SELECT Path FROM Path WHERE PathId=%s",
+            bsnprintf(buf, sizeof(buf), "SELECT Path FROM Path WHERE PathId=%s", 
                edit_int64(id_list.Id[i], ed1));
             db_sql_query(db, buf, print_name_handler, NULL);
          }
@@ -941,24 +922,18 @@ static void eliminate_orphaned_path_records()
       if (!make_id_list(query, &id_list)) {
          exit(1);
       }
-   }
-   /*
-    * Drop temporary index idx_tmp_name
-    */
+   } 
+   /* Drop temporary index idx_tmp_name */
    drop_tmp_idx("idxPIchk", "File");
 }
 
 static void eliminate_orphaned_filename_records()
 {
    idx_tmp_name = NULL;
-   /*
-    * Check the existence of the required "one column" index
-    */
+   /* Check the existence of the required "one column" index */
    if (!check_idx("FilenameId") )      {
       if (yes_no(_("Create temporary index? (yes/no): "))) {
-         /*
-          * Create temporary index FilenameId
-          */
+         /* Create temporary index FilenameId */
          create_tmp_idx("idxFIchk", "File", "FilenameId");
       }
    }
@@ -974,15 +949,13 @@ static void eliminate_orphaned_filename_records()
    if (!make_id_list(query, &id_list)) {
       exit(1);
    }
-   /*
-    * Loop doing 300000 at a time
-    */
+   /* Loop doing 300000 at a time */
    while (id_list.num_ids != 0) {
       printf(_("Found %d orphaned Filename records.\n"), id_list.num_ids);
       if (id_list.num_ids && verbose && yes_no(_("Print them? (yes/no): "))) {
          for (int i=0; i < id_list.num_ids; i++) {
             char ed1[50];
-            bsnprintf(buf, sizeof(buf), "SELECT Name FROM Filename WHERE FilenameId=%s",
+            bsnprintf(buf, sizeof(buf), "SELECT Name FROM Filename WHERE FilenameId=%s", 
                edit_int64(id_list.Id[i], ed1));
             db_sql_query(db, buf, print_name_handler, NULL);
          }
@@ -1000,9 +973,7 @@ static void eliminate_orphaned_filename_records()
          exit(1);
       }
    }
-   /*
-    * Drop temporary index idx_tmp_name
-    */
+   /* Drop temporary index idx_tmp_name */
    drop_tmp_idx("idxFIchk", "File");
 
 }
@@ -1046,8 +1017,7 @@ static void eliminate_orphaned_client_records()
    const char *query;
 
    printf(_("Checking for orphaned Client entries.\n"));
-   /*
-    * In English:
+   /* In English:
     *   Wiffle through Client for every Client
     *   joining with the Job table including every Client even if
     *   there is not a match in Job (left outer join), then
@@ -1088,8 +1058,7 @@ static void eliminate_orphaned_job_records()
    const char *query;
 
    printf(_("Checking for orphaned Job entries.\n"));
-   /*
-    * In English:
+   /* In English:
     *   Wiffle through Job for every Job
     *   joining with the Client table including every Job even if
     *   there is not a match in Client (left outer join), then
@@ -1214,7 +1183,7 @@ static void repair_bad_filenames()
       for (i=0; i < id_list.num_ids; i++) {
          char ed1[50];
          bsnprintf(buf, sizeof(buf),
-            "SELECT Name FROM Filename WHERE FilenameId=%s",
+            "SELECT Name FROM Filename WHERE FilenameId=%s", 
                 edit_int64(id_list.Id[i], ed1));
          if (!db_sql_query(db, buf, print_name_handler, NULL)) {
             printf("%s\n", db_strerror(db));
@@ -1232,14 +1201,12 @@ static void repair_bad_filenames()
          int len;
          char ed1[50];
          bsnprintf(buf, sizeof(buf),
-            "SELECT Name FROM Filename WHERE FilenameId=%s",
+            "SELECT Name FROM Filename WHERE FilenameId=%s", 
                edit_int64(id_list.Id[i], ed1));
          if (!db_sql_query(db, buf, get_name_handler, name)) {
             printf("%s\n", db_strerror(db));
          }
-         /*
-          * Strip trailing slash(es)
-          */
+         /* Strip trailing slash(es) */
          for (len=strlen(name); len > 0 && IsPathSeparator(name[len-1]); len--)
             {  }
          if (len == 0) {
@@ -1258,7 +1225,7 @@ static void repair_bad_filenames()
          }
          db_sql_query(db, buf, NULL, NULL);
       }
-      free_pool_memory(name);
+      free_pool_memory(name); 
    }
 }
 
@@ -1302,17 +1269,13 @@ static void repair_bad_paths()
          if (!db_sql_query(db, buf, get_name_handler, name)) {
             printf("%s\n", db_strerror(db));
          }
-         /*
-          * Strip trailing blanks
-          */
+         /* Strip trailing blanks */
          for (len=strlen(name); len > 0 && name[len-1]==' '; len--) {
             name[len-1] = 0;
          }
-         /*
-          * Add trailing slash
-          */
+         /* Add trailing slash */
          len = pm_strcat(&name, "/");
-         db_escape_string(NULL, db, esc_name, name, len);
+         db_escape_string(NULL, db,  esc_name, name, len);
          bsnprintf(buf, sizeof(buf), "UPDATE Path SET Path='%s' WHERE PathId=%s",
             esc_name, edit_int64(id_list.Id[i], ed1));
          if (verbose > 1) {
@@ -1320,7 +1283,7 @@ static void repair_bad_paths()
          }
          db_sql_query(db, buf, NULL, NULL);
       }
-      free_pool_memory(name);
+      free_pool_memory(name); 
    }
 }
 
@@ -1368,19 +1331,19 @@ typedef struct s_idx_list {
 
 static IDX_LIST idx_list[MAXIDX];
 
-/*
+/* 
  * Called here with each table index to be added to the list
  */
 static int check_idx_handler(void *ctx, int num_fields, char **row)
 {
-   /*
-    * Table | Non_unique | Key_name | Seq_in_index | Column_name |...
-    * File  |          0 | PRIMARY  |            1 | FileId      |...
-    */
+   /* 
+    * Table | Non_unique | Key_name | Seq_in_index | Column_name |... 
+    * File  |          0 | PRIMARY  |            1 | FileId      |... 
+    */ 
    char *name, *key_name, *col_name;
    int i, len;
    int found = false;
-
+ 
    name = (char *)ctx;
    key_name = row[2];
    col_name = row[4];
@@ -1394,9 +1357,7 @@ static int check_idx_handler(void *ctx, int num_fields, char **row)
          break;
       }
    }
-   /*
-    * If the new Key_name, add it to the list
-    */
+   /* If the new Key_name, add it to the list */
    if (!found) {
       len = strlen(key_name) + 1;
       idx_list[i].key_name = (char *)malloc(len);
@@ -1420,45 +1381,42 @@ static bool check_idx(const char *col_name)
    int found = false;
    const char *query = "SHOW INDEX FROM File";
 
-   switch (db_get_type_index(db)) {
-   case SQL_TYPE_MYSQL:
-      memset(&idx_list, 0, sizeof(idx_list));
-      if (!db_sql_query(db, query, check_idx_handler, (void *)col_name)) {
-         printf("%s\n", db_strerror(db));
-      }
-      for (i = 0; (idx_list[i].key_name != NULL) && (i < MAXIDX) ; i++) {
-         /*
-          * NOTE : if (idx_list[i].count_key > 1) then index idx_list[i].key_name is "multiple-column" index
-          */
-         if ((idx_list[i].count_key == 1) && (idx_list[i].count_col == 1)) {
-            /*
-             * "one column" index over *col_name found
-             */
-            found = true;
-         }
-      }
-      if (found) {
-         if (verbose) {
-            printf(_("Ok. Index over the %s column already exists and dbcheck will work faster.\n"), col_name);
-         }
-      } else {
-         printf(_("Note. Index over the %s column not found, that can greatly slow down dbcheck.\n"), col_name);
-      }
-      return found;
-   default:
+   if (db_get_type_index(db) != SQL_TYPE_MYSQL) {
       return true;
    }
+   /* Continue for MySQL */
+   memset(&idx_list, 0, sizeof(idx_list));
+   if (!db_sql_query(db, query, check_idx_handler, (void *)col_name)) {
+      printf("%s\n", db_strerror(db));
+   }
+   for (i = 0; (idx_list[i].key_name != NULL) && (i < MAXIDX) ; i++) {
+      /*
+       * NOTE : if (idx_list[i].count_key > 1) then index idx_list[i].key_name is "multiple-column" index
+       */
+      if ((idx_list[i].count_key == 1) && (idx_list[i].count_col == 1)) {
+         /* "one column" index over *col_name found */
+         found = true;
+      }
+   }
+   if (found) {
+      if (verbose) {
+         printf(_("Ok. Index over the %s column already exists and dbcheck will work faster.\n"), col_name);
+      }
+   } else {
+      printf(_("Note. Index over the %s column not found, that can greatly slow down dbcheck.\n"), col_name);
+   }
+   return found;
 }
 
 /*
  * Create temporary one-column index
  */
-static bool create_tmp_idx(const char *idx_name, const char *table_name,
-                           const char *col_name)
+static bool create_tmp_idx(const char *idx_name, const char *table_name, 
+                           const char *col_name) 
 {
    idx_tmp_name = NULL;
    printf(_("Create temporary index... This may take some time!\n"));
-   bsnprintf(buf, sizeof(buf), "CREATE INDEX %s ON %s (%s)", idx_name, table_name, col_name);
+   bsnprintf(buf, sizeof(buf), "CREATE INDEX %s ON %s (%s)", idx_name, table_name, col_name); 
    if (verbose) {
       printf("%s\n", buf);
    }
